@@ -67,6 +67,7 @@ public class CalledMethodsAnnotatedTypeFactory extends AccumulationAnnotatedType
    * The {@link java.util.Collections#singletonList} method. It is treated specially by {@link
    * #adjustMethodNameUsingValueChecker}.
    */
+  @SuppressWarnings("this-escape")
   private final ExecutableElement collectionsSingletonList =
       TreeUtils.getMethod("java.util.Collections", "singletonList", 1, getProcessingEnv());
 
@@ -95,6 +96,7 @@ public class CalledMethodsAnnotatedTypeFactory extends AccumulationAnnotatedType
    *
    * @param checker the checker
    */
+  @SuppressWarnings("this-escape")
   public CalledMethodsAnnotatedTypeFactory(BaseTypeChecker checker) {
     super(checker, CalledMethods.class, CalledMethodsBottom.class, CalledMethodsPredicate.class);
 
@@ -239,7 +241,7 @@ public class CalledMethodsAnnotatedTypeFactory extends AccumulationAnnotatedType
   // This cannot return a Name because filterKindToMethodName cannot.
   private @Nullable String filterTreeToMethodName(
       Tree filterTree, ValueAnnotatedTypeFactory valueATF) {
-    while (filterTree != null && filterTree.getKind() == Tree.Kind.METHOD_INVOCATION) {
+    while (filterTree != null && filterTree instanceof MethodInvocationTree) {
 
       MethodInvocationTree filterTreeAsMethodInvocation = (MethodInvocationTree) filterTree;
       String filterMethodName = TreeUtils.methodName(filterTreeAsMethodInvocation).toString();
@@ -258,7 +260,7 @@ public class CalledMethodsAnnotatedTypeFactory extends AccumulationAnnotatedType
     if (filterTree == null) {
       return null;
     }
-    if (filterTree.getKind() == Tree.Kind.NEW_CLASS) {
+    if (filterTree instanceof NewClassTree) {
       ExpressionTree constructorArg = ((NewClassTree) filterTree).getArguments().get(0);
       String filterKindName = ValueCheckerUtils.getExactStringValue(constructorArg, valueATF);
       if (filterKindName != null) {
@@ -293,7 +295,7 @@ public class CalledMethodsAnnotatedTypeFactory extends AccumulationAnnotatedType
    * At a fluent method call (which returns {@code this}), add the method to the type of the return
    * value.
    */
-  private class CalledMethodsTreeAnnotator extends AccumulationTreeAnnotator {
+  private class CalledMethodsTreeAnnotator extends TreeAnnotator {
     /**
      * Creates an instance of this tree annotator for the given type factory.
      *
