@@ -41,13 +41,16 @@ import org.checkerframework.javacutil.TypesUtils;
 public class FormatterTreeUtil {
   /** The checker. */
   public final BaseTypeChecker checker;
+
   /** The processing environment. */
   public final ProcessingEnvironment processingEnv;
 
   /** The value() element/field of an @Format annotation. */
   protected final ExecutableElement formatValueElement;
+
   /** The value() element/field of an @InvalidFormat annotation. */
   protected final ExecutableElement invalidFormatValueElement;
+
   // private final ExecutableElement formatArgTypesElement;
 
   public FormatterTreeUtil(BaseTypeChecker checker) {
@@ -108,9 +111,16 @@ public class FormatterTreeUtil {
     NULLARRAY;
   }
 
-  /** A wrapper around a value of type E, plus an ExpressionTree location. */
+  /**
+   * A wrapper around a value of type E, plus an ExpressionTree location.
+   *
+   * @param <E> the type of the wrapped value
+   */
   public static class Result<E> {
+    /** The wrapped value. */
     private final E value;
+
+    /** The location of the value. */
     public final ExpressionTree location;
 
     public Result(E value, ExpressionTree location) {
@@ -133,7 +143,8 @@ public class FormatterTreeUtil {
     return anno != null;
   }
 
-  private ConversionCategory[] asFormatCallCategoriesLowLevel(MethodInvocationNode node) {
+  private ConversionCategory @Nullable [] asFormatCallCategoriesLowLevel(
+      MethodInvocationNode node) {
     Node vararg = node.getArgument(1);
     if (!(vararg instanceof ArrayCreationNode)) {
       return null;
@@ -161,14 +172,14 @@ public class FormatterTreeUtil {
   }
 
   /**
-   * Returns true if {@code node} is a call to a method annotated with {@code @FormatMethod}.
+   * Returns true if {@code tree} is a call to a method annotated with {@code @FormatMethod}.
    *
-   * @param node a method call
+   * @param tree a method call
    * @param atypeFactory a type factory
-   * @return true if {@code node} is a call to a method annotated with {@code @FormatMethod}
+   * @return true if {@code tree} is a call to a method annotated with {@code @FormatMethod}
    */
-  public boolean isFormatMethodCall(MethodInvocationTree node, AnnotatedTypeFactory atypeFactory) {
-    ExecutableElement method = TreeUtils.elementFromUse(node);
+  public boolean isFormatMethodCall(MethodInvocationTree tree, AnnotatedTypeFactory atypeFactory) {
+    ExecutableElement method = TreeUtils.elementFromUse(tree);
     AnnotationMirror anno = atypeFactory.getDeclAnnotation(method, FormatMethod.class);
     return anno != null;
   }
@@ -191,8 +202,9 @@ public class FormatterTreeUtil {
     ExecutableElement methodElement = TreeUtils.elementFromUse(invocationTree);
     int formatStringIndex = FormatterVisitor.formatStringIndex(methodElement);
     if (formatStringIndex == -1) {
-      // Reporting the error is redundant if the method was declared in source code, because the
-      // visitor will have reported it; but it is necessary if the method was declared in byte code.
+      // Reporting the error is redundant if the method was declared in source code, because
+      // the visitor will have reported it; but it is necessary if the method was declared in
+      // byte code.
       atypeFactory
           .getChecker()
           .reportError(invocationTree, "format.method", methodElement.getSimpleName());
@@ -209,13 +221,17 @@ public class FormatterTreeUtil {
   /** Represents a format method invocation in the syntax tree. */
   public class FormatCall {
     /** The call itself. */
-    final MethodInvocationTree invocationTree;
+    /*package-private*/ final MethodInvocationTree invocationTree;
+
     /** The format string argument. */
     private final ExpressionTree formatStringTree;
+
     /** The type of the format string argument. */
     private final AnnotatedTypeMirror formatStringType;
+
     /** The arguments that follow the format string argument. */
     private final List<? extends ExpressionTree> args;
+
     /** The type factory. */
     private final AnnotatedTypeFactory atypeFactory;
 
@@ -248,10 +264,10 @@ public class FormatterTreeUtil {
      * @return an error description if the format string is not annotated as {@code @Format}, or
      *     null if it is
      */
-    public final Result<String> errMissingFormatAnnotation() {
-      if (!formatStringType.hasAnnotation(Format.class)) {
+    public final @Nullable Result<String> errMissingFormatAnnotation() {
+      if (!formatStringType.hasPrimaryAnnotation(Format.class)) {
         String msg = "(is a @Format annotation missing?)";
-        AnnotationMirror inv = formatStringType.getAnnotation(InvalidFormat.class);
+        AnnotationMirror inv = formatStringType.getPrimaryAnnotation(InvalidFormat.class);
         if (inv != null) {
           msg = invalidFormatAnnotationToErrorMessage(inv);
         }
@@ -269,7 +285,7 @@ public class FormatterTreeUtil {
       InvocationType type = InvocationType.VARARG;
 
       if (args.size() == 1) {
-        final ExpressionTree first = args.get(0);
+        ExpressionTree first = args.get(0);
         TypeMirror argType = atypeFactory.getAnnotatedType(first).getUnderlyingType();
         // figure out if argType is an array
         type =
@@ -288,16 +304,16 @@ public class FormatterTreeUtil {
                     return first.accept(
                         new SimpleTreeVisitor<InvocationType, Class<Void>>() {
                           @Override
-                          protected InvocationType defaultAction(Tree node, Class<Void> p) {
+                          protected InvocationType defaultAction(Tree tree, Class<Void> p) {
                             // just a normal array
                             return InvocationType.ARRAY;
                           }
 
                           @Override
-                          public InvocationType visitTypeCast(TypeCastTree node, Class<Void> p) {
+                          public InvocationType visitTypeCast(TypeCastTree tree, Class<Void> p) {
                             // it's a (Object[])null
                             return atypeFactory
-                                        .getAnnotatedType(node.getExpression())
+                                        .getAnnotatedType(tree.getExpression())
                                         .getUnderlyingType()
                                         .getKind()
                                     == TypeKind.NULL
@@ -330,7 +346,7 @@ public class FormatterTreeUtil {
      * @see ConversionCategory
      */
     public final ConversionCategory[] getFormatCategories() {
-      AnnotationMirror anno = formatStringType.getAnnotation(Format.class);
+      AnnotationMirror anno = formatStringType.getPrimaryAnnotation(Format.class);
       return formatAnnotationToCategories(anno);
     }
 
@@ -440,8 +456,7 @@ public class FormatterTreeUtil {
    * @param invalidFormatString an invalid formatter string
    * @return an {@link InvalidFormat} annotation with the given string as its value
    */
-  // package-private
-  AnnotationMirror stringToInvalidFormatAnnotation(String invalidFormatString) {
+  /*package-private*/ AnnotationMirror stringToInvalidFormatAnnotation(String invalidFormatString) {
     AnnotationBuilder builder = new AnnotationBuilder(processingEnv, InvalidFormat.class);
     builder.setValue("value", invalidFormatString);
     return builder.build();
