@@ -86,9 +86,11 @@ public class SameLenAnnotatedTypeFactory extends BaseAnnotatedTypeFactory {
       TreeUtils.getMethod(SameLen.class, "value", 0, processingEnv);
 
   /** Predicates about method calls. */
+  @SuppressWarnings("this-escape")
   private final IndexMethodIdentifier imf = new IndexMethodIdentifier(this);
 
   /** Create a new SameLenAnnotatedTypeFactory. */
+  @SuppressWarnings("this-escape")
   public SameLenAnnotatedTypeFactory(BaseTypeChecker checker) {
     super(checker);
 
@@ -119,7 +121,7 @@ public class SameLenAnnotatedTypeFactory extends BaseAnnotatedTypeFactory {
   public AnnotatedTypeMirror getAnnotatedTypeLhs(Tree tree) {
     AnnotatedTypeMirror atm = super.getAnnotatedTypeLhs(tree);
 
-    if (tree.getKind() == Tree.Kind.VARIABLE) {
+    if (tree instanceof VariableTree) {
       AnnotationMirror sameLenAnno = atm.getPrimaryAnnotation(SameLen.class);
       if (sameLenAnno != null) {
         JavaExpression je = JavaExpression.fromVariableTree((VariableTree) tree);
