@@ -12,6 +12,7 @@ import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewClassTree;
+import com.sun.source.tree.ParenthesizedTree;
 import com.sun.source.tree.ReturnTree;
 import com.sun.source.tree.Scope;
 import com.sun.source.tree.StatementTree;
@@ -310,15 +311,14 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     List<? extends StatementTree> bodyStatements = body.getStatements();
     if (bodyStatements.size() == 1) {
       StatementTree bodyStatement = bodyStatements.get(0);
-      if (bodyStatement.getKind() == Tree.Kind.RETURN) {
+      if (bodyStatement instanceof ReturnTree) {
         ExpressionTree returnExpr =
             TreeUtils.withoutParens(((ReturnTree) bodyStatement).getExpression());
         if (returnExpr.getKind() == Tree.Kind.EQUAL_TO) {
           BinaryTree bt = (BinaryTree) returnExpr;
           ExpressionTree lhsTree = bt.getLeftOperand();
           ExpressionTree rhsTree = bt.getRightOperand();
-          if (lhsTree.getKind() == Tree.Kind.IDENTIFIER
-              && rhsTree.getKind() == Tree.Kind.IDENTIFIER) {
+          if (lhsTree instanceof IdentifierTree && rhsTree instanceof IdentifierTree) {
             Name leftName = ((IdentifierTree) lhsTree).getName();
             Name rightName = ((IdentifierTree) rhsTree).getName();
             Name paramName = equalsMethod.getParameters().get(0).getName();
@@ -347,9 +347,9 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
   public boolean validateTypeOf(Tree tree) {
     // Don't check the result type of a constructor, because it must be @UnknownInterned, even
     // if the type on the class declaration is @Interned.
-    if (tree.getKind() == Tree.Kind.METHOD && TreeUtils.isConstructor((MethodTree) tree)) {
+    if (tree instanceof MethodTree && TreeUtils.isConstructor((MethodTree) tree)) {
       return true;
-    } else if (tree.getKind() == Tree.Kind.NEW_CLASS) {
+    } else if (tree instanceof NewClassTree) {
       NewClassTree newClassTree = (NewClassTree) tree;
       TypeMirror typeMirror = TreeUtils.typeOf(newClassTree);
       AnnotationMirrorSet bounds = atypeFactory.getTypeDeclarationBounds(typeMirror);
@@ -381,12 +381,12 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     TreePath path = getCurrentPath();
     if (path != null) {
       TreePath parentPath = path.getParentPath();
-      while (parentPath != null && parentPath.getLeaf().getKind() == Tree.Kind.PARENTHESIZED) {
+      while (parentPath != null && parentPath.getLeaf() instanceof ParenthesizedTree) {
         parentPath = parentPath.getParentPath();
       }
       if (parentPath != null && parentPath.getParentPath() != null) {
         Tree parent = parentPath.getParentPath().getLeaf();
-        if (parent.getKind() == Tree.Kind.METHOD_INVOCATION) {
+        if (parent instanceof MethodInvocationTree) {
           // Allow new MyInternType().intern(), where "intern" is any method marked
           // @InternMethod.
           ExecutableElement elt = TreeUtils.elementFromUse((MethodInvocationTree) parent);
@@ -474,7 +474,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     ExpressionTree right = binaryTree.getRightOperand();
 
     // Only valid if we're comparing identifiers.
-    if (!(left.getKind() == Tree.Kind.IDENTIFIER && right.getKind() == Tree.Kind.IDENTIFIER)) {
+    if (!(left instanceof IdentifierTree && right instanceof IdentifierTree)) {
       return false;
     }
 
@@ -484,9 +484,9 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
 
     // Ensure the == is in a return or in an if, and that enclosing statement is the first
     // statement in the method.
-    if (parent.getKind() == Tree.Kind.RETURN) {
+    if (parent instanceof ReturnTree) {
       // ensure the return statement is the first statement in the method
-      if (parentPath.getParentPath().getParentPath().getLeaf().getKind() != Tree.Kind.METHOD) {
+      if (!(parentPath.getParentPath().getParentPath().getLeaf() instanceof MethodTree)) {
         return false;
       }
 
@@ -502,9 +502,9 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
         TreePath ppath = parentPath;
         Tree candidateTree;
         while ((candidateTree = ppath.getLeaf()) != null) {
-          if (candidateTree.getKind() == Tree.Kind.IF) {
+          if (candidateTree instanceof IfTree) {
             ifStatementTree = candidateTree;
-          } else if (candidateTree.getKind() == Tree.Kind.METHOD) {
+          } else if (candidateTree instanceof MethodTree) {
             methodTree = (MethodTree) candidateTree;
             break;
           }
@@ -716,7 +716,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
             // Element argElt = TreeUtils.elementFromUse((IdentifierTree) arg);
 
             ExpressionTree exp = tree.getMethodSelect();
-            if (exp.getKind() != Tree.Kind.MEMBER_SELECT) {
+            if (!(exp instanceof MemberSelectTree)) {
               return false;
             }
             MemberSelectTree member = (MemberSelectTree) exp;
@@ -762,7 +762,7 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
     ExpressionTree right = TreeUtils.withoutParens(topBinaryTree.getRightOperand());
 
     // Only valid if we're comparing identifiers.
-    if (!(left.getKind() == Tree.Kind.IDENTIFIER && right.getKind() == Tree.Kind.IDENTIFIER)) {
+    if (!(left instanceof IdentifierTree && right instanceof IdentifierTree)) {
       return false;
     }
 
@@ -818,17 +818,17 @@ public final class InterningVisitor extends BaseTypeVisitor<InterningAnnotatedTy
               return false;
             }
             ExpressionTree arg = args.get(0);
-            if (arg.getKind() != Tree.Kind.IDENTIFIER) {
+            if (!(arg instanceof IdentifierTree)) {
               return false;
             }
             Element argElt = TreeUtils.elementFromUse(arg);
 
             ExpressionTree exp = tree.getMethodSelect();
-            if (exp.getKind() != Tree.Kind.MEMBER_SELECT) {
+            if (!(exp instanceof MemberSelectTree)) {
               return false;
             }
             MemberSelectTree member = (MemberSelectTree) exp;
-            if (member.getExpression().getKind() != Tree.Kind.IDENTIFIER) {
+            if (!(member.getExpression() instanceof IdentifierTree)) {
               return false;
             }
 
