@@ -24,6 +24,7 @@ import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewArrayTree;
 import com.sun.source.tree.NewClassTree;
 import com.sun.source.tree.ParameterizedTypeTree;
+import com.sun.source.tree.PrimitiveTypeTree;
 import com.sun.source.tree.SwitchTree;
 import com.sun.source.tree.SynchronizedTree;
 import com.sun.source.tree.ThrowTree;
@@ -187,11 +188,11 @@ public class NullnessVisitor
         // constructor.
         // Note that this method should return non-null only for fields of this class, not
         // fields of any other class, including outer classes.
-        if (receiver.getKind() != Tree.Kind.IDENTIFIER
+        if (!(receiver instanceof IdentifierTree)
             || !((IdentifierTree) receiver).getName().contentEquals("this")) {
           return null;
         }
-        // fallthrough
+      // fallthrough
       case IDENTIFIER:
         TreePath path = getCurrentPath();
         if (TreePathUtil.inConstructor(path)) {
@@ -249,7 +250,7 @@ public class NullnessVisitor
         checker.reportError(tree, "nullness.on.outer");
       }
     } else if (!(TreeUtils.isSelfAccess(tree)
-        || tree.getExpression().getKind() == Tree.Kind.PARAMETERIZED_TYPE
+        || tree.getExpression() instanceof ParameterizedTypeTree
         // case 8. static member access
         || ElementUtils.isStatic(e))) {
       checkForNullability(tree.getExpression(), DEREFERENCE_OF_NULLABLE);
@@ -415,7 +416,7 @@ public class NullnessVisitor
     // The "reference type" is the type after "instanceof".
     Tree refTypeTree = tree.getType();
     if (refTypeTree != null) {
-      if (refTypeTree.getKind() == Tree.Kind.ANNOTATED_TYPE) {
+      if (refTypeTree instanceof AnnotatedTypeTree) {
         List<? extends AnnotationMirror> annotations =
             TreeUtils.annotationsFromTree((AnnotatedTypeTree) refTypeTree);
         if (AnnotationUtils.containsSame(annotations, NULLABLE)) {
@@ -576,7 +577,7 @@ public class NullnessVisitor
 
     if (classTree.getKind() == Tree.Kind.ENUM) {
       for (Tree member : classTree.getMembers()) {
-        if (member.getKind() == Tree.Kind.VARIABLE
+        if (member instanceof VariableTree
             && TreeUtils.elementFromDeclaration((VariableTree) member).getKind()
                 == ElementKind.ENUM_CONSTANT) {
           VariableTree varDecl = (VariableTree) member;
@@ -598,7 +599,7 @@ public class NullnessVisitor
    * @param typeTree a supertype tree, from an {@code extends} or {@code implements} clause
    */
   private void reportErrorIfSupertypeContainsNullnessAnnotation(Tree typeTree) {
-    if (typeTree.getKind() == Tree.Kind.ANNOTATED_TYPE) {
+    if (typeTree instanceof AnnotatedTypeTree) {
       List<? extends AnnotationTree> annoTrees = ((AnnotatedTypeTree) typeTree).getAnnotations();
       if (atypeFactory.containsNullnessAnnotation(annoTrees)) {
         checker.reportError(typeTree, "nullness.on.supertype");
@@ -817,7 +818,7 @@ public class NullnessVisitor
         case ANNOTATED_TYPE:
           AnnotatedTypeTree at = ((AnnotatedTypeTree) t);
           Tree underlying = at.getUnderlyingType();
-          if (underlying.getKind() == Tree.Kind.PRIMITIVE_TYPE) {
+          if (underlying instanceof PrimitiveTypeTree) {
             if (atypeFactory.containsNullnessAnnotation(null, at)) {
               checker.reportError(t, "nullness.on.primitive");
             }
