@@ -18,10 +18,8 @@ public class StubparserTaintingTest extends CheckerFrameworkPerDirectoryTest {
         testFiles,
         org.checkerframework.checker.tainting.TaintingChecker.class,
         "stubparser-tainting",
-        "-Anomsgtext",
         "-AmergeStubsWithSource",
-        "-Astubs=tests/stubparser-tainting",
-        "-AstubWarnIfNotFound");
+        "-Astubs=tests/stubparser-tainting");
   }
 
   @Parameterized.Parameters
