@@ -5,7 +5,7 @@ import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.tree.TreeInfo;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
-import org.checkerframework.checker.testchecker.ainfer.qual.DefaultType;
+import org.checkerframework.checker.testchecker.ainfer.qual.AinferDefaultType;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.common.basetype.BaseTypeVisitor;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutableType;
@@ -23,12 +23,12 @@ public class AinferTestVisitor extends BaseTypeVisitor<AinferTestAnnotatedTypeFa
   }
 
   @Override
-  public Void visitAnnotation(AnnotationTree node, Void p) {
-    Element anno = TreeInfo.symbol((JCTree) node.getAnnotationType());
-    if (anno.toString().equals(DefaultType.class.getName())) {
-      checker.reportError(node, "annotation.not.allowed.in.src", anno.toString());
+  public Void visitAnnotation(AnnotationTree tree, Void p) {
+    Element anno = TreeInfo.symbol((JCTree) tree.getAnnotationType());
+    if (anno.toString().equals(AinferDefaultType.class.getName())) {
+      checker.reportError(tree, "annotation.not.allowed.in.src", anno.toString());
     }
-    return super.visitAnnotation(node, p);
+    return super.visitAnnotation(tree, p);
   }
 
   @Override
