@@ -1,11 +1,11 @@
-import org.checkerframework.checker.testchecker.ainfer.qual.Top;
+import org.checkerframework.checker.testchecker.ainfer.qual.AinferTop;
 
 public class ConstructorTest {
 
   public ConstructorTest(int top) {}
 
   void test() {
-    @Top int top = (@Top int) 0;
+    @AinferTop int top = (@AinferTop int) 0;
     // :: warning: (argument)
     new ConstructorTest(top);
   }
