@@ -1,9 +1,9 @@
 // test file for https://github.com/typetools/checker-framework/issues/3438
 
-import org.checkerframework.checker.testchecker.ainfer.qual.Sibling1;
+import org.checkerframework.checker.testchecker.ainfer.qual.AinferSibling1;
 
 public class OuterClassWithTypeParam<T> {
   public class InnerClass {
-    Object o = (@Sibling1 Object) null;
+    Object o = (@AinferSibling1 Object) null;
   }
 }
