@@ -7,16 +7,17 @@ import java.net.*;
 import org.checkerframework.checker.calledmethods.qual.*;
 import org.checkerframework.checker.mustcall.qual.*;
 
-@MustCall("close") class SocketContainer {
+@InheritableMustCall("close")
+class SocketContainer {
   @Owning Socket sock;
 
   public SocketContainer(String host, int port) throws Exception {
     // It should be okay to assign to uninitialized owning fields in the constructor.
     // But it isn't! Why?
-    // :: error: required.method.not.called
+    // :: error: (required.method.not.called)
     sock = new Socket(host, port);
     // It's definitely not okay to do it twice!
-    // :: error: required.method.not.called
+    // :: error: (required.method.not.called)
     sock = new Socket(host, port);
   }
 
