@@ -31,6 +31,9 @@ import org.checkerframework.javacutil.TreeUtils;
  * <p>String concatenation compound assignments are desugared to an assignment and a string
  * concatenation.
  *
+ * <p>Assignments desugared from an enhanced-for-loop over an array are marked as such for special
+ * casing.
+ *
  * <p>Numeric compound assignments are desugared to an assignment and a numeric operation.
  */
 public class AssignmentNode extends Node {
@@ -47,6 +50,9 @@ public class AssignmentNode extends Node {
   /** Whether the assignment node is synthetic */
   protected final boolean synthetic;
 
+  /** Whether the assignment node is desugared from an enhanced-for-loop over an array. */
+  protected boolean desugaredFromEnhancedArrayForLoop;
+
   /**
    * Create a (non-synthetic) AssignmentNode.
    *
@@ -79,6 +85,7 @@ public class AssignmentNode extends Node {
     this.lhs = target;
     this.rhs = expression;
     this.synthetic = synthetic;
+    this.desugaredFromEnhancedArrayForLoop = false;
   }
 
   /**
@@ -117,6 +124,20 @@ public class AssignmentNode extends Node {
     return synthetic;
   }
 
+  /**
+   * Check if the assignment node is desugared from an enhanced-for-loop over an array.
+   *
+   * @return true if the assignment node is desugared
+   */
+  public boolean isDesugaredFromEnhancedArrayForLoop() {
+    return desugaredFromEnhancedArrayForLoop;
+  }
+
+  /** set the assignment node as desugared from an enhanced-for-loop over an array */
+  public void setDesugaredFromEnhancedArrayForLoop() {
+    desugaredFromEnhancedArrayForLoop = true;
+  }
+
   @Override
   public <R, P> R accept(NodeVisitor<R, P> visitor, P p) {
     return visitor.visitAssignment(this, p);
