@@ -14,9 +14,11 @@ import os
 from os.path import expanduser
 
 from release_vars import AFU_LIVE_RELEASES_DIR
+from release_vars import ANNO_FILE_UTILITIES
 from release_vars import CF_VERSION
 from release_vars import CHECKER_FRAMEWORK
 from release_vars import CHECKER_LIVE_RELEASES_DIR
+from release_vars import CHECKER_LIVE_API_DIR
 from release_vars import CHECKLINK
 from release_vars import DEV_SITE_DIR
 from release_vars import DEV_SITE_URL
@@ -35,13 +37,14 @@ from release_utils import continue_or_exit
 from release_utils import current_distribution_by_website
 from release_utils import delete_if_exists
 from release_utils import delete_path
+from release_utils import delete_path_if_exists
 from release_utils import ensure_group_access
 from release_utils import get_announcement_email
 from release_utils import print_step
 from release_utils import prompt_to_continue
 from release_utils import prompt_yes_no
 from release_utils import push_changes_prompt_if_fail
-from release_utils import read_command_line_option
+from release_utils import has_command_line_option
 from release_utils import read_first_line
 from release_utils import set_umask
 from release_utils import subprocess
@@ -84,7 +87,7 @@ def copy_release_dir(path_to_dev_releases, path_to_live_releases, release_versio
     # The / at the end of the source location is necessary so that
     # rsync copies the files in the source directory to the destination directory
     # rather than a subdirectory of the destination directory.
-    cmd = "rsync --omit-dir-times --recursive --links --quiet %s/ %s" % (
+    cmd = "rsync --no-group --omit-dir-times --recursive --links --quiet %s/ %s" % (
         source_location,
         dest_location,
     )
@@ -100,7 +103,7 @@ def promote_release(path_to_releases, release_version):
     from_dir = os.path.join(path_to_releases, release_version)
     to_dir = os.path.join(path_to_releases, "..")
     # Trailing slash is crucial.
-    cmd = "rsync -aJ --omit-dir-times %s/ %s" % (from_dir, to_dir)
+    cmd = "rsync -aJ --no-group --omit-dir-times %s/ %s" % (from_dir, to_dir)
     execute(cmd)
 
 
@@ -118,6 +121,7 @@ def copy_releases_to_live_site(cf_version):
     Framework from the dev site to the live site."""
     CHECKER_INTERM_RELEASES_DIR = os.path.join(DEV_SITE_DIR, "releases")
     copy_release_dir(CHECKER_INTERM_RELEASES_DIR, CHECKER_LIVE_RELEASES_DIR, cf_version)
+    delete_path_if_exists(CHECKER_LIVE_API_DIR)
     promote_release(CHECKER_LIVE_RELEASES_DIR, cf_version)
     AFU_INTERM_RELEASES_DIR = os.path.join(
         DEV_SITE_DIR, "annotation-file-utilities", "releases"
@@ -144,9 +148,8 @@ def stage_maven_artifacts_in_maven_central(new_cf_version):
     gnupgPassphrase = read_first_line(
         "/projects/swlab1/checker-framework/hosting-info/release-private.password"
     )
-    # When bufalo uses gpg2 version 2.2+, then remove signing.gnupg.useLegacyGpg=true
     execute(
-        "./gradlew publish -Prelease=true --no-parallel -Psigning.gnupg.useLegacyGpg=true -Psigning.gnupg.keyName=checker-framework-dev@googlegroups.com -Psigning.gnupg.passphrase=%s"
+        "./gradlew publish -Prelease=true --no-parallel -Psigning.gnupg.keyName=checker-framework-dev@googlegroups.com -Psigning.gnupg.passphrase=%s"
         % gnupgPassphrase,
         working_dir=CHECKER_FRAMEWORK,
     )
@@ -185,10 +188,14 @@ def run_link_checker(site, output, additional_param=""):
     out_file.close()
 
     if process.returncode != 0:
-        raise Exception(
-            "Non-zero return code (%s; see output in %s) while executing %s"
-            % (process.returncode, output, cmd)
+        msg = "Non-zero return code (%s; see output in %s) while executing %s" % (
+            process.returncode,
+            output,
+            cmd,
         )
+        print(msg + "\n")
+        if not prompt_yes_no("Continue despite link checker results?", True):
+            raise Exception(msg)
 
     return output
 
@@ -233,16 +240,19 @@ def check_all_links(
     if not is_checkerCheck_empty:
         print("\t" + checkerCheck + "\n")
     if errors_reported:
-        release_option = ""
-        if not test_mode:
-            release_option = " release"
-        raise Exception(
-            "The link checker reported errors.  Please fix them by committing changes to the mainline\n"
-            + 'repository and pushing them to GitHub, running "python release_build.py all" again\n'
-            + '(in order to update the development site), and running "python release_push'
-            + release_option
-            + '" again.'
-        )
+        if not prompt_yes_no("Continue despite link checker results?", True):
+            release_option = ""
+            if not test_mode:
+                release_option = " release"
+            raise Exception(
+                "The link checker reported errors.  Please fix them by committing changes to the mainline\n"
+                + "repository and pushing them to GitHub, then updating the development and live sites by\n"
+                + "running\n"
+                + "  python3 release_build.py all\n"
+                + "  python3 release_push"
+                + release_option
+                + "\n"
+            )
 
 
 def push_interm_to_release_repos():
@@ -292,7 +302,7 @@ def main(argv):
     set_umask()
 
     validate_args(argv)
-    test_mode = not read_command_line_option(argv, "release")
+    test_mode = not has_command_line_option(argv, "release")
 
     m2_settings = expanduser("~") + "/.m2/settings.xml"
     if not os.path.exists(m2_settings):
@@ -362,7 +372,6 @@ def main(argv):
 
     print_step("Push Step 3: Run development sanity tests")  # SEMIAUTO
     if prompt_yes_no("Perform this step?", True):
-
         print_step("3a: Run javac sanity test on development release.")
         if prompt_yes_no("Run javac sanity test on development release?", True):
             javac_sanity_check(dev_checker_website, new_cf_version)
@@ -378,6 +387,9 @@ def main(argv):
         ant_cmd = "./gradlew allTests"
         execute(ant_cmd, True, False, CHECKER_FRAMEWORK)
 
+        ant_cmd = "./gradlew test"
+        execute(ant_cmd, True, False, ANNO_FILE_UTILITIES)
+
     # The Central repository is a repository of build artifacts for build programs like Maven and Ivy.
     # This step stages (but doesn't release) the Checker Framework's Maven artifacts in the Sonatypes
     # Central Repository.
@@ -394,19 +406,19 @@ def main(argv):
 
     print_step("Push Step 5: Stage Maven artifacts in Central")  # SEMIAUTO
 
-    print_step("5a: Stage the artifacts at Maven central.")
+    print_step("Step 5a: Stage the artifacts at Maven central.")
     if (not test_mode) or prompt_yes_no(
         "Stage Maven artifacts in Maven Central?", not test_mode
     ):
         stage_maven_artifacts_in_maven_central(new_cf_version)
 
-        print_step("5b: Close staged artifacts at Maven central.")
+        print_step("Step 5b: Close staged artifacts at Maven central.")
         continue_or_exit(
             "Maven artifacts have been staged!  Please 'close' (but don't release) the artifacts.\n"
             + " * Browse to https://oss.sonatype.org/#stagingRepositories\n"
             + " * Log in using your Sonatype credentials\n"
             + ' * In the search box at upper right, type "checker"\n'
-            + " * In the top pane, click on orgcheckerframework-XXXX\n"
+            + " * In the top pane, click on org.checkerframework-XXXX\n"
             + ' * Click "close" at the top\n'
             + " * For the close message, enter:  Checker Framework release "
             + new_cf_version
@@ -417,7 +429,7 @@ def main(argv):
             "(You can also see the instructions at: http://central.sonatype.org/pages/releasing-the-deployment.html)\n"
         )
 
-        print_step("5c: Run Maven sanity test on Maven central artifacts.")
+        print_step("Step 5c: Run Maven sanity test on Maven central artifacts.")
         if prompt_yes_no("Run Maven sanity test on Maven central artifacts?", True):
             repo_url = input("Please enter the repo URL of the closed artifacts:\n")
 
@@ -498,7 +510,9 @@ def main(argv):
     # available to the Java community through the Central repository. Follow the prompts. The Maven
     # artifacts (such as checker-qual.jar) are still needed, but the Maven plug-in is no longer maintained.
 
-    print_step("Push Step 10. Release staged artifacts in Central repository.")  # MANUAL
+    print_step(
+        "Push Step 10. Release staged artifacts in Central repository."
+    )  # MANUAL
     if test_mode:
         msg = (
             "Test Mode: You are in test_mode.  Please 'DROP' the artifacts. "
@@ -510,7 +524,7 @@ def main(argv):
         msg = (
             "Please 'release' the artifacts.\n"
             + "First log into https://oss.sonatype.org using your Sonatype credentials. Go to Staging Repositories and "
-            + "locate the orgcheckerframework repository and click on it.\n"
+            + "locate the org.checkerframework repository and click on it.\n"
             + "If you have a permissions problem, try logging out and back in.\n"
             + "Finally, click on the Release button at the top of the page. In the dialog box that pops up, "
             + 'leave the "Automatically drop" box checked. For the description, write '
@@ -534,18 +548,18 @@ def main(argv):
 
         msg = (
             "\n"
-            + "* Download the following files to your local machine."
+            + "Download the following files to your local machine."
             + "\n"
-            + "https://checkerframework.org/checker-framework-"
+            + "  https://checkerframework.org/checker-framework-"
             + new_cf_version
             + ".zip\n"
-            + "https://checkerframework.org/annotation-file-utilities/annotation-tools-"
+            + "  https://checkerframework.org/annotation-file-utilities/annotation-tools-"
             + new_cf_version
             + ".zip\n"
             + "\n"
             + "To post the Checker Framework release on GitHub:\n"
             + "\n"
-            + "* Go to https://github.com/typetools/checker-framework/releases/new?tag=checker-framework-"
+            + "* Browse to https://github.com/typetools/checker-framework/releases/new?tag=checker-framework-"
             + new_cf_version
             + "\n"
             + "* For the release title, enter: Checker Framework "
@@ -559,7 +573,7 @@ def main(argv):
             + "\n"
             + "To post the Annotation File Utilities release on GitHub:\n"
             + "\n"
-            + "* Go to https://github.com/typetools/annotation-tools/releases/new?tag="
+            + "* Browse to https://github.com/typetools/annotation-tools/releases/new?tag="
             + new_cf_version
             + "\n"
             + "* For the release title, enter: Annotation File Utilities "
@@ -572,7 +586,7 @@ def main(argv):
             + '* Click on the green "Publish release" button.\n'
         )
 
-        print(msg)
+        continue_or_exit(msg)
 
         print_step("Push Step 12. Announce the release.")  # MANUAL
         continue_or_exit(
@@ -580,28 +594,29 @@ def main(argv):
             + get_announcement_email(new_cf_version)
         )
 
+        print_step("Push Step 13. Prep for next Checker Framework release.")  # MANUAL
+        continue_or_exit(
+            "Change the patch level (last number) of the Checker Framework version\nin build.gradle:  increment it and add -SNAPSHOT\n"
+        )
+
         print_step(
-            "Push Step 13. Update the Checker Framework Gradle plugin."
+            "Push Step 14. Update the Checker Framework Gradle plugin."
         )  # MANUAL
+        print("You might have to wait for Maven Central to propagate changes.\n")
         continue_or_exit(
             "Please update the Checker Framework Gradle plugin:\n"
             + "https://github.com/kelloggm/checkerframework-gradle-plugin/blob/master/RELEASE.md#updating-the-checker-framework-version\n"
         )
         continue_or_exit(
             "Make a pull request to the Checker Framework that\n"
-            + "updates the version number of the Checker Framework"
-            + "Gradle Plugin in docs/example/lombok and docs/example/errorprone .\n"
+            + "updates the version number of the Checker Framework\n"
+            + "Gradle Plugin in docs/examples/lombok and docs/examples/errorprone .\n"
             + "The pull request's tests will fail; you will merge it in a day."
         )
 
-        print_step("Push Step 14. Prep for next Checker Framework release.")  # MANUAL
-        continue_or_exit(
-            "Change the patch level (last number) of the Checker Framework version\nin build.gradle:  increment it and add -SNAPSHOT\n"
-        )
-
     delete_if_exists(RELEASE_BUILD_COMPLETED_FLAG_FILE)
 
-    print("Done with release_push.py")
+    print("Done with release_push.py.\n")
 
 
 if __name__ == "__main__":
