@@ -24,7 +24,7 @@ from release_vars import execute
 # Parse Args Utils # TODO: Perhaps use argparse module
 
 
-def read_command_line_option(argv, argument):
+def has_command_line_option(argv, argument):
     """Returns True if the given command line arguments contain the specified
     argument, False otherwise."""
     for index in range(1, len(argv)):
@@ -130,7 +130,7 @@ def check_tools(tools):
     print(", ".join(tools))
     print(
         (
-            "Note: If you are NOT working on buffalo.cs.washington.edu then you "
+            "Note: If you are NOT working on the CSE file system then you "
             + "likely need to change the variables that are set in release.py\n"
             + 'Search for "Set environment variables".'
         )
@@ -151,6 +151,7 @@ def continue_or_exit(msg):
 # =========================================================================================
 # Version Utils
 
+
 # From http://stackoverflow.com/a/1714190/173852, but doesn't strip trailing zeroes
 def version_number_to_array(version_num):
     """Given a version number, return an array of the elements, as integers."""
@@ -238,7 +239,7 @@ def push_changes_prompt_if_fail(repo_root):
                 + repo_root
                 + "; result="
                 + str(result)
-                + " for command: "
+                + " for command: `"
                 + cmd
                 + "` in "
                 + os.getcwd()
@@ -262,20 +263,22 @@ def update_repo(path, bareflag):
     if bareflag:
         execute("git fetch origin master:master", working_dir=path)
     else:
-        execute("git pull", working_dir=path)
+        execute("git pull --ff-only", working_dir=path)
 
 
 def commit_tag_and_push(version, path, tag_prefix):
     """Commit the changes made for this release, add a tag for this release, and
     push these changes."""
-    execute('git commit -a -m "new release %s"' % (version), working_dir=path)
+    # Do nothing (instead of erring) if there is nothing to commit.
+    if execute("git diff-index --quiet HEAD", False, False, working_dir=path) != 0:
+        execute('git commit -a -m "new release %s"' % (version), working_dir=path)
     execute("git tag %s%s" % (tag_prefix, version), working_dir=path)
     push_changes(path)
 
 
 def clone_from_scratch_or_update(src_repo, dst_repo, clone_from_scratch, bareflag):
-    """If the clone_from_scratch flag is True, clone the given git or
-    Mercurial repo from scratch into the filesystem path specified by dst_repo,
+    """If the clone_from_scratch flag is True, clone the given git
+    repo from scratch into the filesystem path specified by dst_repo,
     deleting it first if the repo is present on the filesystem.
     Otherwise, if a repo exists at the filesystem path given by dst_repo, pull
     the latest changes to it and update it. If the repo does not exist, clone it
@@ -291,7 +294,7 @@ def clone_from_scratch_or_update(src_repo, dst_repo, clone_from_scratch, barefla
 
 
 def delete_and_clone(src_repo, dst_repo, bareflag):
-    """Clone the given git or Mercurial repo from scratch into the filesystem
+    """Clone the given git repo from scratch into the filesystem
     path specified by dst_repo. If a repo exists at the filesystem path given
     by dst_repo, delete it first. The bareflag parameter indicates whether
     the cloned repo must be a bare git repo."""
@@ -300,7 +303,7 @@ def delete_and_clone(src_repo, dst_repo, bareflag):
 
 
 def clone(src_repo, dst_repo, bareflag):
-    """Clone the given git or Mercurial repo from scratch into the filesystem
+    """Clone the given git repo from scratch into the filesystem
     path specified by dst_repo. The bareflag parameter indicates whether the
     cloned repo must be a bare git repo."""
     flags = ""
@@ -554,7 +557,7 @@ http://CheckerFramework.org/
 
 Changes for Checker Framework version %s:
 
-<<Insert latest Checker Framework changelog entry, omitting the first line with the release version and date, and with hard line breaks removed>>
+<<Insert latest Checker Framework changelog entry from https://github.com/typetools/checker-framework/blob/master/docs/CHANGELOG.md, preserving its formatting.>>
 """ % (
         version,
         version,
