@@ -53,7 +53,10 @@ public class PerFileSuite extends RootedSuite {
    *
    * @param klass the class whose tests to run
    */
-  @SuppressWarnings("nullness") // JUnit needs to be annotated
+  @SuppressWarnings({
+    "nullness", // JUnit needs to be annotated
+    "this-escape"
+  })
   public PerFileSuite(Class<?> klass) throws Throwable {
     super(klass, Collections.emptyList());
     TestClass testClass = getTestClass();
