@@ -61,6 +61,9 @@ public class TestUtilities {
   /** True if the JVM is version 21 or above. */
   public static final boolean IS_AT_LEAST_21_JVM = SystemUtil.jreVersion >= 21;
 
+  /** True if the JVM is version 22 or above. */
+  public static final boolean IS_AT_LEAST_22_JVM = SystemUtil.jreVersion >= 22;
+
   static {
     JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
     OutputStream err = new ByteArrayOutputStream();
@@ -261,7 +264,9 @@ public class TestUtilities {
             || (!IS_AT_MOST_17_JVM && nextLine.contains("@above-java17-jdk-skip-test"))
             || (!IS_AT_LEAST_18_JVM && nextLine.contains("@below-java18-jdk-skip-test"))
             || (!IS_AT_MOST_18_JVM && nextLine.contains("@above-java18-jdk-skip-test"))
-            || (!IS_AT_LEAST_21_JVM && nextLine.contains("@below-java21-jdk-skip-test"))) {
+            || (!IS_AT_LEAST_21_JVM && nextLine.contains("@below-java21-jdk-skip-test"))
+            || (!IS_AT_LEAST_22_JVM && nextLine.contains("@below-java22-jdk-skip-test"))) {
+
           return false;
         }
       }
