@@ -12,8 +12,8 @@ import javax.tools.Diagnostic;
 import javax.tools.JavaFileObject;
 import org.checkerframework.checker.nullness.qual.EnsuresNonNullIf;
 import org.checkerframework.checker.nullness.qual.Nullable;
-import org.checkerframework.javacutil.Pair;
 import org.plumelib.util.CollectionsPlume;
+import org.plumelib.util.IPair;
 
 /** A set of utilities and factory methods useful for working with TestDiagnostics. */
 public class TestDiagnosticUtils {
@@ -21,6 +21,7 @@ public class TestDiagnosticUtils {
   /** How the diagnostics appear in Java source files. */
   public static final String DIAGNOSTIC_IN_JAVA_REGEX =
       "\\s*(error|fixable-error|warning|fixable-warning|other):\\s*(\\(?.*\\)?)\\s*";
+
   /** How the diagnostics appear in Java source files. */
   public static final Pattern DIAGNOSTIC_IN_JAVA_PATTERN =
       Pattern.compile(DIAGNOSTIC_IN_JAVA_REGEX);
@@ -81,6 +82,7 @@ public class TestDiagnosticUtils {
         lineNumber,
         stringFromJavaFile);
   }
+
   /**
    * Instantiate a diagnostic from output produced by the Java compiler. The resulting diagnostic is
    * never fixable and always has parentheses.
@@ -88,9 +90,10 @@ public class TestDiagnosticUtils {
   public static TestDiagnostic fromJavaxToolsDiagnostic(
       String diagnosticString, boolean noMsgText) {
     // It would be nice not to parse this from the diagnostic string.
-    // However, diagnostic.toString() may contain "[unchecked]" even though getMessage() does not.
+    // However, diagnostic.toString() may contain "[unchecked]" even though getMessage() does
+    // not.
     // Since we want to match the error messages reported by javac exactly, we must parse.
-    Pair<String, String> trimmed = formatJavaxToolString(diagnosticString, noMsgText);
+    IPair<String, String> trimmed = formatJavaxToolString(diagnosticString, noMsgText);
     return fromPatternMatching(
         DIAGNOSTIC_PATTERN, DIAGNOSTIC_WARNING_PATTERN, trimmed.second, null, trimmed.first);
   }
@@ -112,8 +115,8 @@ public class TestDiagnosticUtils {
         lineNumber,
         DiagnosticKind.JSpecify,
         stringFromJavaFile,
-        /*isFixable=*/ false,
-        /*omitParentheses=*/ true);
+        /* isFixable= */ false,
+        /* omitParentheses= */ true);
   }
 
   /**
@@ -147,7 +150,7 @@ public class TestDiagnosticUtils {
 
     Matcher diagnosticMatcher = diagnosticPattern.matcher(diagnosticString);
     if (diagnosticMatcher.matches()) {
-      Pair<DiagnosticKind, Boolean> categoryToFixable =
+      IPair<DiagnosticKind, Boolean> categoryToFixable =
           parseCategoryString(diagnosticMatcher.group(1 + capturingGroupOffset));
       kind = categoryToFixable.first;
       isFixable = categoryToFixable.second;
@@ -199,14 +202,14 @@ public class TestDiagnosticUtils {
   }
 
   /**
-   * Given a javax diagnostic, return a pair of (trimmed, fileame), where "trimmed" is the first
+   * Given a javax diagnostic, return a pair of (trimmed, filename), where "trimmed" is the first
    * line of the message, without the leading filename.
    *
    * @param original a javax diagnostic
-   * @param noMsgText whether to do work; if false, this returns a pair of (argument, "").
+   * @param noMsgText whether to do work; if false, this returns a pair of (argument, "")
    * @return the diagnostic, split into message and filename
    */
-  public static Pair<String, String> formatJavaxToolString(String original, boolean noMsgText) {
+  public static IPair<String, String> formatJavaxToolString(String original, boolean noMsgText) {
     String trimmed = original;
     String filename = "";
     if (noMsgText) {
@@ -225,7 +228,7 @@ public class TestDiagnosticUtils {
       }
     }
 
-    return Pair.of(trimmed, filename);
+    return IPair.of(trimmed, filename);
   }
 
   /**
@@ -236,8 +239,8 @@ public class TestDiagnosticUtils {
    * @return true if all lines of the message should be shown
    */
   private static boolean retainAllLines(String message) {
-    // Retain all if it is a thrown exception "unexpected Throwable" or it is a Checker Framework
-    // Error (contains "Compilation unit") or is OutOfMemoryError.
+    // Retain all if it is a thrown exception "unexpected Throwable" or it is a Checker
+    // Framework Error (contains "Compilation unit") or is OutOfMemoryError.
     return message.contains("unexpected Throwable")
         || message.contains("Compilation unit")
         || message.contains("OutOfMemoryError");
@@ -247,9 +250,9 @@ public class TestDiagnosticUtils {
    * Given a category string that may be prepended with "fixable-", return the category enum that
    * corresponds with the category and whether or not it is a isFixable error
    */
-  private static Pair<DiagnosticKind, Boolean> parseCategoryString(String category) {
-    final String fixable = "fixable-";
-    final boolean isFixable = category.startsWith(fixable);
+  private static IPair<DiagnosticKind, Boolean> parseCategoryString(String category) {
+    String fixable = "fixable-";
+    boolean isFixable = category.startsWith(fixable);
     if (isFixable) {
       category = category.substring(fixable.length());
     }
@@ -258,7 +261,7 @@ public class TestDiagnosticUtils {
       throw new Error("Unparsable category: " + category);
     }
 
-    return Pair.of(categoryEnum, isFixable);
+    return IPair.of(categoryEnum, isFixable);
   }
 
   /**
@@ -266,7 +269,7 @@ public class TestDiagnosticUtils {
    * continued on the next line.
    */
   public static boolean isJavaDiagnosticLineStart(String originalLine) {
-    final String trimmedLine = originalLine.trim();
+    String trimmedLine = originalLine.trim();
     return trimmedLine.startsWith("// ::") || trimmedLine.startsWith("// warning:");
   }
 
@@ -299,7 +302,7 @@ public class TestDiagnosticUtils {
     if (originalLine == null) {
       return false;
     }
-    final String trimmedLine = originalLine.trim();
+    String trimmedLine = originalLine.trim();
     // Unlike with errors, there is no logic elsewhere for splitting multiple "warning:"s.  So,
     // avoid concatenating them.  Also, each one must begin a line.  They are allowed to wrap to
     // the next line, though.
@@ -322,7 +325,7 @@ public class TestDiagnosticUtils {
    */
   public static TestDiagnosticLine fromJavaSourceLine(
       String filename, String line, long lineNumber) {
-    final String trimmedLine = line.trim();
+    String trimmedLine = line.trim();
     long errorLine = lineNumber + 1;
 
     if (trimmedLine.startsWith("// ::")) {
@@ -365,7 +368,7 @@ public class TestDiagnosticUtils {
 
   /** Convert a line in a DiagnosticFile to a TestDiagnosticLine. */
   public static TestDiagnosticLine fromDiagnosticFileLine(String diagnosticLine) {
-    final String trimmedLine = diagnosticLine.trim();
+    String trimmedLine = diagnosticLine.trim();
     if (trimmedLine.startsWith("#") || trimmedLine.isEmpty()) {
       return new TestDiagnosticLine("", -1, diagnosticLine, Collections.emptyList());
     }
@@ -382,7 +385,7 @@ public class TestDiagnosticUtils {
     for (Diagnostic<? extends JavaFileObject> diagnostic : javaxDiagnostics) {
       // See TestDiagnosticUtils as to why we use diagnostic.toString rather
       // than convert from the diagnostic itself
-      final String diagnosticString = diagnostic.toString();
+      String diagnosticString = diagnostic.toString();
 
       // suppress Xlint warnings
       if (diagnosticString.contains("uses unchecked or unsafe operations.")
