@@ -2,8 +2,9 @@ package org.checkerframework.common.util;
 
 import java.io.BufferedWriter;
 import java.io.File;
-import java.io.FileWriter;
 import java.io.IOException;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
 import java.util.ArrayList;
 import java.util.LinkedHashMap;
 import java.util.List;
@@ -28,9 +29,9 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedTypeVari
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedUnionType;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedWildcardType;
 import org.checkerframework.framework.type.visitor.AnnotatedTypeVisitor;
-import org.checkerframework.framework.util.DefaultAnnotationFormatter;
 import org.checkerframework.framework.util.ExecUtil;
 import org.checkerframework.javacutil.BugInCF;
+import org.checkerframework.javacutil.DefaultAnnotationFormatter;
 import org.plumelib.util.StringsPlume;
 
 /**
@@ -58,8 +59,8 @@ public class TypeVisualizer {
    * @param dest the destination dot file
    * @param type the type to be written
    */
-  public static void drawToDot(final File dest, final AnnotatedTypeMirror type) {
-    final Drawing drawer = new Drawing("Type", type);
+  public static void drawToDot(File dest, AnnotatedTypeMirror type) {
+    Drawing drawer = new Drawing("Type", type);
     drawer.draw(dest);
   }
 
@@ -69,7 +70,7 @@ public class TypeVisualizer {
    * @param dest the destination dot file, this string will be directly passed to new File(dest)
    * @param type the type to be written
    */
-  public static void drawToDot(final String dest, final AnnotatedTypeMirror type) {
+  public static void drawToDot(String dest, AnnotatedTypeMirror type) {
     drawToDot(new File(dest), type);
   }
 
@@ -80,9 +81,9 @@ public class TypeVisualizer {
    * @param dest the destination png file
    * @param type the type to be written
    */
-  public static void drawToPng(final File dest, final AnnotatedTypeMirror type) {
+  public static void drawToPng(File dest, AnnotatedTypeMirror type) {
     try {
-      final File dotFile = File.createTempFile(dest.getName(), ".dot");
+      File dotFile = File.createTempFile(dest.getName(), ".dot");
       drawToDot(dotFile, type);
       execDotToPng(dotFile, dest);
 
@@ -98,7 +99,7 @@ public class TypeVisualizer {
    * @param dest the destination png file, this string will be directly passed to new File(dest)
    * @param type the type to be written
    */
-  public static void drawToPng(final String dest, final AnnotatedTypeMirror type) {
+  public static void drawToPng(String dest, AnnotatedTypeMirror type) {
     drawToPng(new File(dest), type);
   }
 
@@ -109,7 +110,7 @@ public class TypeVisualizer {
    * @param dotFile the dot file to convert
    * @param pngFile the destination of the resultant png file
    */
-  public static void execDotToPng(final File dotFile, final File pngFile) {
+  public static void execDotToPng(File dotFile, File pngFile) {
     String[] cmd =
         new String[] {"dot", "-Tpng", dotFile.getAbsolutePath(), "-o", pngFile.getAbsolutePath()};
     System.out.println("Printing dotFile: " + dotFile + " to loc: " + pngFile);
@@ -124,9 +125,7 @@ public class TypeVisualizer {
    * @return true if the type variable was printed, otherwise false
    */
   public static boolean printTypevarToDotIfMatches(
-      final AnnotatedTypeVariable typeVariable,
-      final List<String> typeVarNames,
-      final String directory) {
+      AnnotatedTypeVariable typeVariable, List<String> typeVarNames, String directory) {
     return printTypevarIfMatches(typeVariable, typeVarNames, directory, false);
   }
 
@@ -137,19 +136,16 @@ public class TypeVisualizer {
    * @return true if the type variable was printed, otherwise false
    */
   public static boolean printTypevarToPngIfMatches(
-      final AnnotatedTypeVariable typeVariable,
-      final List<String> typeVarNames,
-      final String directory) {
+      AnnotatedTypeVariable typeVariable, List<String> typeVarNames, String directory) {
     return printTypevarIfMatches(typeVariable, typeVarNames, directory, true);
   }
 
   private static boolean printTypevarIfMatches(
-      final AnnotatedTypeVariable typeVariable,
-      final List<String> typeVarNames,
-      final String directory,
-      final boolean png) {
-    final String dirPath =
-        directory.endsWith(File.separator) ? directory : directory + File.separator;
+      AnnotatedTypeVariable typeVariable,
+      List<String> typeVarNames,
+      String directory,
+      boolean png) {
+    String dirPath = directory.endsWith(File.separator) ? directory : directory + File.separator;
     String varName = typeVariable.getUnderlyingType().asElement().toString();
 
     if (typeVarNames.contains(varName)) {
@@ -181,7 +177,7 @@ public class TypeVisualizer {
      *
      * @param type the type that the newly-constructed Node represents
      */
-    private Node(final @FindDistinct AnnotatedTypeMirror type) {
+    private Node(@FindDistinct AnnotatedTypeMirror type) {
       this.type = type;
     }
 
@@ -222,49 +218,43 @@ public class TypeVisualizer {
     /** Used to identify nodes uniquely. This field is monotonically increasing. */
     private int nextId = 0;
 
-    public Drawing(final String graphName, final AnnotatedTypeMirror type) {
+    public Drawing(String graphName, AnnotatedTypeMirror type) {
       this.graphName = graphName;
       this.type = type;
     }
 
-    public void draw(final File file) {
+    public void draw(File file) {
       addNodes(type);
       addConnections();
       write(file);
     }
 
-    private void addNodes(final AnnotatedTypeMirror type) {
+    private void addNodes(AnnotatedTypeMirror type) {
       new NodeDrawer().visit(type);
     }
 
     private void addConnections() {
-      final ConnectionDrawer connectionDrawer = new ConnectionDrawer();
-      for (final Node node : nodes.keySet()) {
+      ConnectionDrawer connectionDrawer = new ConnectionDrawer();
+      for (Node node : nodes.keySet()) {
         connectionDrawer.visit(node.type);
       }
     }
 
-    private void write(final File file) {
-      try {
-        BufferedWriter writer = null;
-        try {
-          writer = new BufferedWriter(new FileWriter(file));
-          writer.write("digraph " + graphName + "{");
+    /**
+     * Write this to a file.
+     *
+     * @param file the file to write to
+     */
+    private void write(File file) {
+      try (BufferedWriter writer = Files.newBufferedWriter(file.toPath(), StandardCharsets.UTF_8)) {
+        writer.write("digraph " + graphName + "{");
+        writer.newLine();
+        for (String line : lines) {
+          writer.write(line + ";");
           writer.newLine();
-          for (final String line : lines) {
-            writer.write(line + ";");
-            writer.newLine();
-          }
-
-          writer.write("}");
-          writer.flush();
-        } catch (IOException e) {
-          throw new BugInCF(e, "Exception visualizing type:%nfile=%s%ntype=%s", file, type);
-        } finally {
-          if (writer != null) {
-            writer.close();
-          }
         }
+        writer.write("}");
+        writer.flush();
       } catch (IOException exc) {
         throw new BugInCF(exc, "Exception visualizing type:%nfile=%s%ntype=%s", file, type);
       }
@@ -289,7 +279,7 @@ public class TypeVisualizer {
 
       @Override
       public Void visitDeclared(AnnotatedDeclaredType type, Void aVoid) {
-        final List<AnnotatedTypeMirror> typeArgs = type.getTypeArguments();
+        List<AnnotatedTypeMirror> typeArgs = type.getTypeArguments();
         for (int i = 0; i < typeArgs.size(); i++) {
           lines.add(connect(type, typeArgs.get(i)) + " " + makeTypeArgLabel(i));
         }
@@ -298,7 +288,7 @@ public class TypeVisualizer {
 
       @Override
       public Void visitIntersection(AnnotatedIntersectionType type, Void aVoid) {
-        final List<AnnotatedTypeMirror> bounds = type.getBounds();
+        List<AnnotatedTypeMirror> bounds = type.getBounds();
         for (int i = 0; i < bounds.size(); i++) {
           lines.add(connect(type, bounds.get(i)) + " " + makeLabel("&"));
         }
@@ -307,7 +297,7 @@ public class TypeVisualizer {
 
       @Override
       public Void visitUnion(AnnotatedUnionType type, Void aVoid) {
-        final List<AnnotatedDeclaredType> alternatives = type.getAlternatives();
+        List<AnnotatedDeclaredType> alternatives = type.getAlternatives();
         for (int i = 0; i < alternatives.size(); i++) {
           lines.add(connect(type, alternatives.get(i)) + " " + makeLabel("|"));
         }
@@ -320,10 +310,10 @@ public class TypeVisualizer {
         ExecutableElement methodElem = type.getElement();
         lines.add(connect(type, type.getReturnType()) + " " + makeLabel("returns"));
 
-        final List<? extends TypeParameterElement> typeVarElems = methodElem.getTypeParameters();
-        final List<AnnotatedTypeVariable> typeVars = type.getTypeVariables();
+        List<? extends TypeParameterElement> typeVarElems = methodElem.getTypeParameters();
+        List<AnnotatedTypeVariable> typeVars = type.getTypeVariables();
         for (int i = 0; i < typeVars.size(); i++) {
-          final String typeVarName = typeVarElems.get(i).getSimpleName().toString();
+          String typeVarName = typeVarElems.get(i).getSimpleName().toString();
           lines.add(connect(type, typeVars.get(i)) + " " + makeMethodTypeArgLabel(typeVarName));
         }
 
@@ -331,14 +321,14 @@ public class TypeVisualizer {
           lines.add(connect(type, type.getReceiverType()) + " " + makeLabel("receiver"));
         }
 
-        final List<? extends VariableElement> paramElems = methodElem.getParameters();
-        final List<AnnotatedTypeMirror> params = type.getParameterTypes();
+        List<? extends VariableElement> paramElems = methodElem.getParameters();
+        List<AnnotatedTypeMirror> params = type.getParameterTypes();
         for (int i = 0; i < params.size(); i++) {
-          final String paramName = paramElems.get(i).getSimpleName().toString();
+          String paramName = paramElems.get(i).getSimpleName().toString();
           lines.add(connect(type, params.get(i)) + " " + makeParamLabel(paramName));
         }
 
-        final List<AnnotatedTypeMirror> thrown = type.getThrownTypes();
+        List<AnnotatedTypeMirror> thrown = type.getThrownTypes();
         for (int i = 0; i < thrown.size(); i++) {
           lines.add(connect(type, thrown.get(i)) + " " + makeThrownLabel(i));
         }
@@ -381,42 +371,46 @@ public class TypeVisualizer {
         return null;
       }
 
-      private String connect(final AnnotatedTypeMirror from, final AnnotatedTypeMirror to) {
+      private String connect(AnnotatedTypeMirror from, AnnotatedTypeMirror to) {
         return nodes.get(new Node(from)) + " -> " + nodes.get(new Node(to));
       }
 
-      private String makeLabel(final String text) {
+      private String makeLabel(String text) {
         return "[label=\"" + text + "\"]";
       }
 
-      private String makeTypeArgLabel(final int argIndex) {
+      private String makeTypeArgLabel(int argIndex) {
         return makeLabel("<" + argIndex + ">");
       }
 
-      private String makeMethodTypeArgLabel(final String paramName) {
+      private String makeMethodTypeArgLabel(String paramName) {
         return makeLabel("<" + paramName + ">");
       }
 
-      private String makeParamLabel(final String paramName) {
+      private String makeParamLabel(String paramName) {
         return makeLabel(paramName);
       }
 
-      private String makeThrownLabel(final int index) {
+      private String makeThrownLabel(int index) {
         return makeLabel("throws: " + index);
       }
     }
 
+    /** The default annotation formatter. */
+    private static final DefaultAnnotationFormatter annoFormatter =
+        new DefaultAnnotationFormatter();
+
     /**
      * Scans types and adds a mapping from type to dot node declaration representing that type in
      * the enclosing drawing.
      */
     private class NodeDrawer implements AnnotatedTypeVisitor<Void, Void> {
-      private final DefaultAnnotationFormatter annoFormatter = new DefaultAnnotationFormatter();
 
+      /** Create a new NodeDrawer. */
       public NodeDrawer() {}
 
-      private void visitAll(final List<? extends AnnotatedTypeMirror> types) {
-        for (final AnnotatedTypeMirror type : types) {
+      private void visitAll(List<? extends AnnotatedTypeMirror> types) {
+        for (AnnotatedTypeMirror type : types) {
           visit(type);
         }
       }
@@ -550,17 +544,17 @@ public class TypeVisualizer {
        * @param atm an annotated type
        * @return a string representation of the annotations on {@code atm}
        */
-      public String getAnnoStr(final AnnotatedTypeMirror atm) {
+      public String getAnnoStr(AnnotatedTypeMirror atm) {
         StringJoiner sj = new StringJoiner(" ");
-        for (final AnnotationMirror anno : atm.getAnnotations()) {
+        for (AnnotationMirror anno : atm.getPrimaryAnnotations()) {
           // TODO: More comprehensive escaping
           sj.add(annoFormatter.formatAnnotationMirror(anno).replace("\"", "\\"));
         }
         return sj.toString();
       }
 
-      public boolean checkOrAdd(final AnnotatedTypeMirror atm) {
-        final Node node = new Node(atm);
+      public boolean checkOrAdd(AnnotatedTypeMirror atm) {
+        Node node = new Node(atm);
         if (nodes.containsKey(node)) {
           return false;
         }
@@ -568,29 +562,27 @@ public class TypeVisualizer {
         return true;
       }
 
-      public String makeLabeledNode(final AnnotatedTypeMirror type, final String label) {
+      public String makeLabeledNode(AnnotatedTypeMirror type, String label) {
         return makeLabeledNode(type, label, null);
       }
 
-      public String makeLabeledNode(
-          final AnnotatedTypeMirror type, final String label, final String attributes) {
-        final String attr = (attributes != null) ? ", " + attributes : "";
+      public String makeLabeledNode(AnnotatedTypeMirror type, String label, String attributes) {
+        String attr = (attributes != null) ? ", " + attributes : "";
         return nodes.get(new Node(type)) + " [label=\"" + label + "\"" + attr + "]";
       }
 
-      public void addLabeledNode(final AnnotatedTypeMirror type, final String label) {
+      public void addLabeledNode(AnnotatedTypeMirror type, String label) {
         lines.add(makeLabeledNode(type, label));
       }
 
-      public void addLabeledNode(
-          final AnnotatedTypeMirror type, final String label, final String attributes) {
+      public void addLabeledNode(AnnotatedTypeMirror type, String label, String attributes) {
         lines.add(makeLabeledNode(type, label, attributes));
       }
 
-      public String makeMethodLabel(final AnnotatedExecutableType methodType) {
-        final ExecutableElement methodElem = methodType.getElement();
+      public String makeMethodLabel(AnnotatedExecutableType methodType) {
+        ExecutableElement methodElem = methodType.getElement();
 
-        final StringBuilder builder = new StringBuilder();
+        StringBuilder builder = new StringBuilder();
         builder.append(methodElem.getReturnType().toString());
         builder.append(" <");
 
