@@ -7,6 +7,7 @@ import com.sun.source.tree.Tree;
 import com.sun.source.tree.TypeCastTree;
 import java.util.Collections;
 import java.util.List;
+import java.util.TreeSet;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 import javax.lang.model.element.AnnotationMirror;
@@ -22,6 +23,7 @@ import org.checkerframework.common.basetype.BaseTypeVisitor;
 import org.checkerframework.common.value.qual.IntRangeFromGTENegativeOne;
 import org.checkerframework.common.value.qual.IntRangeFromNonNegative;
 import org.checkerframework.common.value.qual.IntRangeFromPositive;
+import org.checkerframework.common.value.qual.IntVal;
 import org.checkerframework.common.value.qual.StaticallyExecutable;
 import org.checkerframework.common.value.util.NumberUtils;
 import org.checkerframework.common.value.util.Range;
@@ -29,10 +31,13 @@ import org.checkerframework.dataflow.qual.Pure;
 import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedDeclaredType;
 import org.checkerframework.framework.type.visitor.AnnotatedTypeScanner;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.ElementUtils;
 import org.checkerframework.javacutil.TreeUtils;
+import org.checkerframework.javacutil.TypeKindUtils;
 import org.checkerframework.javacutil.TypesUtils;
+import org.plumelib.util.CollectionsPlume;
 
 /** Visitor for the Constant Value type system. */
 public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
@@ -56,21 +61,22 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
    * @param valueExp the AST node for the rvalue (the new value)
    * @param errorKey the error message key to use if the check fails
    * @param extraArgs arguments to the error message key, before "found" and "expected" types
+   * @return true if the check succeeds, false if an error message was issued
    */
   @Override
-  protected void commonAssignmentCheck(
+  protected boolean commonAssignmentCheck(
       AnnotatedTypeMirror varType,
       ExpressionTree valueExp,
       @CompilerMessageKey String errorKey,
       Object... extraArgs) {
 
     replaceSpecialIntRangeAnnotations(varType);
-    super.commonAssignmentCheck(varType, valueExp, errorKey, extraArgs);
+    return super.commonAssignmentCheck(varType, valueExp, errorKey, extraArgs);
   }
 
   @Override
   @FormatMethod
-  protected void commonAssignmentCheck(
+  protected boolean commonAssignmentCheck(
       AnnotatedTypeMirror varType,
       AnnotatedTypeMirror valueType,
       Tree valueTree,
@@ -80,11 +86,11 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
     replaceSpecialIntRangeAnnotations(varType);
 
     if (valueType.getKind() == TypeKind.CHAR
-        && valueType.hasAnnotation(getTypeFactory().UNKNOWNVAL)) {
+        && valueType.hasPrimaryAnnotation(getTypeFactory().UNKNOWNVAL)) {
       valueType.addAnnotation(getTypeFactory().createIntRangeAnnotation(Range.CHAR_EVERYTHING));
     }
 
-    super.commonAssignmentCheck(varType, valueType, valueTree, errorKey, extraArgs);
+    return super.commonAssignmentCheck(varType, valueType, valueTree, errorKey, extraArgs);
   }
 
   /**
@@ -123,9 +129,9 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         new AnnotatedTypeScanner<Void, Void>() {
           @Override
           protected Void scan(AnnotatedTypeMirror type, Void p) {
-            if (type.hasAnnotation(IntRangeFromPositive.class)
-                || type.hasAnnotation(IntRangeFromNonNegative.class)
-                || type.hasAnnotation(IntRangeFromGTENegativeOne.class)) {
+            if (type.hasPrimaryAnnotation(IntRangeFromPositive.class)
+                || type.hasPrimaryAnnotation(IntRangeFromNonNegative.class)
+                || type.hasPrimaryAnnotation(IntRangeFromGTENegativeOne.class)) {
               type.replaceAnnotation(atypeFactory.UNKNOWNVAL);
             }
             return super.scan(type, p);
@@ -161,8 +167,8 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
    * <p>Issues a warning if any @ArrayLen/@ArrayLenRange annotations contain a negative array
    * length.
    *
-   * <p>Issues a warning if any {@literal @}MatchesRegex annotation contains an invalid regular
-   * expression.
+   * <p>Issues a warning if any {@literal @}MatchesRegex or {@literal @}DoesNotMatchRegex annotation
+   * contains an invalid regular expression.
    */
   /* Implementation note: the ValueTypeAnnotator replaces such invalid annotations with valid ones.
    * Therefore, the usual validation in #validateType cannot perform this validation.
@@ -173,25 +179,26 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
    * Therefore, some validation is still done in #validateType below.
    */
   @Override
-  public Void visitAnnotation(AnnotationTree node, Void p) {
-    List<? extends ExpressionTree> args = node.getArguments();
+  public Void visitAnnotation(AnnotationTree tree, Void p) {
+    List<? extends ExpressionTree> args = tree.getArguments();
 
     if (args.isEmpty()) {
       // Nothing to do if there are no annotation arguments.
-      return super.visitAnnotation(node, p);
+      return super.visitAnnotation(tree, p);
     }
 
-    AnnotationMirror anno = TreeUtils.annotationFromAnnotationTree(node);
+    AnnotationMirror anno = TreeUtils.annotationFromAnnotationTree(tree);
     switch (AnnotationUtils.annotationName(anno)) {
       case ValueAnnotatedTypeFactory.INTRANGE_NAME:
         // If there are 2 arguments, issue an error if from.greater.than.to.
-        // If there are fewer than 2 arguments, we needn't worry about this problem because the
-        // other argument will be defaulted to Long.MIN_VALUE or Long.MAX_VALUE accordingly.
+        // If there are fewer than 2 arguments, we needn't worry about this problem because
+        // the other argument will be defaulted to Long.MIN_VALUE or Long.MAX_VALUE
+        // accordingly.
         if (args.size() == 2) {
           long from = getTypeFactory().getIntRangeFromValue(anno);
           long to = getTypeFactory().getIntRangeToValue(anno);
           if (from > to) {
-            checker.reportError(node, "from.greater.than.to");
+            checker.reportError(tree, "from.greater.than.to");
             return null;
           }
         }
@@ -206,11 +213,11 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
             AnnotationUtils.getElementValueArray(anno, "value", Object.class, false);
 
         if (values.isEmpty()) {
-          checker.reportWarning(node, "no.values.given");
+          checker.reportWarning(tree, "no.values.given");
           return null;
         } else if (values.size() > ValueAnnotatedTypeFactory.MAX_VALUES) {
           checker.reportWarning(
-              node,
+              tree,
               (AnnotationUtils.areSameByName(anno, ValueAnnotatedTypeFactory.INTVAL_NAME)
                   ? "too.many.values.given.int"
                   : "too.many.values.given"),
@@ -219,7 +226,7 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         } else if (AnnotationUtils.areSameByName(anno, ValueAnnotatedTypeFactory.ARRAYLEN_NAME)) {
           List<Integer> arrayLens = getTypeFactory().getArrayLength(anno);
           if (Collections.min(arrayLens) < 0) {
-            checker.reportWarning(node, "negative.arraylen", Collections.min(arrayLens));
+            checker.reportWarning(tree, "negative.arraylen", Collections.min(arrayLens));
             return null;
           }
         }
@@ -228,22 +235,34 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         long from = getTypeFactory().getArrayLenRangeFromValue(anno);
         long to = getTypeFactory().getArrayLenRangeToValue(anno);
         if (from > to) {
-          checker.reportError(node, "from.greater.than.to");
+          checker.reportError(tree, "from.greater.than.to");
           return null;
         } else if (from < 0) {
-          checker.reportWarning(node, "negative.arraylen", from);
+          checker.reportWarning(tree, "negative.arraylen", from);
           return null;
         }
         break;
       case ValueAnnotatedTypeFactory.MATCHES_REGEX_NAME:
-        List<String> regexes =
+        List<String> matchesRegexes =
             AnnotationUtils.getElementValueArray(
                 anno, atypeFactory.matchesRegexValueElement, String.class);
-        for (String regex : regexes) {
+        for (String regex : matchesRegexes) {
           try {
             Pattern.compile(regex);
           } catch (PatternSyntaxException pse) {
-            checker.reportWarning(node, "invalid.matches.regex", pse.getMessage());
+            checker.reportWarning(tree, "invalid.matches.regex", pse.getMessage());
+          }
+        }
+        break;
+      case ValueAnnotatedTypeFactory.DOES_NOT_MATCH_REGEX_NAME:
+        List<String> doesNotMatchRegexes =
+            AnnotationUtils.getElementValueArray(
+                anno, atypeFactory.doesNotMatchRegexValueElement, String.class);
+        for (String regex : doesNotMatchRegexes) {
+          try {
+            Pattern.compile(regex);
+          } catch (PatternSyntaxException pse) {
+            checker.reportWarning(tree, "invalid.doesnotmatch.regex", pse.getMessage());
           }
         }
         break;
@@ -251,21 +270,21 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         // Do nothing.
     }
 
-    return super.visitAnnotation(node, p);
+    return super.visitAnnotation(tree, p);
   }
 
   @Override
-  public Void visitTypeCast(TypeCastTree node, Void p) {
-    if (node.getExpression().getKind() == Tree.Kind.NULL_LITERAL) {
+  public Void visitTypeCast(TypeCastTree tree, Void p) {
+    if (tree.getExpression().getKind() == Tree.Kind.NULL_LITERAL) {
       return null;
     }
 
-    AnnotatedTypeMirror castType = atypeFactory.getAnnotatedType(node);
-    AnnotationMirror castAnno = castType.getAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
+    AnnotatedTypeMirror castType = atypeFactory.getAnnotatedType(tree);
+    AnnotationMirror castAnno = castType.getPrimaryAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
     AnnotationMirror exprAnno =
         atypeFactory
-            .getAnnotatedType(node.getExpression())
-            .getAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
+            .getAnnotatedType(tree.getExpression())
+            .getPrimaryAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
 
     // It is always legal to cast to an IntRange type that includes all values
     // of the underlying type. Do not warn about such casts.
@@ -275,8 +294,8 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         && exprAnno != null
         && atypeFactory.isIntRange(castAnno)
         && atypeFactory.isIntRange(exprAnno)) {
-      final Range castRange = atypeFactory.getRange(castAnno);
-      final TypeKind castTypeKind = castType.getKind();
+      Range castRange = atypeFactory.getRange(castAnno);
+      TypeKind castTypeKind = castType.getKind();
       if (castTypeKind == TypeKind.BYTE && castRange.isByteEverything()) {
         return p;
       }
@@ -308,7 +327,85 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         }
       }
     }
-    return super.visitTypeCast(node, p);
+    return super.visitTypeCast(tree, p);
+  }
+
+  // At this point, types are like: (@IntVal(-1) byte, @IntVal(255) int) and knowledge of
+  // signedness is gone.  So, use castType's underlying type to infer correctness of the
+  // cast.  This method returns true for (@IntVal(-1), @IntVal(255)) if the underlying type
+  // is `byte`, but not for any other underlying type.
+  @Override
+  protected boolean isTypeCastSafe(AnnotatedTypeMirror castType, AnnotatedTypeMirror exprType) {
+    TypeKind castTypeKind = TypeKindUtils.primitiveOrBoxedToTypeKind(castType.getUnderlyingType());
+    TypeKind exprTypeKind = TypeKindUtils.primitiveOrBoxedToTypeKind(exprType.getUnderlyingType());
+    if (castTypeKind != null
+        && exprTypeKind != null
+        && TypeKindUtils.isIntegral(castTypeKind)
+        && TypeKindUtils.isIntegral(exprTypeKind)) {
+      AnnotationMirrorSet castAnnos = castType.getPrimaryAnnotations();
+      AnnotationMirrorSet exprAnnos = exprType.getPrimaryAnnotations();
+      if (castAnnos.equals(exprAnnos)) {
+        return true;
+      }
+      assert castAnnos.size() == 1;
+      assert exprAnnos.size() == 1;
+      AnnotationMirror castAnno = castAnnos.first();
+      AnnotationMirror exprAnno = exprAnnos.first();
+      boolean castAnnoIsIntVal = atypeFactory.areSameByClass(castAnno, IntVal.class);
+      boolean exprAnnoIsIntVal = atypeFactory.areSameByClass(exprAnno, IntVal.class);
+      if (castAnnoIsIntVal && exprAnnoIsIntVal) {
+        List<Long> castValues = atypeFactory.getIntValues(castAnno);
+        List<Long> exprValues = atypeFactory.getIntValues(exprAnno);
+        if (castValues.size() == 1 && exprValues.size() == 1) {
+          // Special-case singleton sets for speed.
+          switch (castTypeKind) {
+            case BYTE:
+              return castValues.get(0).byteValue() == exprValues.get(0).byteValue();
+            case INT:
+              return castValues.get(0).intValue() == exprValues.get(0).intValue();
+            case SHORT:
+              return castValues.get(0).shortValue() == exprValues.get(0).shortValue();
+            default:
+              return castValues.get(0).longValue() == exprValues.get(0).longValue();
+          }
+        } else {
+          switch (castTypeKind) {
+            case BYTE:
+              {
+                TreeSet<Byte> castValuesTree =
+                    new TreeSet<Byte>(CollectionsPlume.mapList(Number::byteValue, castValues));
+                TreeSet<Byte> exprValuesTree =
+                    new TreeSet<Byte>(CollectionsPlume.mapList(Number::byteValue, exprValues));
+                return CollectionsPlume.sortedSetContainsAll(castValuesTree, exprValuesTree);
+              }
+            case INT:
+              {
+                TreeSet<Integer> castValuesTree =
+                    new TreeSet<Integer>(CollectionsPlume.mapList(Number::intValue, castValues));
+                TreeSet<Integer> exprValuesTree =
+                    new TreeSet<Integer>(CollectionsPlume.mapList(Number::intValue, exprValues));
+                return CollectionsPlume.sortedSetContainsAll(castValuesTree, exprValuesTree);
+              }
+            case SHORT:
+              {
+                TreeSet<Short> castValuesTree =
+                    new TreeSet<Short>(CollectionsPlume.mapList(Number::shortValue, castValues));
+                TreeSet<Short> exprValuesTree =
+                    new TreeSet<Short>(CollectionsPlume.mapList(Number::shortValue, exprValues));
+                return CollectionsPlume.sortedSetContainsAll(castValuesTree, exprValuesTree);
+              }
+            default:
+              {
+                TreeSet<Long> castValuesTree = new TreeSet<>(castValues);
+                TreeSet<Long> exprValuesTree = new TreeSet<>(exprValues);
+                return CollectionsPlume.sortedSetContainsAll(castValuesTree, exprValuesTree);
+              }
+          }
+        }
+      }
+    }
+
+    return super.isTypeCastSafe(castType, exprType);
   }
 
   /**
@@ -327,7 +424,7 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
       return false;
     }
 
-    AnnotationMirror anno = type.getAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
+    AnnotationMirror anno = type.getPrimaryAnnotationInHierarchy(atypeFactory.UNKNOWNVAL);
     if (anno == null) {
       return false;
     }
@@ -375,25 +472,25 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
   }
 
   @Override
-  public Void visitMethod(MethodTree node, Void p) {
-    super.visitMethod(node, p);
+  public void processMethodTree(String className, MethodTree tree) {
+    super.processMethodTree(className, tree);
 
-    ExecutableElement method = TreeUtils.elementFromDeclaration(node);
+    ExecutableElement method = TreeUtils.elementFromDeclaration(tree);
     if (atypeFactory.getDeclAnnotation(method, StaticallyExecutable.class) != null) {
       // The method is annotated as @StaticallyExecutable.
       if (atypeFactory.getDeclAnnotation(method, Pure.class) == null) {
-        checker.reportWarning(node, "statically.executable.not.pure");
+        checker.reportWarning(tree, "statically.executable.not.pure");
       }
       TypeMirror returnType = method.getReturnType();
       if (returnType.getKind() != TypeKind.VOID && !canBeConstant(returnType)) {
-        checker.reportError(node, "statically.executable.nonconstant.return.type", returnType);
+        checker.reportError(tree, "statically.executable.nonconstant.return.type", returnType);
       }
 
       // Ways to determine the receiver type.
       // 1. This definition of receiverType is null when receiver is implicit and method has
       //    class com.sun.tools.javac.code.Symbol$MethodSymbol.  WHY?
       //        TypeMirror receiverType = method.getReceiverType();
-      //    The same is true of TreeUtils.elementFromDeclaration(node).getReceiverType()
+      //    The same is true of TreeUtils.elementFromDeclaration(tree).getReceiverType()
       //    which seems to conflict with ExecutableType's documentation.
       // 2. Can't use the tree, because the receiver might not be explicit.
       // 3. Check whether method is static and use the declaring class.  Doesn't handle all
@@ -408,7 +505,7 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
           && receiverType.getKind() != TypeKind.NONE
           && !canBeConstant(receiverType)) {
         checker.reportError(
-            node,
+            tree,
             "statically.executable.nonconstant.parameter.type",
             "this (the receiver)",
             returnType);
@@ -418,13 +515,12 @@ public class ValueVisitor extends BaseTypeVisitor<ValueAnnotatedTypeFactory> {
         TypeMirror paramType = param.asType();
         if (paramType.getKind() != TypeKind.NONE && !canBeConstant(paramType)) {
           checker.reportError(
-              node,
+              tree,
               "statically.executable.nonconstant.parameter.type",
               param.getSimpleName().toString(),
               returnType);
         }
       }
     }
-    return null;
   }
 }
