@@ -114,7 +114,12 @@ import java.util.List;
  *
  * @checker_framework.manual #ajava-contents ways in which the two visited ASTs may differ
  */
+@SuppressWarnings("optional:method.invocation") // parallel structure of two data structures
 public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
+
+  /** Create a DoubleJavaParserVisitor. */
+  public DoubleJavaParserVisitor() {}
+
   /**
    * Default action performed on all pairs of nodes from matching ASTs.
    *
@@ -130,17 +135,22 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
    * corresponding elements in order.
    *
    * @param list1 first list of nodes
-   * @param list2 second list of nodes
+   * @param list2 second list of nodes, which has the same size as the first list
    */
   private void visitLists(List<? extends Node> list1, List<? extends Node> list2) {
-    assert list1.size() == list2.size();
+    if (list1.size() != list2.size()) {
+      throw new Error(
+          String.format(
+              "%s.visitLists(%s [size %d], %s [size %d])",
+              this.getClass().getCanonicalName(), list1, list1.size(), list2, list2.size()));
+    }
     for (int i = 0; i < list1.size(); i++) {
       list1.get(i).accept(this, list2.get(i));
     }
   }
 
   @Override
-  public void visit(final AnnotationDeclaration node1, final Node other) {
+  public void visit(AnnotationDeclaration node1, Node other) {
     AnnotationDeclaration node2 = (AnnotationDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getMembers(), node2.getMembers());
@@ -149,17 +159,17 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final AnnotationMemberDeclaration node1, final Node other) {
+  public void visit(AnnotationMemberDeclaration node1, Node other) {
     AnnotationMemberDeclaration node2 = (AnnotationMemberDeclaration) other;
     defaultAction(node1, node2);
-    node1.getDefaultValue().ifPresent(l -> l.accept(this, node2.getDefaultValue().get()));
+    node1.getDefaultValue().ifPresent(dv -> dv.accept(this, node2.getDefaultValue().get()));
     visitLists(node1.getModifiers(), node2.getModifiers());
     node1.getName().accept(this, node2.getName());
     node1.getType().accept(this, node2.getType());
   }
 
   @Override
-  public void visit(final ArrayAccessExpr node1, final Node other) {
+  public void visit(ArrayAccessExpr node1, Node other) {
     ArrayAccessExpr node2 = (ArrayAccessExpr) other;
     defaultAction(node1, node2);
     node1.getIndex().accept(this, node2.getIndex());
@@ -167,31 +177,31 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ArrayCreationExpr node1, final Node other) {
+  public void visit(ArrayCreationExpr node1, Node other) {
     ArrayCreationExpr node2 = (ArrayCreationExpr) other;
     defaultAction(node1, node2);
     node1.getElementType().accept(this, node2.getElementType());
-    node1.getInitializer().ifPresent(l -> l.accept(this, node2.getInitializer().get()));
+    node1.getInitializer().ifPresent(init -> init.accept(this, node2.getInitializer().get()));
     visitLists(node1.getLevels(), node2.getLevels());
   }
 
   @Override
-  public void visit(final ArrayInitializerExpr node1, final Node other) {
+  public void visit(ArrayInitializerExpr node1, Node other) {
     ArrayInitializerExpr node2 = (ArrayInitializerExpr) other;
     defaultAction(node1, node2);
     visitLists(node1.getValues(), node2.getValues());
   }
 
   @Override
-  public void visit(final AssertStmt node1, final Node other) {
+  public void visit(AssertStmt node1, Node other) {
     AssertStmt node2 = (AssertStmt) other;
     defaultAction(node1, node2);
     node1.getCheck().accept(this, node2.getCheck());
-    node1.getMessage().ifPresent(l -> l.accept(this, node2.getMessage().get()));
+    node1.getMessage().ifPresent(m -> m.accept(this, node2.getMessage().get()));
   }
 
   @Override
-  public void visit(final AssignExpr node1, final Node other) {
+  public void visit(AssignExpr node1, Node other) {
     AssignExpr node2 = (AssignExpr) other;
     defaultAction(node1, node2);
     node1.getTarget().accept(this, node2.getTarget());
@@ -199,7 +209,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final BinaryExpr node1, final Node other) {
+  public void visit(BinaryExpr node1, Node other) {
     BinaryExpr node2 = (BinaryExpr) other;
     defaultAction(node1, node2);
     node1.getLeft().accept(this, node2.getLeft());
@@ -207,31 +217,31 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final BlockComment node1, final Node other) {
+  public void visit(BlockComment node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final BlockStmt node1, final Node other) {
+  public void visit(BlockStmt node1, Node other) {
     BlockStmt node2 = (BlockStmt) other;
     defaultAction(node1, node2);
     visitLists(node1.getStatements(), node2.getStatements());
   }
 
   @Override
-  public void visit(final BooleanLiteralExpr node1, final Node other) {
+  public void visit(BooleanLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final BreakStmt node1, final Node other) {
+  public void visit(BreakStmt node1, Node other) {
     BreakStmt node2 = (BreakStmt) other;
     defaultAction(node1, node2);
     node1.getLabel().ifPresent(l -> l.accept(this, node2.getLabel().get()));
   }
 
   @Override
-  public void visit(final CastExpr node1, final Node other) {
+  public void visit(CastExpr node1, Node other) {
     CastExpr node2 = (CastExpr) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
@@ -239,7 +249,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final CatchClause node1, final Node other) {
+  public void visit(CatchClause node1, Node other) {
     CatchClause node2 = (CatchClause) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -247,19 +257,19 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final CharLiteralExpr node1, final Node other) {
+  public void visit(CharLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final ClassExpr node1, final Node other) {
+  public void visit(ClassExpr node1, Node other) {
     ClassExpr node2 = (ClassExpr) other;
     defaultAction(node1, node2);
     node1.getType().accept(this, node2.getType());
   }
 
   @Override
-  public void visit(final ClassOrInterfaceDeclaration node1, final Node other) {
+  public void visit(ClassOrInterfaceDeclaration node1, Node other) {
     ClassOrInterfaceDeclaration node2 = (ClassOrInterfaceDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getExtendedTypes(), node2.getExtendedTypes());
@@ -271,27 +281,27 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ClassOrInterfaceType node1, final Node other) {
+  public void visit(ClassOrInterfaceType node1, Node other) {
     ClassOrInterfaceType node2 = (ClassOrInterfaceType) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
-    node1.getScope().ifPresent(l -> l.accept(this, node2.getScope().get()));
-    node1.getTypeArguments().ifPresent(l -> visitLists(l, node2.getTypeArguments().get()));
+    node1.getScope().ifPresent(s -> s.accept(this, node2.getScope().get()));
+    node1.getTypeArguments().ifPresent(targs -> visitLists(targs, node2.getTypeArguments().get()));
   }
 
   @Override
-  public void visit(final CompilationUnit node1, final Node other) {
+  public void visit(CompilationUnit node1, Node other) {
     CompilationUnit node2 = (CompilationUnit) other;
     defaultAction(node1, node2);
-    node1.getModule().ifPresent(l -> l.accept(this, node2.getModule().get()));
+    node1.getModule().ifPresent(m -> m.accept(this, node2.getModule().get()));
     node1
         .getPackageDeclaration()
-        .ifPresent(l -> l.accept(this, node2.getPackageDeclaration().get()));
+        .ifPresent(pd -> pd.accept(this, node2.getPackageDeclaration().get()));
     visitLists(node1.getTypes(), node2.getTypes());
   }
 
   @Override
-  public void visit(final ConditionalExpr node1, final Node other) {
+  public void visit(ConditionalExpr node1, Node other) {
     ConditionalExpr node2 = (ConditionalExpr) other;
     defaultAction(node1, node2);
     node1.getCondition().accept(this, node2.getCondition());
@@ -300,7 +310,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ConstructorDeclaration node1, final Node other) {
+  public void visit(ConstructorDeclaration node1, Node other) {
     ConstructorDeclaration node2 = (ConstructorDeclaration) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -328,14 +338,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ContinueStmt node1, final Node other) {
+  public void visit(ContinueStmt node1, Node other) {
     ContinueStmt node2 = (ContinueStmt) other;
     defaultAction(node1, node2);
     node1.getLabel().ifPresent(l -> l.accept(this, node2.getLabel().get()));
   }
 
   @Override
-  public void visit(final DoStmt node1, final Node other) {
+  public void visit(DoStmt node1, Node other) {
     DoStmt node2 = (DoStmt) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -343,24 +353,24 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final DoubleLiteralExpr node1, final Node other) {
+  public void visit(DoubleLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final EmptyStmt node1, final Node other) {
+  public void visit(EmptyStmt node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final EnclosedExpr node1, final Node other) {
+  public void visit(EnclosedExpr node1, Node other) {
     EnclosedExpr node2 = (EnclosedExpr) other;
     defaultAction(node1, node2);
     node1.getInner().accept(this, node2.getInner());
   }
 
   @Override
-  public void visit(final EnumConstantDeclaration node1, final Node other) {
+  public void visit(EnumConstantDeclaration node1, Node other) {
     EnumConstantDeclaration node2 = (EnumConstantDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getArguments(), node2.getArguments());
@@ -369,7 +379,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final EnumDeclaration node1, final Node other) {
+  public void visit(EnumDeclaration node1, Node other) {
     EnumDeclaration node2 = (EnumDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getEntries(), node2.getEntries());
@@ -380,7 +390,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ExplicitConstructorInvocationStmt node1, final Node other) {
+  public void visit(ExplicitConstructorInvocationStmt node1, Node other) {
     ExplicitConstructorInvocationStmt node2 = (ExplicitConstructorInvocationStmt) other;
     defaultAction(node1, node2);
     visitLists(node1.getArguments(), node2.getArguments());
@@ -389,14 +399,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ExpressionStmt node1, final Node other) {
+  public void visit(ExpressionStmt node1, Node other) {
     ExpressionStmt node2 = (ExpressionStmt) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
   }
 
   @Override
-  public void visit(final FieldAccessExpr node1, final Node other) {
+  public void visit(FieldAccessExpr node1, Node other) {
     FieldAccessExpr node2 = (FieldAccessExpr) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
@@ -405,7 +415,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final FieldDeclaration node1, final Node other) {
+  public void visit(FieldDeclaration node1, Node other) {
     FieldDeclaration node2 = (FieldDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getModifiers(), node2.getModifiers());
@@ -413,7 +423,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ForEachStmt node1, final Node other) {
+  public void visit(ForEachStmt node1, Node other) {
     ForEachStmt node2 = (ForEachStmt) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -422,7 +432,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ForStmt node1, final Node other) {
+  public void visit(ForStmt node1, Node other) {
     ForStmt node2 = (ForStmt) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -432,7 +442,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final IfStmt node1, final Node other) {
+  public void visit(IfStmt node1, Node other) {
     IfStmt node2 = (IfStmt) other;
     defaultAction(node1, node2);
     node1.getCondition().accept(this, node2.getCondition());
@@ -441,14 +451,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final InitializerDeclaration node1, final Node other) {
+  public void visit(InitializerDeclaration node1, Node other) {
     InitializerDeclaration node2 = (InitializerDeclaration) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
   }
 
   @Override
-  public void visit(final InstanceOfExpr node1, final Node other) {
+  public void visit(InstanceOfExpr node1, Node other) {
     InstanceOfExpr node2 = (InstanceOfExpr) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
@@ -456,17 +466,17 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final IntegerLiteralExpr node1, final Node other) {
+  public void visit(IntegerLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final JavadocComment node1, final Node other) {
+  public void visit(JavadocComment node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final LabeledStmt node1, final Node other) {
+  public void visit(LabeledStmt node1, Node other) {
     LabeledStmt node2 = (LabeledStmt) other;
     defaultAction(node1, node2);
     node1.getLabel().accept(this, node2.getLabel());
@@ -474,24 +484,24 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final LineComment node1, final Node other) {
+  public void visit(LineComment node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final LongLiteralExpr node1, final Node other) {
+  public void visit(LongLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final MarkerAnnotationExpr node1, final Node other) {
+  public void visit(MarkerAnnotationExpr node1, Node other) {
     MarkerAnnotationExpr node2 = (MarkerAnnotationExpr) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
   }
 
   @Override
-  public void visit(final MemberValuePair node1, final Node other) {
+  public void visit(MemberValuePair node1, Node other) {
     MemberValuePair node2 = (MemberValuePair) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
@@ -499,7 +509,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final MethodCallExpr node1, final Node other) {
+  public void visit(MethodCallExpr node1, Node other) {
     MethodCallExpr node2 = (MethodCallExpr) other;
     defaultAction(node1, node2);
     visitLists(node1.getArguments(), node2.getArguments());
@@ -509,7 +519,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final MethodDeclaration node1, final Node other) {
+  public void visit(MethodDeclaration node1, Node other) {
     MethodDeclaration node2 = (MethodDeclaration) other;
     defaultAction(node1, node2);
     node1.getBody().ifPresent(l -> l.accept(this, node2.getBody().get()));
@@ -526,14 +536,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final NameExpr node1, final Node other) {
+  public void visit(NameExpr node1, Node other) {
     NameExpr node2 = (NameExpr) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
   }
 
   @Override
-  public void visit(final NormalAnnotationExpr node1, final Node other) {
+  public void visit(NormalAnnotationExpr node1, Node other) {
     NormalAnnotationExpr node2 = (NormalAnnotationExpr) other;
     defaultAction(node1, node2);
     visitLists(node1.getPairs(), node2.getPairs());
@@ -541,12 +551,12 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final NullLiteralExpr node1, final Node other) {
+  public void visit(NullLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final ObjectCreationExpr node1, final Node other) {
+  public void visit(ObjectCreationExpr node1, Node other) {
     ObjectCreationExpr node2 = (ObjectCreationExpr) other;
     defaultAction(node1, node2);
     node1
@@ -559,14 +569,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final PackageDeclaration node1, final Node other) {
+  public void visit(PackageDeclaration node1, Node other) {
     PackageDeclaration node2 = (PackageDeclaration) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
   }
 
   @Override
-  public void visit(final Parameter node1, final Node other) {
+  public void visit(Parameter node1, Node other) {
     Parameter node2 = (Parameter) other;
     defaultAction(node1, node2);
     visitLists(node1.getModifiers(), node2.getModifiers());
@@ -575,45 +585,45 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final PrimitiveType node1, final Node other) {
+  public void visit(PrimitiveType node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final Name node1, final Node other) {
+  public void visit(Name node1, Node other) {
     Name node2 = (Name) other;
     defaultAction(node1, node2);
     node1.getQualifier().ifPresent(l -> l.accept(this, node2.getQualifier().get()));
   }
 
   @Override
-  public void visit(final SimpleName node1, final Node other) {
+  public void visit(SimpleName node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final ArrayType node1, final Node other) {
+  public void visit(ArrayType node1, Node other) {
     ArrayType node2 = (ArrayType) other;
     defaultAction(node1, node2);
     node1.getComponentType().accept(this, node2.getComponentType());
   }
 
   @Override
-  public void visit(final ArrayCreationLevel node1, final Node other) {
+  public void visit(ArrayCreationLevel node1, Node other) {
     ArrayCreationLevel node2 = (ArrayCreationLevel) other;
     defaultAction(node1, node2);
     node1.getDimension().ifPresent(l -> l.accept(this, node2.getDimension().get()));
   }
 
   @Override
-  public void visit(final IntersectionType node1, final Node other) {
+  public void visit(IntersectionType node1, Node other) {
     IntersectionType node2 = (IntersectionType) other;
     defaultAction(node1, node2);
     visitLists(node1.getElements(), node2.getElements());
   }
 
   @Override
-  public void visit(final UnionType node1, final Node other) {
+  public void visit(UnionType node1, Node other) {
     UnionType node2 = (UnionType) other;
     defaultAction(node1, node2);
     visitLists(node1.getElements(), node2.getElements());
@@ -635,14 +645,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ReturnStmt node1, final Node other) {
+  public void visit(ReturnStmt node1, Node other) {
     ReturnStmt node2 = (ReturnStmt) other;
     defaultAction(node1, node2);
     node1.getExpression().ifPresent(l -> l.accept(this, node2.getExpression().get()));
   }
 
   @Override
-  public void visit(final SingleMemberAnnotationExpr node1, final Node other) {
+  public void visit(SingleMemberAnnotationExpr node1, Node other) {
     SingleMemberAnnotationExpr node2 = (SingleMemberAnnotationExpr) other;
     defaultAction(node1, node2);
     node1.getMemberValue().accept(this, node2.getMemberValue());
@@ -650,19 +660,19 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final StringLiteralExpr node1, final Node other) {
+  public void visit(StringLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final SuperExpr node1, final Node other) {
+  public void visit(SuperExpr node1, Node other) {
     SuperExpr node2 = (SuperExpr) other;
     defaultAction(node1, node2);
     node1.getTypeName().ifPresent(l -> l.accept(this, node2.getTypeName().get()));
   }
 
   @Override
-  public void visit(final SwitchEntry node1, final Node other) {
+  public void visit(SwitchEntry node1, Node other) {
     SwitchEntry node2 = (SwitchEntry) other;
     defaultAction(node1, node2);
     visitLists(node1.getLabels(), node2.getLabels());
@@ -670,7 +680,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final SwitchStmt node1, final Node other) {
+  public void visit(SwitchStmt node1, Node other) {
     SwitchStmt node2 = (SwitchStmt) other;
     defaultAction(node1, node2);
     visitLists(node1.getEntries(), node2.getEntries());
@@ -678,7 +688,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final SynchronizedStmt node1, final Node other) {
+  public void visit(SynchronizedStmt node1, Node other) {
     SynchronizedStmt node2 = (SynchronizedStmt) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -686,21 +696,21 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ThisExpr node1, final Node other) {
+  public void visit(ThisExpr node1, Node other) {
     ThisExpr node2 = (ThisExpr) other;
     defaultAction(node1, node2);
     node1.getTypeName().ifPresent(l -> l.accept(this, node2.getTypeName().get()));
   }
 
   @Override
-  public void visit(final ThrowStmt node1, final Node other) {
+  public void visit(ThrowStmt node1, Node other) {
     ThrowStmt node2 = (ThrowStmt) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
   }
 
   @Override
-  public void visit(final TryStmt node1, final Node other) {
+  public void visit(TryStmt node1, Node other) {
     TryStmt node2 = (TryStmt) other;
     defaultAction(node1, node2);
     visitLists(node1.getCatchClauses(), node2.getCatchClauses());
@@ -710,45 +720,45 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final LocalClassDeclarationStmt node1, final Node other) {
+  public void visit(LocalClassDeclarationStmt node1, Node other) {
     LocalClassDeclarationStmt node2 = (LocalClassDeclarationStmt) other;
     defaultAction(node1, node2);
     node1.getClassDeclaration().accept(this, node2.getClassDeclaration());
   }
 
   @Override
-  public void visit(LocalRecordDeclarationStmt node1, final Node other) {
+  public void visit(LocalRecordDeclarationStmt node1, Node other) {
     LocalRecordDeclarationStmt node2 = (LocalRecordDeclarationStmt) other;
     defaultAction(node1, node2);
     node1.getRecordDeclaration().accept(this, node2.getRecordDeclaration());
   }
 
   @Override
-  public void visit(final TypeParameter node1, final Node other) {
+  public void visit(TypeParameter node1, Node other) {
     TypeParameter node2 = (TypeParameter) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
-    // Since ajava files and its corresponding Java file may differ in whether they contain a type
-    // bound, only visit type bounds if they're present in both nodes.
+    // Since ajava files and its corresponding Java file may differ in whether they contain a
+    // type bound, only visit type bounds if they're present in both nodes.
     if (node1.getTypeBound().isEmpty() == node2.getTypeBound().isEmpty()) {
       visitLists(node1.getTypeBound(), node2.getTypeBound());
     }
   }
 
   @Override
-  public void visit(final UnaryExpr node1, final Node other) {
+  public void visit(UnaryExpr node1, Node other) {
     UnaryExpr node2 = (UnaryExpr) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
   }
 
   @Override
-  public void visit(final UnknownType node1, final Node other) {
+  public void visit(UnknownType node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final VariableDeclarationExpr node1, final Node other) {
+  public void visit(VariableDeclarationExpr node1, Node other) {
     VariableDeclarationExpr node2 = (VariableDeclarationExpr) other;
     defaultAction(node1, node2);
     visitLists(node1.getModifiers(), node2.getModifiers());
@@ -756,7 +766,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final VariableDeclarator node1, final Node other) {
+  public void visit(VariableDeclarator node1, Node other) {
     VariableDeclarator node2 = (VariableDeclarator) other;
     defaultAction(node1, node2);
     node1.getInitializer().ifPresent(l -> l.accept(this, node2.getInitializer().get()));
@@ -765,12 +775,12 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final VoidType node1, final Node other) {
+  public void visit(VoidType node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final WhileStmt node1, final Node other) {
+  public void visit(WhileStmt node1, Node other) {
     WhileStmt node2 = (WhileStmt) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -778,7 +788,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final WildcardType node1, final Node other) {
+  public void visit(WildcardType node1, Node other) {
     WildcardType node2 = (WildcardType) other;
     defaultAction(node1, node2);
     node1.getExtendedType().ifPresent(l -> l.accept(this, node2.getExtendedType().get()));
@@ -786,7 +796,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final LambdaExpr node1, final Node other) {
+  public void visit(LambdaExpr node1, Node other) {
     LambdaExpr node2 = (LambdaExpr) other;
     defaultAction(node1, node2);
     node1.getBody().accept(this, node2.getBody());
@@ -794,7 +804,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final MethodReferenceExpr node1, final Node other) {
+  public void visit(MethodReferenceExpr node1, Node other) {
     MethodReferenceExpr node2 = (MethodReferenceExpr) other;
     defaultAction(node1, node2);
     node1.getScope().accept(this, node2.getScope());
@@ -802,21 +812,21 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final TypeExpr node1, final Node other) {
+  public void visit(TypeExpr node1, Node other) {
     TypeExpr node2 = (TypeExpr) other;
     defaultAction(node1, node2);
     node1.getType().accept(this, node2.getType());
   }
 
   @Override
-  public void visit(final ImportDeclaration node1, final Node other) {
+  public void visit(ImportDeclaration node1, Node other) {
     ImportDeclaration node2 = (ImportDeclaration) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
   }
 
   @Override
-  public void visit(final ModuleDeclaration node1, final Node other) {
+  public void visit(ModuleDeclaration node1, Node other) {
     ModuleDeclaration node2 = (ModuleDeclaration) other;
     defaultAction(node1, node2);
     visitLists(node1.getDirectives(), node2.getDirectives());
@@ -824,7 +834,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ModuleRequiresDirective node1, final Node other) {
+  public void visit(ModuleRequiresDirective node1, Node other) {
     ModuleRequiresDirective node2 = (ModuleRequiresDirective) other;
     defaultAction(node1, node2);
     visitLists(node1.getModifiers(), node2.getModifiers());
@@ -832,7 +842,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ModuleExportsDirective node1, final Node other) {
+  public void visit(ModuleExportsDirective node1, Node other) {
     ModuleExportsDirective node2 = (ModuleExportsDirective) other;
     defaultAction(node1, node2);
     visitLists(node1.getModuleNames(), node2.getModuleNames());
@@ -840,7 +850,7 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ModuleProvidesDirective node1, final Node other) {
+  public void visit(ModuleProvidesDirective node1, Node other) {
     ModuleProvidesDirective node2 = (ModuleProvidesDirective) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
@@ -848,14 +858,14 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final ModuleUsesDirective node1, final Node other) {
+  public void visit(ModuleUsesDirective node1, Node other) {
     ModuleUsesDirective node2 = (ModuleUsesDirective) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
   }
 
   @Override
-  public void visit(final ModuleOpensDirective node1, final Node other) {
+  public void visit(ModuleOpensDirective node1, Node other) {
     ModuleOpensDirective node2 = (ModuleOpensDirective) other;
     defaultAction(node1, node2);
     visitLists(node1.getModuleNames(), node2.getModuleNames());
@@ -863,12 +873,12 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final UnparsableStmt node1, final Node other) {
+  public void visit(UnparsableStmt node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final ReceiverParameter node1, final Node other) {
+  public void visit(ReceiverParameter node1, Node other) {
     ReceiverParameter node2 = (ReceiverParameter) other;
     defaultAction(node1, node2);
     node1.getName().accept(this, node2.getName());
@@ -876,17 +886,17 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final VarType node1, final Node other) {
+  public void visit(VarType node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final Modifier node1, final Node other) {
+  public void visit(Modifier node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final SwitchExpr node1, final Node other) {
+  public void visit(SwitchExpr node1, Node other) {
     SwitchExpr node2 = (SwitchExpr) other;
     defaultAction(node1, node2);
     visitLists(node1.getEntries(), node2.getEntries());
@@ -894,12 +904,12 @@ public abstract class DoubleJavaParserVisitor extends VoidVisitorAdapter<Node> {
   }
 
   @Override
-  public void visit(final TextBlockLiteralExpr node1, final Node other) {
+  public void visit(TextBlockLiteralExpr node1, Node other) {
     defaultAction(node1, other);
   }
 
   @Override
-  public void visit(final YieldStmt node1, final Node other) {
+  public void visit(YieldStmt node1, Node other) {
     YieldStmt node2 = (YieldStmt) other;
     defaultAction(node1, node2);
     node1.getExpression().accept(this, node2.getExpression());
