@@ -1,15 +1,20 @@
 package org.checkerframework.framework.ajava;
 
+import com.github.javaparser.JavaToken;
+import com.github.javaparser.JavaToken.Kind;
 import com.github.javaparser.Position;
+import com.github.javaparser.TokenRange;
 import com.github.javaparser.ast.CompilationUnit;
 import com.github.javaparser.ast.ImportDeclaration;
 import com.github.javaparser.ast.Node;
+import com.github.javaparser.ast.NodeList;
 import com.github.javaparser.ast.body.FieldDeclaration;
 import com.github.javaparser.ast.body.MethodDeclaration;
 import com.github.javaparser.ast.body.TypeDeclaration;
 import com.github.javaparser.ast.expr.AnnotationExpr;
 import com.github.javaparser.ast.nodeTypes.NodeWithAnnotations;
 import com.github.javaparser.ast.type.ArrayType;
+import com.github.javaparser.ast.type.ArrayType.ArrayBracketPair;
 import com.github.javaparser.ast.type.ClassOrInterfaceType;
 import com.github.javaparser.ast.type.Type;
 import com.github.javaparser.printer.DefaultPrettyPrinter;
@@ -46,16 +51,18 @@ import javax.tools.JavaCompiler.CompilationTask;
 import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
 import javax.tools.ToolProvider;
+import org.checkerframework.checker.nullness.qual.MonotonicNonNull;
 import org.checkerframework.checker.signature.qual.DotSeparatedIdentifiers;
 import org.checkerframework.checker.signature.qual.FullyQualifiedName;
 import org.checkerframework.framework.stub.AnnotationFileParser;
 import org.checkerframework.framework.util.JavaParserUtil;
+import org.plumelib.util.CollectionsPlume;
 import org.plumelib.util.FilesPlume;
 
 /** This program inserts annotations from an ajava file into a Java file. See {@link #main}. */
 public class InsertAjavaAnnotations {
   /** Element utilities. */
-  private Elements elements;
+  private final Elements elements;
 
   /**
    * Constructs an {@code InsertAjavaAnnotations} using the given {@code Elements} instance.
@@ -79,31 +86,41 @@ public class InsertAjavaAnnotations {
     }
 
     DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();
-    JavaFileManager fileManager = compiler.getStandardFileManager(diagnostics, null, null);
-    if (fileManager == null) {
-      System.err.println("Could not get file manager");
-      System.exit(1);
-    }
+    try (JavaFileManager fileManager = compiler.getStandardFileManager(diagnostics, null, null)) {
+      if (fileManager == null) {
+        System.err.println("Could not get file manager");
+        System.exit(1);
+      }
 
-    CompilationTask cTask =
-        compiler.getTask(
-            null, fileManager, diagnostics, Collections.emptyList(), null, Collections.emptyList());
-    if (!(cTask instanceof JavacTask)) {
-      System.err.println("Could not get a valid JavacTask: " + cTask.getClass());
-      System.exit(1);
-    }
+      CompilationTask cTask =
+          compiler.getTask(
+              null,
+              fileManager,
+              diagnostics,
+              Collections.emptyList(),
+              null,
+              Collections.emptyList());
+      if (!(cTask instanceof JavacTask)) {
+        System.err.println("Could not get a valid JavacTask: " + cTask.getClass());
+        System.exit(1);
+      }
 
-    return ((JavacTask) cTask).getElements();
+      return ((JavacTask) cTask).getElements();
+    } catch (IOException e) {
+      throw new Error(e);
+    }
   }
 
   /** Represents some text to be inserted at a file and its location. */
   private static class Insertion {
     /** Offset of the insertion in the file, measured in characters from the beginning. */
-    public int position;
+    public final int position;
+
     /** The contents of the insertion. */
-    public String contents;
+    public final String contents;
+
     /** Whether the insertion should be on its own separate line. */
-    public boolean ownLine;
+    public final boolean ownLine;
 
     /**
      * Constructs an insertion with the given position and contents.
@@ -152,21 +169,25 @@ public class InsertAjavaAnnotations {
      * <p>The map is populated from import statements and also when parsing a file that uses the
      * fully qualified name of an annotation it doesn't import.
      */
-    private Map<String, TypeElement> allAnnotations;
+    private @MonotonicNonNull Map<String, TypeElement> allAnnotations = null;
 
     /** The annotation insertions seen so far. */
-    public List<Insertion> insertions;
+    public final List<Insertion> insertions = new ArrayList<>();
+
     /** A printer for annotations. */
-    private DefaultPrettyPrinter printer;
+    private final DefaultPrettyPrinter printer = new DefaultPrettyPrinter();
+
     /** The lines of the String representation of the second AST. */
-    private List<String> lines;
+    private final List<String> lines;
+
     /** The line separator used in the text the second AST was parsed from */
-    private String lineSeparator;
+    private final String lineSeparator;
+
     /**
      * Stores the offsets of the lines in the string representation of the second AST. At index i,
      * stores the number of characters from the start of the file to the beginning of the ith line.
      */
-    private List<Integer> cumulativeLineSizes;
+    private final List<Integer> cumulativeLineSizes;
 
     /**
      * Constructs a {@code BuildInsertionsVisitor} where {@code destFileContents} is the String
@@ -178,12 +199,10 @@ public class InsertAjavaAnnotations {
      */
     public BuildInsertionsVisitor(String destFileContents, String lineSeparator) {
       allAnnotations = null;
-      insertions = new ArrayList<>();
-      printer = new DefaultPrettyPrinter();
       String[] lines = destFileContents.split(lineSeparator);
       this.lines = Arrays.asList(lines);
       this.lineSeparator = lineSeparator;
-      cumulativeLineSizes = new ArrayList<>();
+      cumulativeLineSizes = new ArrayList<>(lines.length);
       cumulativeLineSizes.add(0);
       for (int i = 1; i < lines.length; i++) {
         int lastSize = cumulativeLineSizes.get(i - 1);
@@ -211,8 +230,8 @@ public class InsertAjavaAnnotations {
       // `src`'s annotations are type annotations.
       Position position;
       if (dest instanceof ClassOrInterfaceType) {
-        // In a multi-part name like my.package.MyClass, type annotations go directly in front of
-        // MyClass instead of the full name.
+        // In a multi-part name like my.package.MyClass, type annotations go directly in
+        // front of MyClass instead of the full name.
         position = ((ClassOrInterfaceType) dest).getName().getBegin().get();
       } else {
         position = dest.getBegin().get();
@@ -223,26 +242,33 @@ public class InsertAjavaAnnotations {
     @Override
     public void visit(ArrayType src, Node other) {
       ArrayType dest = (ArrayType) other;
-      // The second component of this pair contains a list of ArrayBracketPairs from left to
-      // right. For example, if src contains String[][], then the list will contain the
-      // types String[] and String[][]. To insert array annotations in the correct location,
-      // we insert them directly to the right of the end of the previous element.
-      Pair<Type, List<ArrayType.ArrayBracketPair>> srcArrayTypes = ArrayType.unwrapArrayTypes(src);
-      Pair<Type, List<ArrayType.ArrayBracketPair>> destArrayTypes =
-          ArrayType.unwrapArrayTypes(dest);
-      // The first annotations go directly after the element type.
-      Position firstPosition = destArrayTypes.a.getEnd().get();
-      addAnnotations(firstPosition, srcArrayTypes.b.get(0).getAnnotations(), 1, false);
-      for (int i = 1; i < srcArrayTypes.b.size(); i++) {
-        Position position = destArrayTypes.b.get(i - 1).getTokenRange().get().toRange().get().end;
-        addAnnotations(position, srcArrayTypes.b.get(i).getAnnotations(), 1, true);
+      Pair<Type, List<ArrayBracketPair>> destArrayTypes = ArrayType.unwrapArrayTypes(dest);
+      TokenRange innerMostCom = destArrayTypes.a.getTokenRange().get();
+
+      List<Position> positions = new ArrayList<>();
+      for (JavaToken token : dest.getTokenRange().get().withBegin(innerMostCom.getEnd())) {
+        if (token.getKind() == Kind.LBRACKET.getKind()) {
+          positions.add(token.getRange().get().begin);
+        }
       }
 
-      // Visit the component type.
-      srcArrayTypes.a.accept(this, destArrayTypes.a);
+      // At the end of the loop, these two variables will contain the innermost array type.
+      ArrayType srcArray = src;
+      ArrayType destArray = dest;
+      for (Position position : positions) {
+        addAnnotations(position, srcArray.getAnnotations(), 0, true);
+        if (srcArray.getComponentType().isArrayType()) {
+          srcArray = (ArrayType) srcArray.getComponentType();
+          destArray = (ArrayType) destArray.getComponentType();
+        }
+      }
+
+      // Visit the innermost component type.
+      srcArray.getComponentType().accept(this, destArray.getComponentType());
     }
 
     @Override
+    @SuppressWarnings("optional:method.invocation") // parallel structure of two data structures
     public void visit(CompilationUnit src, Node other) {
       CompilationUnit dest = (CompilationUnit) other;
       defaultAction(src, dest);
@@ -250,16 +276,18 @@ public class InsertAjavaAnnotations {
       // Gather annotations used in the ajava file.
       allAnnotations = getImportedAnnotations(src);
 
-      // Move any annotations that JavaParser puts in the declaration position but belong only in
-      // the type position.
+      // Move any annotations that JavaParser puts in the declaration position but belong only
+      // in the type position.
       src.accept(new TypeAnnotationMover(allAnnotations, elements), null);
 
       // Transfer import statements from the ajava file to the Java file.
 
       List<String> newImports;
       { // set `newImports`
-        Set<String> existingImports = new HashSet<>();
-        for (ImportDeclaration importDecl : dest.getImports()) {
+        NodeList<ImportDeclaration> destImports = dest.getImports();
+        Set<String> existingImports =
+            new HashSet<>(CollectionsPlume.mapCapacity(destImports.size()));
+        for (ImportDeclaration importDecl : destImports) {
           existingImports.add(printer.print(importDecl));
         }
 
@@ -299,10 +327,11 @@ public class InsertAjavaAnnotations {
         insertions.add(new Insertion(position, insertionContent));
       }
 
-      src.getModule().ifPresent(l -> l.accept(this, dest.getModule().get()));
+      src.getModule().ifPresent(m -> m.accept(this, dest.getModule().get()));
       src.getPackageDeclaration()
-          .ifPresent(l -> l.accept(this, dest.getPackageDeclaration().get()));
-      for (int i = 0; i < src.getTypes().size(); i++) {
+          .ifPresent(pd -> pd.accept(this, dest.getPackageDeclaration().get()));
+      int numTypes = src.getTypes().size();
+      for (int i = 0; i < numTypes; i++) {
         src.getTypes().get(i).accept(this, dest.getTypes().get(i));
       }
     }
@@ -506,27 +535,26 @@ public class InsertAjavaAnnotations {
   }
 
   /**
-   * Inserts all annotations from the ajava file at {@code annotationFilePath} into {@code
-   * javaFilePath}.
+   * Inserts all annotations from an ajava file into a Java file.
    *
-   * @param annotationFilePath path to an ajava file
-   * @param javaFilePath path to a Java file to insert annotation into
+   * @param annotationFileName an ajava file
+   * @param javaFileName a Java file to insert annotation into
    */
-  public void insertAnnotations(String annotationFilePath, String javaFilePath) {
+  public void insertAnnotations(String annotationFileName, String javaFileName) {
     try {
-      File javaFile = new File(javaFilePath);
-      String fileContents = FilesPlume.readFile(javaFile);
-      String lineSeparator = FilesPlume.inferLineSeparator(annotationFilePath);
-      FileInputStream annotationInputStream = new FileInputStream(annotationFilePath);
-      String result = insertAnnotations(annotationInputStream, fileContents, lineSeparator);
-      annotationInputStream.close();
-      FilesPlume.writeFile(javaFile, result);
+      File javaFile = new File(javaFileName);
+      String fileContents = FilesPlume.readString(Path.of(javaFileName));
+      String lineSeparator = FilesPlume.inferLineSeparator(annotationFileName);
+      try (FileInputStream annotationInputStream = new FileInputStream(annotationFileName)) {
+        String result = insertAnnotations(annotationInputStream, fileContents, lineSeparator);
+        FilesPlume.writeString(javaFile, result);
+      }
     } catch (IOException e) {
       System.err.println(
           "Failed to insert annotations from file "
-              + annotationFilePath
+              + annotationFileName
               + " into file "
-              + javaFilePath);
+              + javaFileName);
       System.exit(1);
     }
   }
@@ -574,8 +602,11 @@ public class InsertAjavaAnnotations {
               System.exit(1);
             }
 
-            Set<String> annotationFilesForRoot = new LinkedHashSet<>();
-            for (TypeDeclaration<?> type : root.getTypes()) {
+            List<TypeDeclaration<?>> rootTypes = root.getTypes();
+            // Estimate of size.
+            Set<String> annotationFilesForRoot =
+                new LinkedHashSet<>(CollectionsPlume.mapCapacity(rootTypes.size()));
+            for (TypeDeclaration<?> type : rootTypes) {
               String name = JavaParserUtil.getFullyQualifiedName(type, root);
               annotationFilesForRoot.addAll(annotationFiles.getAnnotationFileForType(name));
             }
