@@ -1,5 +1,7 @@
 package org.checkerframework.framework.flow;
 
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -9,9 +11,9 @@ import java.util.StringJoiner;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.BinaryOperator;
 import javax.lang.model.element.AnnotationMirror;
-import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.Name;
+import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.Types;
 import org.checkerframework.checker.nullness.qual.EnsuresNonNullIf;
@@ -31,17 +33,18 @@ import org.checkerframework.dataflow.expression.FieldAccess;
 import org.checkerframework.dataflow.expression.JavaExpression;
 import org.checkerframework.dataflow.expression.LocalVariable;
 import org.checkerframework.dataflow.expression.MethodCall;
+import org.checkerframework.dataflow.expression.SuperReference;
 import org.checkerframework.dataflow.expression.ThisReference;
 import org.checkerframework.dataflow.qual.SideEffectFree;
-import org.checkerframework.dataflow.util.PurityUtils;
 import org.checkerframework.framework.qual.MonotonicQualifier;
 import org.checkerframework.framework.type.AnnotatedTypeFactory;
 import org.checkerframework.framework.type.GenericAnnotatedTypeFactory;
 import org.checkerframework.javacutil.AnnotationBuilder;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.BugInCF;
-import org.checkerframework.javacutil.Pair;
+import org.checkerframework.javacutil.ElementUtils;
 import org.plumelib.util.CollectionsPlume;
+import org.plumelib.util.IPair;
 import org.plumelib.util.ToStringComparator;
 import org.plumelib.util.UniqueId;
 
@@ -89,18 +92,19 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    * Information collected about array elements, using the internal representation {@link
    * ArrayAccess}.
    */
-  protected Map<ArrayAccess, V> arrayValues;
+  protected final Map<ArrayAccess, V> arrayValues;
 
   /**
-   * Information collected about method calls, using the internal representation {@link MethodCall}.
+   * Information collected about the expressions to which method calls evaluate, using the internal
+   * representation {@link MethodCall}.
    */
-  protected Map<MethodCall, V> methodValues;
+  protected final Map<MethodCall, V> methodCallExpressions;
 
   /**
    * Information collected about <i>classname</i>.class values, using the internal representation
    * {@link ClassName}.
    */
-  protected Map<ClassName, V> classValues;
+  protected final Map<ClassName, V> classValues;
 
   /**
    * Should the analysis use sequential Java semantics (i.e., assume that only one thread is running
@@ -108,10 +112,17 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    */
   protected final boolean sequentialSemantics;
 
+  /** True if -AassumeSideEffectFree or -AassumePure was passed on the command line. */
+  private final boolean assumeSideEffectFree;
+
+  /** True if -AassumePureGetters was passed on the command line. */
+  private final boolean assumePureGetters;
+
   /** The unique ID for the next-created object. */
-  static final AtomicLong nextUid = new AtomicLong(0);
+  private static final AtomicLong nextUid = new AtomicLong(0);
+
   /** The unique ID of this object. */
-  final transient long uid = nextUid.getAndIncrement();
+  private final transient long uid = nextUid.getAndIncrement();
 
   @Override
   public long getUid() {
@@ -130,25 +141,35 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    */
   protected CFAbstractStore(CFAbstractAnalysis<V, S, ?> analysis, boolean sequentialSemantics) {
     this.analysis = analysis;
-    localVariableValues = new HashMap<>();
-    thisValue = null;
-    fieldValues = new HashMap<>();
-    methodValues = new HashMap<>();
-    arrayValues = new HashMap<>();
-    classValues = new HashMap<>();
+    this.localVariableValues = new HashMap<>();
+    this.thisValue = null;
+    this.fieldValues = new HashMap<>();
+    this.methodCallExpressions = new HashMap<>();
+    this.arrayValues = new HashMap<>();
+    this.classValues = new HashMap<>();
     this.sequentialSemantics = sequentialSemantics;
+    this.assumeSideEffectFree =
+        analysis.checker.hasOption("assumeSideEffectFree")
+            || analysis.checker.hasOption("assumePure");
+    this.assumePureGetters = analysis.checker.hasOption("assumePureGetters");
   }
 
-  /** Copy constructor. */
+  /**
+   * Copy constructor.
+   *
+   * @param other a CFAbstractStore to copy into this
+   */
   protected CFAbstractStore(CFAbstractStore<V, S> other) {
     this.analysis = other.analysis;
-    localVariableValues = new HashMap<>(other.localVariableValues);
-    thisValue = other.thisValue;
-    fieldValues = new HashMap<>(other.fieldValues);
-    methodValues = new HashMap<>(other.methodValues);
-    arrayValues = new HashMap<>(other.arrayValues);
-    classValues = new HashMap<>(other.classValues);
-    sequentialSemantics = other.sequentialSemantics;
+    this.localVariableValues = new HashMap<>(other.localVariableValues);
+    this.thisValue = other.thisValue;
+    this.fieldValues = new HashMap<>(other.fieldValues);
+    this.methodCallExpressions = new HashMap<>(other.methodCallExpressions);
+    this.arrayValues = new HashMap<>(other.arrayValues);
+    this.classValues = new HashMap<>(other.classValues);
+    this.sequentialSemantics = other.sequentialSemantics;
+    this.assumeSideEffectFree = other.assumeSideEffectFree;
+    this.assumePureGetters = other.assumePureGetters;
   }
 
   /**
@@ -181,9 +202,11 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    * @param atypeFactory the type factory used to retrieve annotations on the method element
    * @param method the method element
    * @return whether the method is side-effect-free
+   * @deprecated use {@link org.checkerframework.javacutil.AnnotationProvider#isSideEffectFree}
    */
+  @Deprecated // 2022-09-27
   protected boolean isSideEffectFree(AnnotatedTypeFactory atypeFactory, ExecutableElement method) {
-    return PurityUtils.isSideEffectFree(atypeFactory, method);
+    return atypeFactory.isSideEffectFree(method);
   }
 
   /* --------------------------------------------------------- */
@@ -199,31 +222,42 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    *       org.checkerframework.dataflow.qual.SideEffectFree} or {@link
    *       org.checkerframework.dataflow.qual.Pure}), then no information needs to be removed.
    *   <li>Otherwise, all information about field accesses {@code a.f} needs to be removed, except
-   *       if the method {@code n} cannot modify {@code a.f} (e.g., if {@code a} is a local variable
-   *       or {@code this}, and {@code f} is final).
+   *       if the method {@code n} cannot modify {@code a.f}. This unmodifiability property holds if
+   *       {@code a} is a local variable or {@code this}, and {@code f} is final, or if {@code a.f}
+   *       has a {@link MonotonicQualifier} in the current store. Subclasses can change this
+   *       behavior by overriding {@link #newFieldValueAfterMethodCall(FieldAccess,
+   *       GenericAnnotatedTypeFactory, CFAbstractValue)}.
    *   <li>Furthermore, if the field has a monotonic annotation, then its information can also be
    *       kept.
    * </ol>
    *
    * Furthermore, if the method is deterministic, we store its result {@code val} in the store.
+   *
+   * @param methodInvocationNode method whose information is being updated
+   * @param atypeFactory the type factory of the associated checker
+   * @param val abstract value of the method call
    */
   public void updateForMethodCall(
-      MethodInvocationNode n, AnnotatedTypeFactory atypeFactory, V val) {
-    ExecutableElement method = n.getTarget().getMethod();
+      MethodInvocationNode methodInvocationNode, AnnotatedTypeFactory atypeFactory, V val) {
+    ExecutableElement method = methodInvocationNode.getTarget().getMethod();
+    @SuppressWarnings("unchecked")
+    GenericAnnotatedTypeFactory<V, S, ?, ?> gatypeFactory =
+        (GenericAnnotatedTypeFactory<V, S, ?, ?>) atypeFactory;
 
-    // case 1: remove information if necessary
-    if (!(analysis.checker.hasOption("assumeSideEffectFree")
-        || analysis.checker.hasOption("assumePure")
-        || isSideEffectFree(atypeFactory, method))) {
+    // Case 1: The method is side-effect-free.
+    boolean hasSideEffect =
+        !(assumeSideEffectFree
+            || (assumePureGetters && ElementUtils.isGetter(method))
+            || atypeFactory.isSideEffectFree(method));
+    if (hasSideEffect) {
 
-      boolean sideEffectsUnrefineAliases =
-          ((GenericAnnotatedTypeFactory) atypeFactory).sideEffectsUnrefineAliases;
+      boolean sideEffectsUnrefineAliases = gatypeFactory.sideEffectsUnrefineAliases;
 
       // update local variables
       // TODO: Also remove if any element/argument to the annotation is not
       // isUnmodifiableByOtherCode.  Example: @KeyFor("valueThatCanBeMutated").
       if (sideEffectsUnrefineAliases) {
-        localVariableValues.entrySet().removeIf(e -> !e.getKey().isUnmodifiableByOtherCode());
+        localVariableValues.entrySet().removeIf(e -> e.getKey().isModifiableByOtherCode());
       }
 
       // update this value
@@ -233,68 +267,124 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
 
       // update field values
       if (sideEffectsUnrefineAliases) {
-        fieldValues.entrySet().removeIf(e -> !e.getKey().isUnmodifiableByOtherCode());
+        fieldValues.entrySet().removeIf(e -> e.getKey().isModifiableByOtherCode());
       } else {
-        Map<FieldAccess, V> newFieldValues =
-            new HashMap<>(CollectionsPlume.mapCapacity(fieldValues));
-        for (Map.Entry<FieldAccess, V> e : fieldValues.entrySet()) {
-          FieldAccess fieldAccess = e.getKey();
-          V otherVal = e.getValue();
-
-          // case 3: the field has a monotonic annotation
-          if (!((GenericAnnotatedTypeFactory<?, ?, ?, ?>) atypeFactory)
-              .getSupportedMonotonicTypeQualifiers()
-              .isEmpty()) {
-            List<Pair<AnnotationMirror, AnnotationMirror>> fieldAnnotations =
-                atypeFactory.getAnnotationWithMetaAnnotation(
-                    fieldAccess.getField(), MonotonicQualifier.class);
-            V newOtherVal = null;
-            for (Pair<AnnotationMirror, AnnotationMirror> fieldAnnotation : fieldAnnotations) {
-              AnnotationMirror monotonicAnnotation = fieldAnnotation.second;
-              @SuppressWarnings("deprecation") // permitted for use in the framework
-              Name annotation =
-                  AnnotationUtils.getElementValueClassName(monotonicAnnotation, "value", false);
-              AnnotationMirror target =
-                  AnnotationBuilder.fromName(atypeFactory.getElementUtils(), annotation);
-              // Make sure the 'target' annotation is present.
-              if (AnnotationUtils.containsSame(otherVal.getAnnotations(), target)) {
-                newOtherVal =
-                    analysis
-                        .createSingleAnnotationValue(target, otherVal.getUnderlyingType())
-                        .mostSpecific(newOtherVal, null);
-              }
-            }
-            if (newOtherVal != null) {
-              // keep information for all hierarchies where we had a
-              // monotone annotation.
-              newFieldValues.put(fieldAccess, newOtherVal);
-              continue;
-            }
-          }
-
-          // case 2:
-          if (!fieldAccess.isUnassignableByOtherCode()) {
-            continue; // remove information completely
-          }
-
-          // keep information
-          newFieldValues.put(fieldAccess, otherVal);
-        }
-        fieldValues = newFieldValues;
+        // Case 2 (unassignable fields) and case 3 (monotonic fields)
+        updateFieldValuesForMethodCall(gatypeFactory);
       }
 
       // update array values
       arrayValues.clear();
 
       // update method values
-      methodValues.keySet().removeIf(e -> !e.isUnmodifiableByOtherCode());
+      methodCallExpressions.keySet().removeIf(MethodCall::isModifiableByOtherCode);
     }
 
     // store information about method call if possible
-    JavaExpression methodCall = JavaExpression.fromNode(n);
+    JavaExpression methodCall = JavaExpression.fromNode(methodInvocationNode);
     replaceValue(methodCall, val);
   }
 
+  /**
+   * Returns the new value of a field after a method call, or {@code null} if the field should be
+   * removed from the store.
+   *
+   * <p>In this default implementation, the field's value is preserved if it is either unassignable
+   * (see {@link FieldAccess#isUnassignableByOtherCode()}) or has a monotonic qualifier (see {@link
+   * #newMonotonicFieldValueAfterMethodCall(FieldAccess, GenericAnnotatedTypeFactory,
+   * CFAbstractValue)}). Otherwise, it is removed from the store.
+   *
+   * @param fieldAccess the field whose value to update
+   * @param atypeFactory AnnotatedTypeFactory of the associated checker
+   * @param value the field's value before the method call
+   * @return the field's value after the method call, or {@code null} if the field should be removed
+   *     from the store
+   */
+  protected V newFieldValueAfterMethodCall(
+      FieldAccess fieldAccess, GenericAnnotatedTypeFactory<V, S, ?, ?> atypeFactory, V value) {
+    // Handle unassignable fields.
+    if (!fieldAccess.isAssignableByOtherCode()) {
+      return value;
+    }
+
+    // Handle fields with monotonic annotations.
+    return newMonotonicFieldValueAfterMethodCall(fieldAccess, atypeFactory, value);
+  }
+
+  /**
+   * Computes the value of a field whose declaration has a monotonic annotation, or returns {@code
+   * null} if the field has no monotonic annotation.
+   *
+   * <p>Used by {@link #newFieldValueAfterMethodCall(FieldAccess, GenericAnnotatedTypeFactory,
+   * CFAbstractValue)} to handle fields with monotonic annotations.
+   *
+   * @param fieldAccess the field whose value to compute
+   * @param atypeFactory AnnotatedTypeFactory of the associated checker
+   * @param value the field's value before the method call
+   * @return the field's value after the method call, or {@code null} if the field has no monotonic
+   *     annotation
+   */
+  protected V newMonotonicFieldValueAfterMethodCall(
+      FieldAccess fieldAccess, GenericAnnotatedTypeFactory<V, S, ?, ?> atypeFactory, V value) {
+
+    // case 3: the field has a monotonic annotation
+    if (atypeFactory.getSupportedMonotonicTypeQualifiers().isEmpty()) {
+      return null;
+    }
+
+    List<IPair<AnnotationMirror, AnnotationMirror>> fieldAnnotationPairs =
+        atypeFactory.getAnnotationWithMetaAnnotation(
+            fieldAccess.getField(), MonotonicQualifier.class);
+    List<AnnotationMirror> metaAnnotations =
+        CollectionsPlume.withoutDuplicates(
+            CollectionsPlume.mapList(pair -> pair.second, fieldAnnotationPairs));
+    List<AnnotationMirror> monotonicAnnotations = new ArrayList<>(metaAnnotations.size());
+    for (AnnotationMirror metaAnnotation : metaAnnotations) {
+      @SuppressWarnings("deprecation") // permitted for use in the framework
+      Name annoName = AnnotationUtils.getElementValueClassName(metaAnnotation, "value", false);
+      monotonicAnnotations.add(
+          AnnotationBuilder.fromName(atypeFactory.getElementUtils(), annoName));
+    }
+    Collection<AnnotationMirror> valueAnnos = value.getAnnotations();
+    V newValue = null;
+    for (AnnotationMirror monotonicAnnotation : monotonicAnnotations) {
+      // Make sure the target annotation is present.
+      if (AnnotationUtils.containsSame(valueAnnos, monotonicAnnotation)) {
+        newValue =
+            analysis
+                .createSingleAnnotationValue(monotonicAnnotation, value.getUnderlyingType())
+                .mostSpecific(newValue, null);
+      }
+    }
+    return newValue;
+  }
+
+  /**
+   * Helper for {@link #updateForMethodCall(MethodInvocationNode, AnnotatedTypeFactory,
+   * CFAbstractValue)}. Remove any information about field values that might not be valid any more
+   * after a method call, and add information guaranteed by the method.
+   *
+   * <p>More specifically, remove all information about fields except for unassignable fields and
+   * fields that have a monotonic annotation.
+   *
+   * @param atypeFactory AnnotatedTypeFactory of the associated checker
+   */
+  private void updateFieldValuesForMethodCall(
+      GenericAnnotatedTypeFactory<V, S, ?, ?> atypeFactory) {
+    Map<FieldAccess, V> newFieldValues = new HashMap<>(CollectionsPlume.mapCapacity(fieldValues));
+    for (Map.Entry<FieldAccess, V> e : fieldValues.entrySet()) {
+      FieldAccess fieldAccess = e.getKey();
+      V value = e.getValue();
+
+      V newValue = newFieldValueAfterMethodCall(fieldAccess, atypeFactory, value);
+      if (newValue != null) {
+        // Keep information for all hierarchies where we had a monotonic annotation.
+        newFieldValues.put(fieldAccess, newValue);
+      }
+    }
+    fieldValues = newFieldValues;
+  }
+
   /**
    * Add the annotation {@code a} for the expression {@code expr} (correctly deciding where to store
    * the information depending on the type of the expression {@code expr}).
@@ -405,6 +495,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
   public static boolean canInsertJavaExpression(JavaExpression expr) {
     if (expr instanceof FieldAccess
         || expr instanceof ThisReference
+        || expr instanceof SuperReference
         || expr instanceof LocalVariable
         || expr instanceof MethodCall
         || expr instanceof ArrayAccess
@@ -435,6 +526,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
   public final void insertValue(JavaExpression expr, @Nullable V value) {
     insertValue(expr, value, false);
   }
+
   /**
    * Like {@link #insertValue(JavaExpression, CFAbstractValue)}, but updates the store even if
    * {@code expr} is nondeterministic.
@@ -444,9 +536,9 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    * {@code nondet()} is 3, because it might not be 3 the next time {@code nondet()} is executed.
    *
    * <p>However, contracts can mention a nondeterministic JavaExpression. For example, a contract
-   * might have a postcondition that{@code nondet()} is odd. This means that the next call to{@code
-   * nondet()} will return odd. Such a postcondition may be evicted from the store by calling a
-   * side-effecting method.
+   * might have a postcondition that {@code nondet()} is odd. This means that the next call to
+   * {@code nondet()} will return odd. Such a postcondition may be evicted from the store by calling
+   * a side-effecting method.
    *
    * @param expr the expression to insert in the store
    * @param value the value of the expression
@@ -483,7 +575,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     }
     if (!(permitNondeterministic || expr.isDeterministic(analysis.getTypeFactory()))) {
       // Nondeterministic expressions may not be stored.
-      // (They are likely to be quickly evicted, as soon as a side-effecting method is called.)
+      // (They are likely to be quickly evicted, as soon as a side-effecting method is
+      // called.)
       return false;
     }
     return true;
@@ -543,7 +636,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       // Only store information about final fields (where the receiver is
       // also fixed) if concurrent semantics are enabled.
       boolean isMonotonic = isMonotonicUpdate(fieldAcc, value);
-      if (sequentialSemantics || isMonotonic || fieldAcc.isUnassignableByOtherCode()) {
+      if (sequentialSemantics || isMonotonic || !fieldAcc.isAssignableByOtherCode()) {
         V oldValue = fieldValues.get(fieldAcc);
         V newValue = merger.apply(oldValue, value);
         if (newValue != null) {
@@ -554,10 +647,10 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       MethodCall method = (MethodCall) expr;
       // Don't store any information if concurrent semantics are enabled.
       if (sequentialSemantics) {
-        V oldValue = methodValues.get(method);
+        V oldValue = methodCallExpressions.get(method);
         V newValue = merger.apply(oldValue, value);
         if (newValue != null) {
-          methodValues.put(method, newValue);
+          methodCallExpressions.put(method, newValue);
         }
       }
     } else if (expr instanceof ArrayAccess) {
@@ -569,9 +662,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
           arrayValues.put(arrayAccess, newValue);
         }
       }
-    } else if (expr instanceof ThisReference) {
-      ThisReference thisRef = (ThisReference) expr;
-      if (sequentialSemantics || thisRef.isUnassignableByOtherCode()) {
+    } else if (expr instanceof ThisReference || expr instanceof SuperReference) {
+      if (sequentialSemantics || !expr.isAssignableByOtherCode()) {
         V oldValue = thisValue;
         V newValue = merger.apply(oldValue, value);
         if (newValue != null) {
@@ -580,7 +672,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       }
     } else if (expr instanceof ClassName) {
       ClassName className = (ClassName) expr;
-      if (sequentialSemantics || className.isUnassignableByOtherCode()) {
+      if (sequentialSemantics || !className.isAssignableByOtherCode()) {
         V oldValue = classValues.get(className);
         V newValue = merger.apply(oldValue, value);
         if (newValue != null) {
@@ -611,18 +703,17 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     // TODO: Update the javadoc of this method when the above to-do item is addressed.
     if (!sequentialSemantics) { // only compute if necessary
       AnnotatedTypeFactory atypeFactory = this.analysis.atypeFactory;
-      List<Pair<AnnotationMirror, AnnotationMirror>> fieldAnnotations =
+      List<IPair<AnnotationMirror, AnnotationMirror>> fieldAnnotations =
           atypeFactory.getAnnotationWithMetaAnnotation(
               fieldAcc.getField(), MonotonicQualifier.class);
-      for (Pair<AnnotationMirror, AnnotationMirror> fieldAnnotation : fieldAnnotations) {
-        AnnotationMirror monotonicAnnotation = fieldAnnotation.second;
+      for (IPair<AnnotationMirror, AnnotationMirror> fieldAnnotation : fieldAnnotations) {
+        AnnotationMirror metaAnnotation = fieldAnnotation.second;
         @SuppressWarnings("deprecation") // permitted for use in the framework
-        Name annotation =
-            AnnotationUtils.getElementValueClassName(monotonicAnnotation, "value", false);
-        AnnotationMirror target =
-            AnnotationBuilder.fromName(atypeFactory.getElementUtils(), annotation);
+        Name annoName = AnnotationUtils.getElementValueClassName(metaAnnotation, "value", false);
+        AnnotationMirror monotonicAnnotation =
+            AnnotationBuilder.fromName(atypeFactory.getElementUtils(), annoName);
         // Make sure the 'target' annotation is present.
-        if (AnnotationUtils.containsSame(value.getAnnotations(), target)) {
+        if (AnnotationUtils.containsSame(value.getAnnotations(), monotonicAnnotation)) {
           isMonotonic = true;
           break;
         }
@@ -675,14 +766,16 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       fieldValues.remove(fieldAcc);
     } else if (expr instanceof MethodCall) {
       MethodCall method = (MethodCall) expr;
-      methodValues.remove(method);
+      methodCallExpressions.remove(method);
     } else if (expr instanceof ArrayAccess) {
       ArrayAccess a = (ArrayAccess) expr;
       arrayValues.remove(a);
     } else if (expr instanceof ClassName) {
       ClassName c = (ClassName) expr;
       classValues.remove(c);
-    } else { // thisValue ...
+    } else if (expr instanceof ThisReference) {
+      thisValue = null;
+    } else {
       // No other types of expressions are stored.
     }
   }
@@ -698,14 +791,14 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     if (expr instanceof LocalVariable) {
       LocalVariable localVar = (LocalVariable) expr;
       return localVariableValues.get(localVar);
-    } else if (expr instanceof ThisReference) {
+    } else if (expr instanceof ThisReference || expr instanceof SuperReference) {
       return thisValue;
     } else if (expr instanceof FieldAccess) {
       FieldAccess fieldAcc = (FieldAccess) expr;
       return fieldValues.get(fieldAcc);
     } else if (expr instanceof MethodCall) {
       MethodCall method = (MethodCall) expr;
-      return methodValues.get(method);
+      return methodCallExpressions.get(method);
     } else if (expr instanceof ArrayAccess) {
       ArrayAccess a = (ArrayAccess) expr;
       return arrayValues.get(a);
@@ -731,7 +824,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       return fieldValues.get((FieldAccess) je);
     } else if (je instanceof ClassName) {
       return classValues.get((ClassName) je);
-    } else if (je instanceof ThisReference) {
+    } else if (je instanceof ThisReference || je instanceof SuperReference) {
       // "return thisValue" is wrong, because the node refers to an outer this.
       // So, return null for now.  TODO: improve.
       return null;
@@ -767,7 +860,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     if (method == null) {
       return null;
     }
-    return methodValues.get(method);
+    return methodCallExpressions.get(method);
   }
 
   /**
@@ -816,7 +909,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       // also fixed) if concurrent semantics are enabled.
       if (sequentialSemantics
           || isMonotonicUpdate(fieldAccess, val)
-          || fieldAccess.isUnassignableByOtherCode()) {
+          || !fieldAccess.isAssignableByOtherCode()) {
         fieldValues.put(fieldAccess, val);
       }
     }
@@ -877,8 +970,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    *     abstract value is not known).
    */
   protected void removeConflicting(FieldAccess fieldAccess, @Nullable V val) {
-    final Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator =
-        fieldValues.entrySet().iterator();
+    Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator = fieldValues.entrySet().iterator();
     while (fieldValuesIterator.hasNext()) {
       Map.Entry<FieldAccess, V> entry = fieldValuesIterator.next();
       FieldAccess otherFieldAccess = entry.getKey();
@@ -903,8 +995,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       }
     }
 
-    final Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator =
-        arrayValues.entrySet().iterator();
+    Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator = arrayValues.entrySet().iterator();
     while (arrayValuesIterator.hasNext()) {
       Map.Entry<ArrayAccess, V> entry = arrayValuesIterator.next();
       ArrayAccess otherArrayAccess = entry.getKey();
@@ -915,7 +1006,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     }
 
     // case 3:
-    methodValues.clear();
+    methodCallExpressions.clear();
   }
 
   /**
@@ -938,8 +1029,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    *     abstract value is not known).
    */
   protected void removeConflicting(ArrayAccess arrayAccess, @Nullable V val) {
-    final Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator =
-        arrayValues.entrySet().iterator();
+    Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator = arrayValues.entrySet().iterator();
     while (arrayValuesIterator.hasNext()) {
       Map.Entry<ArrayAccess, V> entry = arrayValuesIterator.next();
       ArrayAccess otherArrayAccess = entry.getKey();
@@ -954,8 +1044,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     }
 
     // case 2:
-    final Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator =
-        fieldValues.entrySet().iterator();
+    Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator = fieldValues.entrySet().iterator();
     while (fieldValuesIterator.hasNext()) {
       Map.Entry<FieldAccess, V> entry = fieldValuesIterator.next();
       FieldAccess otherFieldAccess = entry.getKey();
@@ -968,7 +1057,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     }
 
     // case 3:
-    methodValues.clear();
+    methodCallExpressions.clear();
   }
 
   /**
@@ -985,8 +1074,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    * </ol>
    */
   protected void removeConflicting(LocalVariable var) {
-    final Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator =
-        fieldValues.entrySet().iterator();
+    Iterator<Map.Entry<FieldAccess, V>> fieldValuesIterator = fieldValues.entrySet().iterator();
     while (fieldValuesIterator.hasNext()) {
       Map.Entry<FieldAccess, V> entry = fieldValuesIterator.next();
       FieldAccess otherFieldAccess = entry.getKey();
@@ -996,8 +1084,7 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       }
     }
 
-    final Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator =
-        arrayValues.entrySet().iterator();
+    Iterator<Map.Entry<ArrayAccess, V>> arrayValuesIterator = arrayValues.entrySet().iterator();
     while (arrayValuesIterator.hasNext()) {
       Map.Entry<ArrayAccess, V> entry = arrayValuesIterator.next();
       ArrayAccess otherArrayAccess = entry.getKey();
@@ -1007,14 +1094,14 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       }
     }
 
-    final Iterator<Map.Entry<MethodCall, V>> methodValuesIterator =
-        methodValues.entrySet().iterator();
-    while (methodValuesIterator.hasNext()) {
-      Map.Entry<MethodCall, V> entry = methodValuesIterator.next();
+    Iterator<Map.Entry<MethodCall, V>> methodCallValuesIterator =
+        methodCallExpressions.entrySet().iterator();
+    while (methodCallValuesIterator.hasNext()) {
+      Map.Entry<MethodCall, V> entry = methodCallValuesIterator.next();
       MethodCall otherMethodAccess = entry.getKey();
       // case 3:
       if (otherMethodAccess.containsSyntacticEqualJavaExpression(var)) {
-        methodValuesIterator.remove();
+        methodCallValuesIterator.remove();
       }
     }
   }
@@ -1039,11 +1126,12 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    * Returns the current abstract value of a local variable, or {@code null} if no information is
    * available.
    *
+   * @param n the local variable
    * @return the current abstract value of a local variable, or {@code null} if no information is
    *     available
    */
   public @Nullable V getValue(LocalVariableNode n) {
-    Element el = n.getElement();
+    VariableElement el = n.getElement();
     return localVariableValues.get(new LocalVariable(el));
   }
 
@@ -1087,8 +1175,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     S newStore = analysis.createEmptyStore(sequentialSemantics);
 
     for (Map.Entry<LocalVariable, V> e : other.localVariableValues.entrySet()) {
-      // local variables that are only part of one store, but not the other are discarded, as one of
-      // store implicitly contains 'top' for that variable.
+      // local variables that are only part of one store, but not the other are discarded, as
+      // one of store implicitly contains 'top' for that variable.
       LocalVariable localVar = e.getKey();
       V thisVal = localVariableValues.get(localVar);
       if (thisVal != null) {
@@ -1105,15 +1193,16 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     {
       V otherVal = other.thisValue;
       V myVal = thisValue;
-      V mergedVal = myVal == null ? null : upperBoundOfValues(otherVal, myVal, shouldWiden);
-      if (mergedVal != null) {
-        newStore.thisValue = mergedVal;
+      if (myVal == null || otherVal == null) {
+        newStore.thisValue = null;
+      } else {
+        newStore.thisValue = upperBoundOfValues(otherVal, myVal, shouldWiden);
       }
     }
 
     for (Map.Entry<FieldAccess, V> e : other.fieldValues.entrySet()) {
-      // information about fields that are only part of one store, but not the other are discarded,
-      // as one store implicitly contains 'top' for that field.
+      // information about fields that are only part of one store, but not the other are
+      // discarded, as one store implicitly contains 'top' for that field.
       FieldAccess el = e.getKey();
       V thisVal = fieldValues.get(el);
       if (thisVal != null) {
@@ -1125,8 +1214,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
       }
     }
     for (Map.Entry<ArrayAccess, V> e : other.arrayValues.entrySet()) {
-      // information about arrays that are only part of one store, but not the other are discarded,
-      // as one store implicitly contains 'top' for that array access.
+      // information about arrays that are only part of one store, but not the other are
+      // discarded, as one store implicitly contains 'top' for that array access.
       ArrayAccess el = e.getKey();
       V thisVal = arrayValues.get(el);
       if (thisVal != null) {
@@ -1137,16 +1226,16 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
         }
       }
     }
-    for (Map.Entry<MethodCall, V> e : other.methodValues.entrySet()) {
-      // information about methods that are only part of one store, but not the other are discarded,
-      // as one store implicitly contains 'top' for that field.
+    for (Map.Entry<MethodCall, V> e : other.methodCallExpressions.entrySet()) {
+      // information about methods that are only part of one store, but not the other are
+      // discarded, as one store implicitly contains 'top' for that field.
       MethodCall el = e.getKey();
-      V thisVal = methodValues.get(el);
+      V thisVal = methodCallExpressions.get(el);
       if (thisVal != null) {
         V otherVal = e.getValue();
         V mergedVal = upperBoundOfValues(otherVal, thisVal, shouldWiden);
         if (mergedVal != null) {
-          newStore.methodValues.put(el, mergedVal);
+          newStore.methodCallExpressions.put(el, mergedVal);
         }
       }
     }
@@ -1199,9 +1288,9 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
         return false;
       }
     }
-    for (Map.Entry<MethodCall, V> e : other.methodValues.entrySet()) {
+    for (Map.Entry<MethodCall, V> e : other.methodCallExpressions.entrySet()) {
       MethodCall key = e.getKey();
-      V value = methodValues.get(key);
+      V value = methodCallExpressions.get(key);
       if (value == null || !value.equals(e.getValue())) {
         return false;
       }
@@ -1241,8 +1330,9 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
 
   @Override
   public String visualize(CFGVisualizer<?, S, ?> viz) {
-    /* This cast is guaranteed to be safe, as long as the CFGVisualizer is created by
-     * CFGVisualizer<Value, Store, TransferFunction> createCFGVisualizer() of GenericAnnotatedTypeFactory */
+    // This cast is guaranteed to be safe, as long as the CFGVisualizer is created by
+    // CFGVisualizer<Value, Store, TransferFunction> createCFGVisualizer() of
+    // GenericAnnotatedTypeFactory.
     @SuppressWarnings("unchecked")
     CFGVisualizer<V, S, ?> castedViz = (CFGVisualizer<V, S, ?>) viz;
     String internal = internalVisualize(castedViz);
@@ -1261,8 +1351,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
    */
   protected String internalVisualize(CFGVisualizer<V, S, ?> viz) {
     StringJoiner res = new StringJoiner(viz.getSeparator());
-    for (Map.Entry<LocalVariable, V> entry : localVariableValues.entrySet()) {
-      res.add(viz.visualizeStoreLocalVar(entry.getKey(), entry.getValue()));
+    for (LocalVariable lv : ToStringComparator.sorted(localVariableValues.keySet())) {
+      res.add(viz.visualizeStoreLocalVar(lv, localVariableValues.get(lv)));
     }
     if (thisValue != null) {
       res.add(viz.visualizeStoreThisVal(thisValue));
@@ -1273,8 +1363,8 @@ public abstract class CFAbstractStore<V extends CFAbstractValue<V>, S extends CF
     for (ArrayAccess fa : ToStringComparator.sorted(arrayValues.keySet())) {
       res.add(viz.visualizeStoreArrayVal(fa, arrayValues.get(fa)));
     }
-    for (MethodCall fa : ToStringComparator.sorted(methodValues.keySet())) {
-      res.add(viz.visualizeStoreMethodVals(fa, methodValues.get(fa)));
+    for (MethodCall fa : ToStringComparator.sorted(methodCallExpressions.keySet())) {
+      res.add(viz.visualizeStoreMethodVals(fa, methodCallExpressions.get(fa)));
     }
     for (ClassName fa : ToStringComparator.sorted(classValues.keySet())) {
       res.add(viz.visualizeStoreClassVals(fa, classValues.get(fa)));
