@@ -1,10 +1,9 @@
 package org.checkerframework.framework.flow;
 
-import java.util.Set;
-import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.type.TypeMirror;
 import org.checkerframework.common.basetype.BaseTypeChecker;
 import org.checkerframework.framework.type.GenericAnnotatedTypeFactory;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 
 /** The default org.checkerframework.dataflow analysis used in the Checker Framework. */
 public class CFAnalysis extends CFAbstractAnalysis<CFValue, CFStore, CFTransfer> {
@@ -32,7 +31,7 @@ public class CFAnalysis extends CFAbstractAnalysis<CFValue, CFStore, CFTransfer>
   }
 
   @Override
-  public CFValue createAbstractValue(Set<AnnotationMirror> annotations, TypeMirror underlyingType) {
+  public CFValue createAbstractValue(AnnotationMirrorSet annotations, TypeMirror underlyingType) {
     return defaultCreateAbstractValue(this, annotations, underlyingType);
   }
 }
