@@ -128,7 +128,10 @@ public class AnnotationClassLoader implements Closeable {
    *
    * @param checker a {@link BaseTypeChecker} or its subclass
    */
-  @SuppressWarnings("signature") // TODO: reduce use of string manipulation
+  @SuppressWarnings({
+    "signature", // TODO: reduce use of string manipulation
+    "this-escape"
+  })
   public AnnotationClassLoader(BaseTypeChecker checker) {
     this.checker = checker;
     processingEnv = checker.getProcessingEnvironment();
@@ -501,6 +504,7 @@ public class AnnotationClassLoader implements Closeable {
    * Loads the set of annotation classes in the qual directory of a checker shipped with the Checker
    * Framework.
    */
+  @SuppressWarnings("this-escape")
   private void loadBundledAnnotationClasses() {
     // retrieve the fully qualified class names of the annotations
     Set<@BinaryName String> annotationNames;
