@@ -192,7 +192,7 @@ public final class ElementAnnotationApplier {
 
     if (paramDecl != null) {
       Tree parentTree = typeFactory.getPath(paramDecl).getParentPath().getLeaf();
-      if (parentTree != null && parentTree.getKind() == Tree.Kind.LAMBDA_EXPRESSION) {
+      if (parentTree != null && parentTree instanceof LambdaExpressionTree) {
         return IPair.of(paramDecl, (LambdaExpressionTree) parentTree);
       }
     }
