@@ -5,7 +5,6 @@ import com.sun.source.tree.MemberReferenceTree;
 import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.NewClassTree;
 import com.sun.source.tree.Tree;
-import com.sun.source.tree.Tree.Kind;
 import com.sun.tools.javac.code.Attribute;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Type;
@@ -721,7 +720,7 @@ public class AnnotatedTypes {
       AnnotatedExecutableType preType,
       boolean inferTypeArgs) {
 
-    if (expr.getKind() != Kind.MEMBER_REFERENCE
+    if (!(expr instanceof MemberReferenceTree)
         && elt.getTypeParameters().isEmpty()
         && !TreeUtils.isDiamondTree(expr)) {
       return emptyFalsePair;
