@@ -6,11 +6,11 @@ import com.sun.source.tree.IdentifierTree;
 import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
+import com.sun.source.tree.ModifiersTree;
 import com.sun.source.tree.Tree;
 import com.sun.source.tree.TypeParameterTree;
 import com.sun.source.tree.VariableTree;
 import com.sun.source.util.TreePath;
-import com.sun.tools.javac.code.Type.WildcardType;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.IdentityHashMap;
@@ -28,6 +28,7 @@ import javax.lang.model.element.TypeParameterElement;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.util.Elements;
 import org.checkerframework.checker.interning.qual.FindDistinct;
+import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.framework.qual.AnnotatedFor;
 import org.checkerframework.framework.qual.DefaultQualifier;
 import org.checkerframework.framework.qual.TypeUseLocation;
@@ -42,13 +43,15 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedWildcard
 import org.checkerframework.framework.type.GenericAnnotatedTypeFactory;
 import org.checkerframework.framework.type.QualifierHierarchy;
 import org.checkerframework.framework.type.visitor.AnnotatedTypeScanner;
+import org.checkerframework.framework.util.AnnotatedTypes;
 import org.checkerframework.javacutil.AnnotationBuilder;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.BugInCF;
-import org.checkerframework.javacutil.CollectionUtils;
 import org.checkerframework.javacutil.ElementUtils;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypesUtils;
+import org.plumelib.util.CollectionsPlume;
 import org.plumelib.util.StringsPlume;
 
 /**
@@ -84,8 +87,10 @@ public class QualifierDefaults {
 
   /** The value() element/field of a @DefaultQualifier annotation. */
   protected final ExecutableElement defaultQualifierValueElement;
+
   /** The locations() element/field of a @DefaultQualifier annotation. */
   protected final ExecutableElement defaultQualifierLocationsElement;
+
   /** The value() element/field of a @DefaultQualifier.List annotation. */
   protected final ExecutableElement defaultQualifierListValueElement;
 
@@ -103,7 +108,7 @@ public class QualifierDefaults {
 
   /** Mapping from an Element to the bound type. */
   protected final Map<Element, BoundType> elementToBoundType =
-      CollectionUtils.createLRUCache(CACHE_SIZE);
+      CollectionsPlume.createLruCache(CACHE_SIZE);
 
   /**
    * Defaults that apply for a certain Element. On the one hand this is used for caching (an earlier
@@ -218,8 +223,8 @@ public class QualifierDefaults {
   /** Add standard unchecked defaults that do not conflict with previously added defaults. */
   public void addUncheckedStandardDefaults() {
     QualifierHierarchy qualHierarchy = this.atypeFactory.getQualifierHierarchy();
-    Set<? extends AnnotationMirror> tops = qualHierarchy.getTopAnnotations();
-    Set<? extends AnnotationMirror> bottoms = qualHierarchy.getBottomAnnotations();
+    AnnotationMirrorSet tops = qualHierarchy.getTopAnnotations();
+    AnnotationMirrorSet bottoms = qualHierarchy.getBottomAnnotations();
 
     for (TypeUseLocation loc : STANDARD_UNCHECKED_DEFAULTS_TOP) {
       // Only add standard defaults in locations where a default has not be specified
@@ -243,13 +248,14 @@ public class QualifierDefaults {
   /** Add standard CLIMB defaults that do not conflict with previously added defaults. */
   public void addClimbStandardDefaults() {
     QualifierHierarchy qualHierarchy = this.atypeFactory.getQualifierHierarchy();
-    Set<? extends AnnotationMirror> tops = qualHierarchy.getTopAnnotations();
-    Set<? extends AnnotationMirror> bottoms = qualHierarchy.getBottomAnnotations();
+    AnnotationMirrorSet tops = qualHierarchy.getTopAnnotations();
+    AnnotationMirrorSet bottoms = qualHierarchy.getBottomAnnotations();
 
     for (TypeUseLocation loc : STANDARD_CLIMB_DEFAULTS_TOP) {
       for (AnnotationMirror top : tops) {
         if (!conflictsWithExistingDefaults(checkedCodeDefaults, top, loc)) {
-          // Only add standard defaults in locations where a default has not been specified
+          // Only add standard defaults in locations where a default has not been
+          // specified
           addCheckedCodeDefault(top, loc);
         }
       }
@@ -258,7 +264,8 @@ public class QualifierDefaults {
     for (TypeUseLocation loc : STANDARD_CLIMB_DEFAULTS_BOTTOM) {
       for (AnnotationMirror bottom : bottoms) {
         if (!conflictsWithExistingDefaults(checkedCodeDefaults, bottom, loc)) {
-          // Only add standard defaults in locations where a default has not been specified
+          // Only add standard defaults in locations where a default has not been
+          // specified
           addCheckedCodeDefault(bottom, loc);
         }
       }
@@ -266,8 +273,11 @@ public class QualifierDefaults {
   }
 
   /**
-   * Sets the default annotations. A programmer may override this by writing the @DefaultQualifier
+   * Adds a default annotation. A programmer may override this by writing the @DefaultQualifier
    * annotation on an element.
+   *
+   * @param absoluteDefaultAnno the default annotation mirror
+   * @param location the type use location
    */
   public void addCheckedCodeDefault(
       AnnotationMirror absoluteDefaultAnno, TypeUseLocation location) {
@@ -275,7 +285,12 @@ public class QualifierDefaults {
     checkedCodeDefaults.add(new Default(absoluteDefaultAnno, location));
   }
 
-  /** Sets the default annotation for unchecked elements. */
+  /**
+   * Add a default annotation for unchecked elements.
+   *
+   * @param uncheckedDefaultAnno the default annotation mirror
+   * @param location the type use location
+   */
   public void addUncheckedCodeDefault(
       AnnotationMirror uncheckedDefaultAnno, TypeUseLocation location) {
     checkDuplicates(uncheckedCodeDefaults, uncheckedDefaultAnno, location);
@@ -299,7 +314,13 @@ public class QualifierDefaults {
     }
   }
 
-  /** Sets the default annotations for a certain Element. */
+  /**
+   * Sets the default annotations for a certain Element.
+   *
+   * @param elem the scope to set the default within
+   * @param elementDefaultAnno the default to set
+   * @param location the location to apply the default to
+   */
   public void addElementDefault(
       Element elem, AnnotationMirror elementDefaultAnno, TypeUseLocation location) {
     DefaultSet prevset = elementDefaults.get(elem);
@@ -339,14 +360,22 @@ public class QualifierDefaults {
     }
   }
 
+  /**
+   * Returns true if there are conflicts with existing defaults.
+   *
+   * @param previousDefaults the previous defaults
+   * @param newAnno the new annotation
+   * @param newLoc the location of the type use
+   * @return true if there are conflicts with existing defaults
+   */
   private boolean conflictsWithExistingDefaults(
       DefaultSet previousDefaults, AnnotationMirror newAnno, TypeUseLocation newLoc) {
-    final QualifierHierarchy qualHierarchy = atypeFactory.getQualifierHierarchy();
+    QualifierHierarchy qualHierarchy = atypeFactory.getQualifierHierarchy();
 
     for (Default previous : previousDefaults) {
       if (!AnnotationUtils.areSame(newAnno, previous.anno) && previous.location == newLoc) {
-        final AnnotationMirror previousTop = qualHierarchy.getTopAnnotation(previous.anno);
-        if (qualHierarchy.isSubtype(newAnno, previousTop)) {
+        AnnotationMirror previousTop = qualHierarchy.getTopAnnotation(previous.anno);
+        if (qualHierarchy.isSubtypeQualifiersOnly(newAnno, previousTop)) {
           return true;
         }
       }
@@ -414,7 +443,7 @@ public class QualifierDefaults {
    * @param tree the tree
    * @return the nearest enclosing element for a tree
    */
-  private Element nearestEnclosingExceptLocal(Tree tree) {
+  private @Nullable Element nearestEnclosingExceptLocal(Tree tree) {
     TreePath path = atypeFactory.getPath(tree);
     if (path == null) {
       Element element = atypeFactory.getEnclosingElementForArtificialTree(tree);
@@ -447,10 +476,11 @@ public class QualifierDefaults {
               break;
             }
           }
-          if (prev != null && prev.getKind() == Tree.Kind.MODIFIERS) {
-            // Annotations are modifiers. We do not want to apply the local variable default to
-            // annotations. Without this, test fenum/TestSwitch failed, because the default for an
-            // argument became incompatible with the declared type.
+          if (prev != null && prev instanceof ModifiersTree) {
+            // Annotations are modifiers. We do not want to apply the local variable
+            // default to annotations. Without this, test fenum/TestSwitch failed,
+            // because the default for an argument became incompatible with the declared
+            // type.
             break;
           }
           return TreeUtils.elementFromDeclaration((VariableTree) t);
@@ -495,7 +525,7 @@ public class QualifierDefaults {
       case IDENTIFIER:
         elt = TreeUtils.elementFromUse((IdentifierTree) tree);
         if (ElementUtils.isTypeDeclaration(elt)) {
-          // If the Idenitifer is a type, then use the scope of the tree.
+          // If the identifier is a type, then use the scope of the tree.
           elt = nearestEnclosingExceptLocal(tree);
         }
         break;
@@ -504,9 +534,9 @@ public class QualifierDefaults {
         elt = TreeUtils.elementFromUse((MethodInvocationTree) tree);
         break;
 
-        // TODO cases for array access, etc. -- every expression tree
-        // (The above probably means that we should use defaults in the
-        // scope of the declaration of the array.  Is that right?  -MDE)
+      // TODO cases for array access, etc. -- every expression tree
+      // (The above probably means that we should use defaults in the
+      // scope of the declaration of the array.  Is that right?  -MDE)
 
       default:
         // If no associated symbol was found, use the tree's (lexical) scope.
@@ -523,14 +553,14 @@ public class QualifierDefaults {
     applyToTypeVar =
         defaultTypeVarLocals
             && elt != null
-            && elt.getKind() == ElementKind.LOCAL_VARIABLE
+            && ElementUtils.isLocalVariable(elt)
             && type.getKind() == TypeKind.TYPEVAR;
     applyDefaultsElement(elt, type);
     applyToTypeVar = false;
   }
 
   /** The default {@code value} element for a @DefaultQualifier annotation. */
-  private static TypeUseLocation[] defaultQualifierValueDefault =
+  private static final TypeUseLocation[] defaultQualifierValueDefault =
       new TypeUseLocation[] {org.checkerframework.framework.qual.TypeUseLocation.ALL};
 
   /**
@@ -539,7 +569,7 @@ public class QualifierDefaults {
    * @param dq a @DefaultQualifier annotation
    * @return a DefaultSet corresponding to the @DefaultQualifier annotation
    */
-  private DefaultSet fromDefaultQualifier(AnnotationMirror dq) {
+  private @Nullable DefaultSet fromDefaultQualifier(AnnotationMirror dq) {
     @SuppressWarnings("unchecked")
     Name cls = AnnotationUtils.getElementValueClassName(dq, defaultQualifierValueElement);
     AnnotationMirror anno = AnnotationBuilder.fromName(elements, cls);
@@ -569,7 +599,7 @@ public class QualifierDefaults {
     }
   }
 
-  private boolean isElementAnnotatedForThisChecker(final Element elt) {
+  private boolean isElementAnnotatedForThisChecker(Element elt) {
     boolean elementAnnotatedForThisChecker = false;
 
     if (elt == null) {
@@ -580,7 +610,7 @@ public class QualifierDefaults {
       return elementAnnotatedFors.get(elt);
     }
 
-    final AnnotationMirror annotatedFor = atypeFactory.getDeclAnnotation(elt, AnnotatedFor.class);
+    AnnotationMirror annotatedFor = atypeFactory.getDeclAnnotation(elt, AnnotatedFor.class);
 
     if (annotatedFor != null) {
       elementAnnotatedForThisChecker =
@@ -602,12 +632,23 @@ public class QualifierDefaults {
       }
     }
 
-    elementAnnotatedFors.put(elt, elementAnnotatedForThisChecker);
+    if (atypeFactory.shouldCache
+        && !atypeFactory.stubTypes.isParsing()
+        && !atypeFactory.ajavaTypes.isParsing()) {
+      elementAnnotatedFors.put(elt, elementAnnotatedForThisChecker);
+    }
 
     return elementAnnotatedForThisChecker;
   }
 
-  private DefaultSet defaultsAt(final Element elt) {
+  /**
+   * Returns the defaults that apply to the given Element, considering defaults from enclosing
+   * Elements.
+   *
+   * @param elt the element
+   * @return the defaults
+   */
+  private DefaultSet defaultsAt(Element elt) {
     if (elt == null) {
       return DefaultSet.EMPTY;
     }
@@ -638,10 +679,10 @@ public class QualifierDefaults {
         if (qualifiers == null) {
           qualifiers = new DefaultSet();
         }
-        @SuppressWarnings("unchecked")
+
         List<AnnotationMirror> values =
-            AnnotationUtils.getElementValue(
-                dqListAnno, defaultQualifierListValueElement, List.class);
+            AnnotationUtils.getElementValueArray(
+                dqListAnno, defaultQualifierListValueElement, AnnotationMirror.class);
         for (AnnotationMirror dqAnno : values) {
           Set<Default> p = fromDefaultQualifier(dqAnno);
           if (p != null) {
@@ -680,7 +721,7 @@ public class QualifierDefaults {
    * @param annotationScope the element that the conservative default might apply to
    * @return whether the conservative default applies to the given element
    */
-  public boolean applyConservativeDefaults(final Element annotationScope) {
+  public boolean applyConservativeDefaults(Element annotationScope) {
     if (annotationScope == null) {
       return false;
     }
@@ -704,7 +745,7 @@ public class QualifierDefaults {
       return useConservativeDefaultsBytecode && !isElementAnnotatedForThisChecker(annotationScope);
     } else if (isFromStubFile) {
       // TODO: Types in stub files not annotated for a particular checker should be
-      // treated as unchecked bytecode.   For now, all types in stub files are treated as
+      // treated as unchecked bytecode.  For now, all types in stub files are treated as
       // checked code. Eventually, @AnnotateFor(checker) will be programmatically added
       // to methods in stub files supplied via the @Stubfile annotation.  Stub files will
       // be treated like unchecked code except for methods in the scope for an @AnnotatedFor.
@@ -729,7 +770,7 @@ public class QualifierDefaults {
    *     inference)
    * @checker_framework.manual #annotating-libraries Annotating libraries
    */
-  private void applyDefaultsElement(final Element annotationScope, final AnnotatedTypeMirror type) {
+  private void applyDefaultsElement(Element annotationScope, AnnotatedTypeMirror type) {
     DefaultSet defaults = defaultsAt(annotationScope);
     DefaultApplierElement applier =
         createDefaultApplierElement(atypeFactory, annotationScope, type, applyToTypeVar);
@@ -817,8 +858,7 @@ public class QualifierDefaults {
      * @param type type to which qual would be applied
      * @return true if this application should proceed
      */
-    protected boolean shouldBeAnnotated(
-        final AnnotatedTypeMirror type, final boolean applyToTypeVar) {
+    protected boolean shouldBeAnnotated(AnnotatedTypeMirror type, boolean applyToTypeVar) {
 
       return !(type == null
           // TODO: executables themselves should not be annotated
@@ -838,10 +878,9 @@ public class QualifierDefaults {
      * @param qual annotation to add
      */
     protected void addAnnotation(AnnotatedTypeMirror type, AnnotationMirror qual) {
-      // Add the default annotation, but only if no other
-      // annotation is present.
-      if (!type.isAnnotatedInHierarchy(qual) && type.getKind() != TypeKind.EXECUTABLE) {
-        type.addAnnotation(qual);
+      // Add the default annotation, but only if no other annotation is present.
+      if (type.getKind() != TypeKind.EXECUTABLE) {
+        type.addMissingAnnotation(qual);
       }
     }
 
@@ -918,8 +957,7 @@ public class QualifierDefaults {
                 && t.getKind() == TypeKind.EXECUTABLE
                 && isTopLevelType) {
 
-              final AnnotatedDeclaredType receiver =
-                  ((AnnotatedExecutableType) t).getReceiverType();
+              AnnotatedDeclaredType receiver = ((AnnotatedExecutableType) t).getReceiverType();
               if (shouldBeAnnotated(receiver, false)) {
                 addAnnotation(receiver, qual);
               }
@@ -930,7 +968,7 @@ public class QualifierDefaults {
                 && scope.getKind() == ElementKind.METHOD
                 && t.getKind() == TypeKind.EXECUTABLE
                 && isTopLevelType) {
-              final AnnotatedTypeMirror returnType = ((AnnotatedExecutableType) t).getReturnType();
+              AnnotatedTypeMirror returnType = ((AnnotatedExecutableType) t).getReturnType();
               if (shouldBeAnnotated(returnType, false)) {
                 addAnnotation(returnType, qual);
               }
@@ -941,9 +979,9 @@ public class QualifierDefaults {
                 && scope.getKind() == ElementKind.CONSTRUCTOR
                 && t.getKind() == TypeKind.EXECUTABLE
                 && isTopLevelType) {
-              // This is the return type of a constructor declaration (not a constructor
-              // invocation).
-              final AnnotatedTypeMirror returnType = ((AnnotatedExecutableType) t).getReturnType();
+              // This is the return type of a constructor declaration (not a
+              // constructor invocation).
+              AnnotatedTypeMirror returnType = ((AnnotatedExecutableType) t).getReturnType();
               if (shouldBeAnnotated(returnType, false)) {
                 addAnnotation(returnType, qual);
               }
@@ -1039,9 +1077,9 @@ public class QualifierDefaults {
           AnnotatedTypeMirror lowerBound,
           AnnotationMirror qual) {
 
-        final boolean prevIsUpperBound = isUpperBound;
-        final boolean prevIsLowerBound = isLowerBound;
-        final BoundType prevBoundType = boundType;
+        boolean prevIsUpperBound = isUpperBound;
+        boolean prevIsLowerBound = isLowerBound;
+        BoundType prevBoundType = boundType;
 
         boundType = getBoundType(boundedType);
 
@@ -1086,8 +1124,8 @@ public class QualifierDefaults {
      */
     UNBOUNDED;
 
-    public boolean isOneOf(final BoundType... choices) {
-      for (final BoundType choice : choices) {
+    public boolean isOneOf(BoundType... choices) {
+      for (BoundType choice : choices) {
         if (this == choice) {
           return true;
         }
@@ -1104,7 +1142,7 @@ public class QualifierDefaults {
    *     AnnotatedTypeVariable.
    * @return the boundType for type
    */
-  private BoundType getBoundType(final AnnotatedTypeMirror type) {
+  private BoundType getBoundType(AnnotatedTypeMirror type) {
     if (type instanceof AnnotatedTypeVariable) {
       return getTypeVarBoundType((AnnotatedTypeVariable) type);
     }
@@ -1122,7 +1160,7 @@ public class QualifierDefaults {
    * @param typeVar the type variable
    * @return the bound type of the input typeVar
    */
-  private BoundType getTypeVarBoundType(final AnnotatedTypeVariable typeVar) {
+  private BoundType getTypeVarBoundType(AnnotatedTypeVariable typeVar) {
     return getTypeVarBoundType((TypeParameterElement) typeVar.getUnderlyingType().asElement());
   }
 
@@ -1133,8 +1171,8 @@ public class QualifierDefaults {
    * @return the boundType (UPPER or UNBOUNDED) of the declaration of typeParamElem
    */
   // Results are cached in {@link elementToBoundType}.
-  private BoundType getTypeVarBoundType(final TypeParameterElement typeParamElem) {
-    final BoundType prev = elementToBoundType.get(typeParamElem);
+  private BoundType getTypeVarBoundType(TypeParameterElement typeParamElem) {
+    BoundType prev = elementToBoundType.get(typeParamElem);
     if (prev != null) {
       return prev;
     }
@@ -1158,8 +1196,8 @@ public class QualifierDefaults {
       }
 
     } else {
-      if (typeParamDecl.getKind() == Tree.Kind.TYPE_PARAMETER) {
-        final TypeParameterTree tptree = (TypeParameterTree) typeParamDecl;
+      if (typeParamDecl instanceof TypeParameterTree) {
+        TypeParameterTree tptree = (TypeParameterTree) typeParamDecl;
 
         List<? extends Tree> bnds = tptree.getBounds();
         if (bnds != null && !bnds.isEmpty()) {
@@ -1184,24 +1222,22 @@ public class QualifierDefaults {
    * Returns the BoundType of annotatedWildcard. If it is unbounded, use the type parameter to which
    * its an argument.
    *
-   * @param annotatedWildcard the annotated wildcard type
+   * @param wildcardType the annotated wildcard type
    * @return the BoundType of annotatedWildcard. If it is unbounded, use the type parameter to which
    *     its an argument
    */
-  public BoundType getWildcardBoundType(final AnnotatedWildcardType annotatedWildcard) {
-
-    final WildcardType wildcard = (WildcardType) annotatedWildcard.getUnderlyingType();
-
-    final BoundType boundType;
-    if (wildcard.isUnbound() && wildcard.bound != null) {
-      boundType = getTypeVarBoundType((TypeParameterElement) wildcard.bound.asElement());
-
+  public BoundType getWildcardBoundType(AnnotatedWildcardType wildcardType) {
+    if (AnnotatedTypes.hasNoExplicitBound(wildcardType)) {
+      TypeParameterElement e = TypesUtils.wildcardToTypeParam(wildcardType.getUnderlyingType());
+      if (e != null) {
+        return getTypeVarBoundType(e);
+      } else {
+        return BoundType.UNBOUNDED;
+      }
+    } else if (AnnotatedTypes.hasExplicitSuperBound(wildcardType)) {
+      return BoundType.LOWER;
     } else {
-      // note: isSuperBound will be true for unbounded and lowers, but the unbounded case is
-      // already handled
-      boundType = wildcard.isSuperBound() ? BoundType.LOWER : BoundType.UPPER;
+      return BoundType.UPPER;
     }
-
-    return boundType;
   }
 }
