@@ -33,10 +33,13 @@ import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
 import org.checkerframework.checker.nullness.qual.Nullable;
+import org.checkerframework.dataflow.cfg.node.Node;
 import org.checkerframework.dataflow.expression.FormalParameter;
 import org.checkerframework.dataflow.expression.JavaExpression;
 import org.checkerframework.dataflow.expression.JavaExpressionConverter;
 import org.checkerframework.dataflow.expression.LocalVariable;
+import org.checkerframework.dataflow.expression.SuperReference;
+import org.checkerframework.dataflow.expression.ThisReference;
 import org.checkerframework.dataflow.expression.Unknown;
 import org.checkerframework.framework.source.SourceChecker;
 import org.checkerframework.framework.type.AnnotatedTypeFactory;
@@ -44,6 +47,7 @@ import org.checkerframework.framework.type.AnnotatedTypeMirror;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedDeclaredType;
 import org.checkerframework.framework.type.AnnotatedTypeMirror.AnnotatedExecutableType;
 import org.checkerframework.framework.type.AnnotatedTypeParameterBounds;
+import org.checkerframework.framework.type.QualifierHierarchy;
 import org.checkerframework.framework.type.treeannotator.TreeAnnotator;
 import org.checkerframework.framework.type.visitor.AnnotatedTypeScanner;
 import org.checkerframework.framework.type.visitor.DoubleAnnotatedTypeScanner;
@@ -51,6 +55,7 @@ import org.checkerframework.framework.type.visitor.SimpleAnnotatedTypeScanner;
 import org.checkerframework.framework.util.JavaExpressionParseUtil.JavaExpressionParseException;
 import org.checkerframework.framework.util.StringToJavaExpression;
 import org.checkerframework.javacutil.AnnotationBuilder;
+import org.checkerframework.javacutil.AnnotationMirrorSet;
 import org.checkerframework.javacutil.AnnotationUtils;
 import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.ElementUtils;
@@ -66,7 +71,8 @@ import org.plumelib.util.CollectionsPlume;
  * <ol>
  *   <li>Converts the expression strings in an {@link AnnotationMirror} {@code am}, by creating a
  *       new annotation whose Java expression elements are the result of the conversion. See {@link
- *       #convertAnnotationMirror(StringToJavaExpression, AnnotationMirror)}. Subclasses can
+ *       #convertAnnotationMirror(StringToJavaExpression, AnnotationMirror)}, though clients do not
+ *       call it (they call other methods in this class, which eventually call it). Subclasses can
  *       specialize this process by overriding methods in this class. Methods in this class always
  *       standardize Java expressions and may additionally viewpoint-adapt or delocalize
  *       expressions. Below is an explanation of each kind of conversion.
@@ -76,9 +82,9 @@ import org.plumelib.util.CollectionsPlume;
  *             field f may appear in an expression string as "f" or "this.f"; this class
  *             standardizes both strings to "this.f". All dependent type annotations must be
  *             standardized so that the implementation of {@link
- *             org.checkerframework.framework.type.QualifierHierarchy#isSubtype(AnnotationMirror,
- *             AnnotationMirror)} can assume that two expressions are equivalent if their string
- *             representations are {@code equals()}.
+ *             org.checkerframework.framework.type.QualifierHierarchy#isSubtypeShallow(AnnotationMirror,
+ *             TypeMirror, AnnotationMirror, TypeMirror)} can assume that two expressions are
+ *             equivalent if their string representations are {@code equals()}.
  *         <li>Viewpoint-adaption: converts an expression to some use site. For example, in method
  *             bodies, formal parameter references such as "#2" are converted to the name of the
  *             formal parameter. Another example, is at method call site, "this" is converted to the
@@ -114,6 +120,12 @@ public class DependentTypesHelper {
   /** This scans an annotated type and returns a list of {@link DependentTypesError}. */
   private final ExpressionErrorCollector expressionErrorCollector = new ExpressionErrorCollector();
 
+  /**
+   * This scans the annotated type and replaces any dependent type annotation that has a parse error
+   * with the top annotation in the hierarchy.
+   */
+  protected final ErrorAnnoReplacer errorAnnoReplacer;
+
   /**
    * A scanner that applies a function to each {@link AnnotationMirror} and replaces it in the given
    * {@code AnnotatedTypeMirror}. (This side-effects the {@code AnnotatedTypeMirror}.)
@@ -124,7 +136,7 @@ public class DependentTypesHelper {
    * Copies annotations that might have been viewpoint adapted from the visited type (the first
    * formal parameter of {@code ViewpointAdaptedCopier#visit}) to the second formal parameter.
    */
-  private final ViewpointAdaptedCopier viewpointAdaptedCopier = new ViewpointAdaptedCopier();
+  protected final ViewpointAdaptedCopier viewpointAdaptedCopier = new ViewpointAdaptedCopier();
 
   /** The type mirror for java.lang.Object. */
   protected final TypeMirror objectTM;
@@ -136,7 +148,7 @@ public class DependentTypesHelper {
    */
   public DependentTypesHelper(AnnotatedTypeFactory factory) {
     this.factory = factory;
-
+    this.errorAnnoReplacer = new ErrorAnnoReplacer(factory.getQualifierHierarchy());
     this.annoToElements = new HashMap<>();
     for (Class<? extends Annotation> expressionAnno : factory.getSupportedTypeQualifiers()) {
       List<ExecutableElement> elementList =
@@ -188,7 +200,7 @@ public class DependentTypesHelper {
   /**
    * Returns the elements of the annotation that are Java expressions.
    *
-   * @param am AnnotationMirror
+   * @param am an annotation
    * @return the elements of the annotation that are Java expressions
    */
   private List<ExecutableElement> getListOfExpressionElements(AnnotationMirror am) {
@@ -205,12 +217,13 @@ public class DependentTypesHelper {
     return new DependentTypesTreeAnnotator(factory, this);
   }
 
-  ///
-  /// Methods that convert annotations
-  ///
+  //
+  // Methods that convert annotations
+  //
 
   /** If true, log information about where lambdas are created. */
-  private static boolean debugStringToJavaExpression = false;
+  // This variable is only set here; edit the source code to modify it.
+  private static final boolean debugStringToJavaExpression = false;
 
   /**
    * Viewpoint-adapts the dependent type annotations on the bounds of the type parameters of the
@@ -273,7 +286,7 @@ public class DependentTypesHelper {
   }
 
   /**
-   * Viewpoint-adapts a method or constructor invocation.
+   * Viewpoint-adapts dependent type annotations in a method or constructor type.
    *
    * <p>{@code methodType} has been viewpoint-adapted to the call site, except for any dependent
    * type annotations. (For example, type variables have been substituted and polymorphic qualifiers
@@ -440,7 +453,7 @@ public class DependentTypesHelper {
         }
         Tree enclTree = pathTillEnclTree.getLeaf();
 
-        if (enclTree.getKind() == Tree.Kind.METHOD) {
+        if (enclTree instanceof MethodTree) {
           MethodTree methodDeclTree = (MethodTree) enclTree;
           StringToJavaExpression stringToJavaExpr =
               stringExpr ->
@@ -570,13 +583,15 @@ public class DependentTypesHelper {
           if (elt.getKind() == ElementKind.PARAMETER) {
             // The tree might be null when
             // org.checkerframework.framework.flow.CFAbstractTransfer.getValueFromFactory() gets the
-            // assignment context for a pseudo assignment of an argument to a method parameter.
+            // assignment context for a pseudo assignment of an argument to a method
+            // parameter.
             return;
           }
           throw new BugInCF(this.getClass() + ": tree not found");
         } else if (TreeUtils.typeOf(declarationTree) == null) {
-          // org.checkerframework.framework.flow.CFAbstractTransfer.getValueFromFactory() gets the
-          // assignment context for a pseudo assignment of an argument to a method parameter.
+          // org.checkerframework.framework.flow.CFAbstractTransfer.getValueFromFactory()
+          // gets the assignment context for a pseudo assignment of an argument to a
+          // method parameter.
           return;
         }
 
@@ -592,7 +607,10 @@ public class DependentTypesHelper {
 
   /** Thrown when a non-parameter local variable is found. */
   @SuppressWarnings("serial")
-  private static class FoundLocalException extends RuntimeException {}
+  private static class FoundLocalVarException extends RuntimeException {
+    /** Creates a FoundLocalVarException. */
+    public FoundLocalVarException() {}
+  }
 
   /**
    * Viewpoint-adapt all dependent type annotations to the method declaration, {@code
@@ -635,14 +653,14 @@ public class DependentTypesHelper {
                     LocalVariable localVarExpr, Void unused) {
                   int index = paramsAsLocals.indexOf(localVarExpr);
                   if (index == -1) {
-                    throw new FoundLocalException();
+                    throw new FoundLocalVarException();
                   }
                   return parameters.get(index);
                 }
               };
           try {
             return jec.convert(javaExpr);
-          } catch (FoundLocalException ex) {
+          } catch (FoundLocalVarException ex) {
             return null;
           }
         };
@@ -654,6 +672,89 @@ public class DependentTypesHelper {
     convertAnnotatedTypeMirror(stringToJavaExpr, atm);
   }
 
+  /**
+   * Delocalizes dependent type annotations in {@code atm} so that they can be placed on the
+   * declaration of the given method or constructor being invoked. Used by whole program inference
+   * to infer dependent types for method/constructor parameters based on the actual arguments used
+   * at call sites.
+   *
+   * @param atm the annotated type mirror to delocalize
+   * @param invocationTree the method or constructor invocation
+   * @param arguments the actual arguments to the method or constructor
+   * @param receiver the actual receiver, if there was one; null if not
+   * @param methodElt the declaration of the method or constructor being invoked
+   */
+  public void delocalizeAtCallsite(
+      AnnotatedTypeMirror atm,
+      Tree invocationTree,
+      List<Node> arguments,
+      @Nullable Node receiver,
+      ExecutableElement methodElt) {
+
+    // TODO: this method should also take the receiver parameter, if there was one at the
+    // callsite, as an argument. Before it does, WPI needs to infer receiver types from
+    // callsites.
+
+    if (!hasDependentType(atm)) {
+      return;
+    }
+
+    // For use in stringToJavaExpr below, to avoid re-computation. Especially
+    // important for the TreePath, which is expensive to compute.
+    List<JavaExpression> argsAsExprs = CollectionsPlume.mapList(LocalVariable::fromNode, arguments);
+    JavaExpression receiverAsExpr = receiver == null ? null : LocalVariable.fromNode(receiver);
+    TreePath path = factory.getPath(invocationTree);
+
+    StringToJavaExpression stringToJavaExpr =
+        stringExpr -> {
+          JavaExpression expr =
+              StringToJavaExpression.atPath(stringExpr, path, factory.getChecker());
+          JavaExpressionConverter jec =
+              new JavaExpressionConverter() {
+                @Override
+                public JavaExpression convert(JavaExpression javaExpr) {
+                  // if javaExpr is an argument to the method,
+                  // then return formal parameter expression.
+                  int index = argsAsExprs.indexOf(javaExpr);
+                  if (index != -1) {
+                    return FormalParameter.getFormalParameters(methodElt).get(index);
+                  }
+                  if (javaExpr.equals(receiverAsExpr)) {
+                    return new ThisReference(ElementUtils.enclosingTypeElement(methodElt).asType());
+                  }
+                  return super.convert(javaExpr);
+                }
+
+                // Local variables and this references at the call site that do not
+                // correspond to any parameter need to be removed from the dependent
+                // type annotation, which returning null from these methods
+                // accomplishes.
+                @Override
+                public JavaExpression visitLocalVariable(LocalVariable local, Void unused) {
+                  throw new FoundLocalVarException();
+                }
+
+                @Override
+                public JavaExpression visitThisReference(ThisReference thisRef, Void unused) {
+                  throw new FoundLocalVarException();
+                }
+
+                @Override
+                public JavaExpression visitSuperReference(SuperReference superRef, Void unused) {
+                  throw new FoundLocalVarException();
+                }
+              };
+
+          try {
+            return jec.convert(expr);
+          } catch (FoundLocalVarException ex) {
+            return null;
+          }
+        };
+
+    convertAnnotatedTypeMirror(stringToJavaExpr, atm);
+  }
+
   /**
    * Calls {@link #convertAnnotationMirror(StringToJavaExpression, AnnotationMirror)} on each
    * annotation mirror on type with {@code stringToJavaExpr}. And replaces the annotation with the
@@ -732,7 +833,7 @@ public class DependentTypesHelper {
   /**
    * This method is for subclasses to override to change JavaExpressions in some way before they are
    * inserted into new annotations. This method is called after parsing and viewpoint-adaptation
-   * have occurred. {@code javaExpr} may be a {@link PassThroughExpression}.
+   * have occurred. {@code javaExpr} may be a {@link DependentTypesHelper.PassThroughExpression}.
    *
    * <p>If {@code null} is returned then the expression is not added to the new annotation.
    *
@@ -851,17 +952,18 @@ public class DependentTypesHelper {
       }
       visitedNodes.put(type, null);
 
-      // If the type variable has a primary annotation, then it is viewpoint-adapted before this
-      // method is called.  The viewpoint-adapted primary annotation was already copied to the upper
-      // and lower bounds.  These annotations cannot be viewpoint-adapted again, so remove them,
-      // viewpoint-adapt any other annotations in the bound, and then add them back.
-      Set<AnnotationMirror> primarys = type.getAnnotations();
-      type.getLowerBound().removeAnnotations(primarys);
+      // If the type variable has a primary annotation, then it is viewpoint-adapted before
+      // this method is called.  The viewpoint-adapted primary annotation was already copied
+      // to the upper and lower bounds.  These annotations cannot be viewpoint-adapted again,
+      // so remove them, viewpoint-adapt any other annotations in the bound, and then add them
+      // back.
+      AnnotationMirrorSet primarys = type.getPrimaryAnnotations();
+      type.getLowerBound().removePrimaryAnnotations(primarys);
       Void r = scan(type.getLowerBound(), func);
       type.getLowerBound().addAnnotations(primarys);
       visitedNodes.put(type, r);
 
-      type.getUpperBound().removeAnnotations(primarys);
+      type.getUpperBound().removePrimaryAnnotations(primarys);
       r = scanAndReduce(type.getUpperBound(), func, r);
       type.getUpperBound().addAnnotations(primarys);
       visitedNodes.put(type, r);
@@ -871,16 +973,19 @@ public class DependentTypesHelper {
     @Override
     protected Void scan(
         AnnotatedTypeMirror type, Function<AnnotationMirror, AnnotationMirror> func) {
-      for (AnnotationMirror anno : AnnotationUtils.createAnnotationSet(type.getAnnotations())) {
+      if (visitedNodes.containsKey(type)) {
+        return null;
+      }
+      for (AnnotationMirror anno : new AnnotationMirrorSet(type.getPrimaryAnnotations())) {
         AnnotationMirror newAnno = func.apply(anno);
         if (newAnno != null) {
           // This code must remove and then add, rather than call `replace`, because a
           // type may have multiple annotations with the same class, but different
           // elements.  (This is a bug; see
-          // https://github.com/typetools/checker-framework/issues/4451.)
+          // https://github.com/typetools/checker-framework/issues/4451 .)
           // AnnotatedTypeMirror#replace only removes one annotation that is in the same
           // hierarchy as the passed argument.
-          type.removeAnnotation(anno);
+          type.removePrimaryAnnotation(anno);
           type.addAnnotation(newAnno);
         }
       }
@@ -888,9 +993,9 @@ public class DependentTypesHelper {
     }
   }
 
-  ///
-  /// Methods that check and report errors
-  ///
+  //
+  // Methods that check and report errors
+  //
 
   /**
    * Reports an expression.unparsable error for each Java expression in the given type that is an
@@ -910,7 +1015,7 @@ public class DependentTypesHelper {
     }
 
     // Report the error at the type rather than at the variable.
-    if (errorTree.getKind() == Tree.Kind.VARIABLE) {
+    if (errorTree instanceof VariableTree) {
       Tree typeTree = ((VariableTree) errorTree).getType();
       // Don't report the error at the type if the type is not present in source code.
       if (((JCTree) typeTree).getPreferredPosition() != -1) {
@@ -919,8 +1024,8 @@ public class DependentTypesHelper {
         for (AnnotationTree annoTree : modifiers.getAnnotations()) {
           String annoString = annoTree.toString();
           for (String annoName : annoToElements.keySet()) {
-            // TODO: Simple string containment seems too simplistic.  At least check for a word
-            // boundary.
+            // TODO: Simple string containment seems too simplistic.  At least check for
+            // a word boundary.
             if (annoString.contains(annoName)) {
               errorTree = annoTree;
               break;
@@ -1041,22 +1146,22 @@ public class DependentTypesHelper {
    * Reports an expression.unparsable error for each Java expression in the given type variables
    * that is an expression error string.
    *
-   * @param node a method declaration
+   * @param tree a method declaration
    * @param methodType annotated type of the method
    */
   private void checkTypeVariablesForErrorExpressions(
-      MethodTree node, AnnotatedExecutableType methodType) {
+      MethodTree tree, AnnotatedExecutableType methodType) {
     for (int i = 0; i < methodType.getTypeVariables().size(); i++) {
       AnnotatedTypeMirror atm = methodType.getTypeVariables().get(i);
       StringToJavaExpression stringToJavaExpr =
-          stringExpr -> StringToJavaExpression.atMethodBody(stringExpr, node, factory.getChecker());
+          stringExpr -> StringToJavaExpression.atMethodBody(stringExpr, tree, factory.getChecker());
       if (debugStringToJavaExpression) {
         System.out.printf(
             "checkTypeVariablesForErrorExpressions(%s, %s) created %s%n",
-            node, methodType, stringToJavaExpr);
+            tree, methodType, stringToJavaExpr);
       }
       convertAnnotatedTypeMirror(stringToJavaExpr, atm);
-      checkTypeForErrorExpressions(atm, node.getTypeParameters().get(i));
+      checkTypeForErrorExpressions(atm, tree.getTypeParameters().get(i));
     }
   }
 
@@ -1088,7 +1193,7 @@ public class DependentTypesHelper {
       super(
           (AnnotatedTypeMirror type, Void aVoid) -> {
             List<DependentTypesError> errors = new ArrayList<>();
-            for (AnnotationMirror am : type.getAnnotations()) {
+            for (AnnotationMirror am : type.getPrimaryAnnotations()) {
               if (isExpressionAnno(am)) {
                 errors.addAll(errorElements(am));
               }
@@ -1100,6 +1205,38 @@ public class DependentTypesHelper {
     }
   }
 
+  /**
+   * Replaces a dependent type annotation with a parser error with the top qualifier in the
+   * hierarchy.
+   */
+  protected class ErrorAnnoReplacer extends SimpleAnnotatedTypeScanner<Void, Void> {
+
+    /**
+     * Create an ErrorAnnoReplacer.
+     *
+     * @param qh the qualifier hierarchy
+     */
+    private ErrorAnnoReplacer(QualifierHierarchy qh) {
+      super(
+          (AnnotatedTypeMirror type, Void aVoid) -> {
+            AnnotationMirrorSet replacementAnnos = null;
+            for (AnnotationMirror am : type.getPrimaryAnnotations()) {
+              if (isExpressionAnno(am) && !errorElements(am).isEmpty()) {
+                if (replacementAnnos == null) {
+                  replacementAnnos = new AnnotationMirrorSet();
+                }
+                replacementAnnos.add(qh.getTopAnnotation(am));
+              }
+            }
+
+            if (replacementAnnos != null) {
+              type.replaceAnnotations(replacementAnnos);
+            }
+            return null;
+          });
+    }
+  }
+
   /**
    * Appends list2 to list1 in a new list. If either list is empty, returns the other. Thus, the
    * result may be aliased to one of the arguments and the client should only read, not write into,
@@ -1128,21 +1265,26 @@ public class DependentTypesHelper {
    * visited type to the second formal parameter except for annotations on types that have been
    * substituted.
    */
-  private class ViewpointAdaptedCopier extends DoubleAnnotatedTypeScanner<Void> {
+  protected class ViewpointAdaptedCopier extends DoubleAnnotatedTypeScanner<Void> {
+
+    /** Create a ViewpointAdaptedCopier. */
+    private ViewpointAdaptedCopier() {}
+
     @Override
     protected Void scan(AnnotatedTypeMirror from, AnnotatedTypeMirror to) {
       if (from == null || to == null) {
         return null;
       }
-      Set<AnnotationMirror> replacements = AnnotationUtils.createAnnotationSet();
+      AnnotationMirrorSet replacements = new AnnotationMirrorSet();
       for (String vpa : annoToElements.keySet()) {
-        AnnotationMirror anno = from.getAnnotation(vpa);
+        AnnotationMirror anno = from.getPrimaryAnnotation(vpa);
         if (anno != null) {
           // Only replace annotations that might have been changed.
           replacements.add(anno);
         }
       }
       to.replaceAnnotations(replacements);
+
       if (from.getKind() != to.getKind()
           || (from.getKind() == TypeKind.TYPEVAR
               && TypesUtils.isCapturedTypeVariable(to.getUnderlyingType()))) {
@@ -1179,7 +1321,7 @@ public class DependentTypesHelper {
    * @param atm a type
    * @return true if {@code atm} has any dependent type annotations
    */
-  private boolean hasDependentType(AnnotatedTypeMirror atm) {
+  protected boolean hasDependentType(AnnotatedTypeMirror atm) {
     if (atm == null) {
       return false;
     }
@@ -1192,10 +1334,11 @@ public class DependentTypesHelper {
   }
 
   /** Returns true if the passed AnnotatedTypeMirror has any dependent type annotations. */
+  @SuppressWarnings("this-escape")
   private final AnnotatedTypeScanner<Boolean, Void> hasDependentTypeScanner =
       new SimpleAnnotatedTypeScanner<>(
           (type, __) -> {
-            for (AnnotationMirror annotationMirror : type.getAnnotations()) {
+            for (AnnotationMirror annotationMirror : type.getPrimaryAnnotations()) {
               if (isExpressionAnno(annotationMirror)) {
                 return true;
               }
