@@ -164,7 +164,7 @@ public class InferenceFactory {
         HashSet<Kind> kinds =
             new HashSet<>(Arrays.asList(Tree.Kind.LAMBDA_EXPRESSION, Tree.Kind.METHOD));
         Tree enclosing = TreePathUtil.enclosingOfKind(path, kinds);
-        if (enclosing.getKind() == Tree.Kind.METHOD) {
+        if (enclosing instanceof MethodTree) {
           MethodTree methodTree = (MethodTree) enclosing;
           AnnotatedTypeMirror res = factory.getMethodReturnType(methodTree);
           return new ProperType(res, TreeUtils.typeOf(methodTree.getReturnType()), this.context);
@@ -324,7 +324,7 @@ public class InferenceFactory {
     argumentTree = TreeUtils.withoutParens(argumentTree);
     if (argumentTree == path.getLeaf()) {
       return true;
-    } else if (argumentTree.getKind() == Tree.Kind.CONDITIONAL_EXPRESSION) {
+    } else if (argumentTree instanceof ConditionalExpressionTree) {
       ConditionalExpressionTree conditionalExpressionTree =
           (ConditionalExpressionTree) argumentTree;
       return isArgument(path, conditionalExpressionTree.getTrueExpression())
@@ -349,7 +349,7 @@ public class InferenceFactory {
    */
   private static @Nullable DeclaredType getReceiverType(ExpressionTree tree) {
     Tree receiverTree;
-    if (tree.getKind() == Tree.Kind.NEW_CLASS) {
+    if (tree instanceof NewClassTree) {
       receiverTree = ((NewClassTree) tree).getEnclosingExpression();
       if (receiverTree == null && ((NewClassTree) tree).getClassBody() == null) {
         TypeMirror t = TreeUtils.elementFromUse((NewClassTree) tree).getReceiverType();
@@ -381,15 +381,14 @@ public class InferenceFactory {
    */
   public static ExecutableType getTypeOfMethodAdaptedToUse(
       ExpressionTree expressionTree, Java8InferenceContext context) {
-    assert expressionTree.getKind() == Kind.NEW_CLASS
-        || expressionTree.getKind() == Kind.METHOD_INVOCATION;
+    assert expressionTree instanceof NewClassTree || expressionTree instanceof MethodInvocationTree;
 
     ExecutableElement ele = (ExecutableElement) TreeUtils.elementFromUse(expressionTree);
     ExecutableType executableType = null;
     // First adapt to receiver
     if (!ElementUtils.isStatic(ele)) {
       DeclaredType receiverType = getReceiverType(expressionTree);
-      if (receiverType == null && expressionTree.getKind() == Kind.METHOD_INVOCATION) {
+      if (receiverType == null && expressionTree instanceof MethodInvocationTree) {
         receiverType = context.enclosingType;
       } else if (receiverType != null) {
         receiverType = (DeclaredType) context.types.capture((Type) receiverType);
@@ -400,7 +399,7 @@ public class InferenceFactory {
               == null) {
         TypeMirror enclosing = receiverType.getEnclosingType();
         if (enclosing == null || enclosing.getKind() != TypeKind.DECLARED) {
-          if (expressionTree.getKind() == Tree.Kind.NEW_CLASS) {
+          if (expressionTree instanceof NewClassTree) {
             // No receiver for the constructor.
             executableType = (ExecutableType) ele.asType();
           } else {
@@ -421,8 +420,7 @@ public class InferenceFactory {
     }
 
     // Adapt to class type arguments.
-    if (expressionTree.getKind() == Tree.Kind.NEW_CLASS
-        && !TreeUtils.isDiamondTree(expressionTree)) {
+    if (expressionTree instanceof NewClassTree && !TreeUtils.isDiamondTree(expressionTree)) {
       NewClassTree newClassTree = (NewClassTree) expressionTree;
       List<? extends Tree> typeArgs = TreeUtils.getTypeArgumentsToNewClassTree(newClassTree);
       if (!typeArgs.isEmpty()) {
@@ -446,7 +444,7 @@ public class InferenceFactory {
     }
     // Adapt to explicit method type arguments.
     List<? extends Tree> typeArgs;
-    if (expressionTree.getKind() == Kind.METHOD_INVOCATION) {
+    if (expressionTree instanceof MethodInvocationTree) {
       typeArgs = ((MethodInvocationTree) expressionTree).getTypeArguments();
     } else {
       typeArgs = ((NewClassTree) expressionTree).getTypeArguments();
@@ -708,7 +706,7 @@ public class InferenceFactory {
    */
   public InvocationType getTypeOfMethodAdaptedToUse(ExpressionTree invocation) {
     AnnotatedExecutableType executableType;
-    if (invocation.getKind() == Kind.METHOD_INVOCATION) {
+    if (invocation instanceof MethodInvocationTree) {
       executableType =
           typeFactory.methodFromUseWithoutTypeArgInference((MethodInvocationTree) invocation)
               .executableType;
@@ -871,20 +869,33 @@ public class InferenceFactory {
     if (properTypes.isEmpty()) {
       return null;
     }
-    TypeMirror tiTypeMirror = null;
-    AnnotatedTypeMirror ti = null;
-    for (ProperType liProperType : properTypes) {
-      AnnotatedTypeMirror li = liProperType.getAnnotatedType();
-      TypeMirror liTypeMirror = liProperType.getJavaType();
-      if (ti == null) {
-        ti = li;
-        tiTypeMirror = liTypeMirror;
+
+    TypeMirror lubTM = null;
+    AnnotatedTypeMirror lubATM = null;
+    boolean ignoreAnnotations = false;
+    for (ProperType properType : properTypes) {
+      AnnotatedTypeMirror atm = properType.getAnnotatedType();
+      TypeMirror tm = properType.getJavaType();
+      if (lubATM == null) {
+        lubATM = atm;
+        lubTM = tm;
+        ignoreAnnotations = properType.ignoreAnnotations;
       } else {
-        tiTypeMirror = lub(context.env, tiTypeMirror, liTypeMirror);
-        ti = AnnotatedTypes.leastUpperBound(typeFactory, ti, li, tiTypeMirror);
+        lubTM = lub(context.env, lubTM, tm);
+        if (properType.ignoreAnnotations == ignoreAnnotations) {
+          lubATM = AnnotatedTypes.leastUpperBound(typeFactory, lubATM, atm, lubTM);
+        } else if (properType.ignoreAnnotations) {
+          lubATM =
+              AnnotatedTypes.asSuper(
+                  typeFactory, lubATM, AnnotatedTypeMirror.createType(lubTM, typeFactory, false));
+        } else {
+          lubATM =
+              AnnotatedTypes.asSuper(
+                  typeFactory, atm, AnnotatedTypeMirror.createType(lubTM, typeFactory, false));
+        }
       }
     }
-    return new ProperType(ti, tiTypeMirror, context);
+    return new ProperType(lubATM, lubTM, context, ignoreAnnotations);
   }
 
   /**
@@ -921,22 +932,29 @@ public class InferenceFactory {
     AnnotatedTypeMirror aAtm = a.getAnnotatedType();
     AnnotatedTypeMirror bAtm = b.getAnnotatedType();
     AnnotatedTypeMirror glbATM = AnnotatedTypes.annotatedGLB(typeFactory, aAtm, bAtm);
+    if (a.ignoreAnnotations != b.ignoreAnnotations) {
+      if (a.ignoreAnnotations) {
+        glbATM.replaceAnnotations(bAtm.getPrimaryAnnotations());
+      } else {
+        glbATM.replaceAnnotations(aAtm.getPrimaryAnnotations());
+      }
+    }
     if (context.types.isSameType(aJavaType, (Type) glb)) {
-      return a.create(glbATM, glb);
+      return a.create(glbATM, glb, false);
     }
 
     if (context.types.isSameType(bJavaType, (Type) glb)) {
-      return b.create(glbATM, glb);
+      return b.create(glbATM, glb, false);
     }
 
     if (a.isInferenceType()) {
-      return a.create(glbATM, glb);
+      return a.create(glbATM, glb, false);
     } else if (b.isInferenceType()) {
-      return b.create(glbATM, glb);
+      return b.create(glbATM, glb, false);
     }
 
     assert a.isProper() && b.isProper();
-    return new ProperType(glbATM, glb, context);
+    return new ProperType(glbATM, glb, context, a.ignoreAnnotations && b.ignoreAnnotations);
   }
 
   /**
@@ -945,8 +963,7 @@ public class InferenceFactory {
    * @return the proper type for RuntimeException
    */
   public ProperType getRuntimeException() {
-    AnnotatedTypeMirror runtimeEx =
-        AnnotatedTypeMirror.createType(context.runtimeEx, typeFactory, false);
+    AnnotatedTypeMirror runtimeEx = typeFactory.getAnnotatedType(RuntimeException.class);
     runtimeEx.addMissingAnnotations(typeFactory.getQualifierHierarchy().getTopAnnotations());
     return new ProperType(runtimeEx, context.runtimeEx, context);
   }
@@ -990,7 +1007,7 @@ public class InferenceFactory {
     }
     List<? extends AnnotatedTypeMirror> thrownTypes;
     List<? extends TypeMirror> thrownTypeMirrors;
-    if (expression.getKind() == Tree.Kind.LAMBDA_EXPRESSION) {
+    if (expression instanceof LambdaExpressionTree) {
       thrownTypeMirrors =
           CheckedExceptionsUtil.thrownCheckedExceptions((LambdaExpressionTree) expression, context);
       thrownTypes =
@@ -1032,7 +1049,10 @@ public class InferenceFactory {
         for (UseOfVariable ei : es) {
           constraintSet.add(
               new Typing(
-                  new ProperType(iter2.next(), xi, context), ei, TypeConstraint.Kind.SUBTYPE));
+                  "Exception constraint for %s" + expression,
+                  new ProperType(iter2.next(), xi, context),
+                  ei,
+                  TypeConstraint.Kind.SUBTYPE));
           ei.setHasThrowsBound(true);
         }
       }
@@ -1102,7 +1122,7 @@ public class InferenceFactory {
     }
     context.typeFactory.capturedTypeVarSubstitutor.substitute(
         typeVariable, Collections.singletonMap(typeVariable.getUnderlyingType(), typeVariable));
-    return upperBound.create(typeVariable, freshTypeVariable);
+    return upperBound.create(typeVariable, freshTypeVariable, false);
   }
 
   /**
