@@ -12,15 +12,11 @@ import org.junit.runners.Parameterized.Parameters;
  */
 public class AccumulationTest extends CheckerFrameworkPerDirectoryTest {
 
-  /** @param testFiles the files containing test code, which will be type-checked */
+  /**
+   * @param testFiles the files containing test code, which will be type-checked
+   */
   public AccumulationTest(List<File> testFiles) {
-    super(
-        testFiles,
-        TestAccumulationChecker.class,
-        "accumulation",
-        "-Anomsgtext",
-        "-encoding",
-        "UTF-8");
+    super(testFiles, TestAccumulationChecker.class, "accumulation", "-encoding", "UTF-8");
   }
 
   @Parameters
