@@ -9,9 +9,11 @@ import org.junit.runners.Parameterized.Parameters;
 /** */
 public class LubGlbTest extends CheckerFrameworkPerDirectoryTest {
 
-  /** @param testFiles the files containing test code, which will be type-checked */
+  /**
+   * @param testFiles the files containing test code, which will be type-checked
+   */
   public LubGlbTest(List<File> testFiles) {
-    super(testFiles, LubGlbChecker.class, "lubglb", "-Anomsgtext");
+    super(testFiles, LubGlbChecker.class, "lubglb");
   }
 
   @Parameters
