@@ -9,13 +9,14 @@ import org.junit.runners.Parameterized.Parameters;
 /** Tests the constant value propagation type system without overflow. */
 public class ValueIgnoreRangeOverflowTest extends CheckerFrameworkPerDirectoryTest {
 
-  /** @param testFiles the files containing test code, which will be type-checked */
+  /**
+   * @param testFiles the files containing test code, which will be type-checked
+   */
   public ValueIgnoreRangeOverflowTest(List<File> testFiles) {
     super(
         testFiles,
         org.checkerframework.common.value.ValueChecker.class,
         "value",
-        "-Anomsgtext",
         "-A" + ValueChecker.REPORT_EVAL_WARNS,
         "-A" + ValueChecker.IGNORE_RANGE_OVERFLOW);
   }
