@@ -1,5 +1,6 @@
 package org.checkerframework.javacutil;
 
+import com.sun.tools.javac.code.BoundKind;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Symtab;
 import com.sun.tools.javac.code.Type;
@@ -9,21 +10,26 @@ import com.sun.tools.javac.code.TypeTag;
 import com.sun.tools.javac.model.JavacTypes;
 import com.sun.tools.javac.processing.JavacProcessingEnvironment;
 import com.sun.tools.javac.util.Context;
+import com.sun.tools.javac.util.Names;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Locale;
 import java.util.Set;
 import java.util.StringJoiner;
 import javax.annotation.processing.ProcessingEnvironment;
 import javax.lang.model.element.Element;
+import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.Name;
 import javax.lang.model.element.NestingKind;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.TypeParameterElement;
 import javax.lang.model.type.ArrayType;
 import javax.lang.model.type.DeclaredType;
+import javax.lang.model.type.ExecutableType;
+import javax.lang.model.type.IntersectionType;
 import javax.lang.model.type.PrimitiveType;
 import javax.lang.model.type.TypeKind;
 import javax.lang.model.type.TypeMirror;
@@ -32,6 +38,7 @@ import javax.lang.model.type.UnionType;
 import javax.lang.model.type.WildcardType;
 import javax.lang.model.util.Elements;
 import javax.lang.model.util.Types;
+import org.checkerframework.checker.interning.qual.EqualsMethod;
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.signature.qual.BinaryName;
 import org.checkerframework.checker.signature.qual.CanonicalNameOrEmpty;
@@ -52,21 +59,21 @@ public final class TypesUtils {
     throw new AssertionError("Class TypesUtils cannot be instantiated.");
   }
 
-  /// Creating types
+  // Creating types
 
   /**
    * Returns the {@link TypeMirror} for a given {@link Class}.
    *
    * @param clazz a class
    * @param types the type utilities
-   * @param elements the element utiliites
+   * @param elements the element utilities
    * @return the TypeMirror for {@code clazz}
    */
   public static TypeMirror typeFromClass(Class<?> clazz, Types types, Elements elements) {
     if (clazz == void.class) {
       return types.getNoType(TypeKind.VOID);
     } else if (clazz.isPrimitive()) {
-      String primitiveName = clazz.getName().toUpperCase();
+      String primitiveName = clazz.getName().toUpperCase(Locale.getDefault());
       TypeKind primitiveKind = TypeKind.valueOf(primitiveName);
       return types.getPrimitiveType(primitiveKind);
     } else if (clazz.isArray()) {
@@ -77,7 +84,7 @@ public final class TypesUtils {
       assert name != null : "@AssumeAssertion(nullness): assumption";
       TypeElement element = elements.getTypeElement(name);
       if (element == null) {
-        throw new BugInCF("Unrecognized class: " + clazz);
+        throw new BugInCF("No element for: " + clazz);
       }
       return element.asType();
     }
@@ -95,7 +102,7 @@ public final class TypesUtils {
     return t.getArrayType(componentType);
   }
 
-  /// Creating a Class<?>
+  // Creating a Class<?>
 
   /**
    * Returns the {@link Class} for a given {@link TypeMirror}. Returns {@code Object.class} if it
@@ -141,7 +148,7 @@ public final class TypesUtils {
 
         try {
           return Class.forName(typeString);
-        } catch (ClassNotFoundException | UnsupportedClassVersionError e) {
+        } catch (ClassNotFoundException | NoClassDefFoundError | UnsupportedClassVersionError e) {
           return Object.class;
         }
 
@@ -150,7 +157,7 @@ public final class TypesUtils {
     }
   }
 
-  /// Getters
+  // Getters
 
   /**
    * Gets the fully qualified name for a provided type. It returns an empty name if type is an
@@ -167,10 +174,10 @@ public final class TypesUtils {
   }
 
   /**
-   * Returns the simple type name, without annotations.
+   * Returns the simple type name, without annotations but including array brackets.
    *
    * @param type a type
-   * @return the simple type name, without annotations
+   * @return the simple type name
    */
   public static String simpleTypeName(TypeMirror type) {
     switch (type.getKind()) {
@@ -180,6 +187,12 @@ public final class TypesUtils {
         return ((TypeVariable) type).asElement().getSimpleName().toString();
       case DECLARED:
         return ((DeclaredType) type).asElement().getSimpleName().toString();
+      case INTERSECTION:
+        StringJoiner sjI = new StringJoiner(" & ");
+        for (TypeMirror bound : ((IntersectionType) type).getBounds()) {
+          sjI.add(simpleTypeName(bound));
+        }
+        return sjI.toString();
       case NULL:
         return "<nulltype>";
       case VOID:
@@ -197,6 +210,8 @@ public final class TypesUtils {
           sj.add(simpleTypeName(alternative));
         }
         return sj.toString();
+      case PACKAGE:
+        return "PACKAGE:" + type;
       default:
         if (type.getKind().isPrimitive()) {
           return TypeAnnotationUtils.unannotatedType(type).toString();
@@ -208,10 +223,10 @@ public final class TypesUtils {
   }
 
   /**
-   * Returns the binary name.
+   * Returns the binary name of a type.
    *
    * @param type a type
-   * @return the binary name
+   * @return its binary name
    */
   public static @BinaryName String binaryName(TypeMirror type) {
     if (type.getKind() != TypeKind.DECLARED) {
@@ -253,7 +268,7 @@ public final class TypesUtils {
     return result;
   }
 
-  /// Equality
+  // Equality
 
   /**
    * Returns true iff the arguments are both the same declared types.
@@ -269,7 +284,7 @@ public final class TypesUtils {
     if (t1.tsym.name != t2.tsym.name) {
       return false;
     }
-    return t1.toString().equals(t1.toString());
+    return t1.toString().equals(t2.toString());
   }
 
   /**
@@ -287,7 +302,7 @@ public final class TypesUtils {
     return (left.getKind() == right.getKind());
   }
 
-  /// Predicates
+  // Predicates
 
   /**
    * Checks if the type represents a java.lang.Object declared type.
@@ -320,42 +335,74 @@ public final class TypesUtils {
   }
 
   /**
-   * Checks if the type represents a boolean type, that is either boolean (primitive type) or
-   * java.lang.Boolean.
+   * Returns true if the type is either {@code boolean} (primitive type) or {@code
+   * java.lang.Boolean}.
    *
    * @param type the type to test
    * @return true iff type represents a boolean type
    */
   public static boolean isBooleanType(TypeMirror type) {
-    return isDeclaredOfName(type, "java.lang.Boolean") || type.getKind() == TypeKind.BOOLEAN;
+    return type.getKind() == TypeKind.BOOLEAN || isDeclaredOfName(type, "java.lang.Boolean");
+  }
+
+  /**
+   * Returns true if the type is {@code char} or {@code Character}.
+   *
+   * @param type a type
+   * @return true if the type is {@code char} or {@code Character}
+   */
+  public static boolean isCharOrCharacter(TypeMirror type) {
+    return type.getKind() == TypeKind.CHAR
+        || TypesUtils.isDeclaredOfName(type, "java.lang.Character");
   }
 
   /**
    * Check if the type represents a declared type of the given qualified name.
    *
    * @param type the type
-   * @return type iff type represents a declared type of the qualified name
+   * @param qualifiedName the name to check {@code type} against
+   * @return true iff type represents a declared type of the qualified name
    */
   public static boolean isDeclaredOfName(TypeMirror type, CharSequence qualifiedName) {
     return type.getKind() == TypeKind.DECLARED
         && getQualifiedName((DeclaredType) type).contentEquals(qualifiedName);
   }
 
+  /**
+   * Check if the type represents a declared type whose fully-qualified name is any of the given
+   * names.
+   *
+   * @param type the type
+   * @param qualifiedNames fully-qualified type names to check for
+   * @return true iff type represents a declared type whose fully-qualified name is one of the given
+   *     names
+   */
+  public static boolean isDeclaredOfName(TypeMirror type, Collection<String> qualifiedNames) {
+    return type.getKind() == TypeKind.DECLARED
+        && qualifiedNames.contains(getQualifiedName((DeclaredType) type));
+  }
+
+  /** The fully-qualified names of the boxed types. */
+  private static Set<String> fqBoxedTypes =
+      new HashSet<>(
+          Arrays.asList(
+              "java.lang.Boolean",
+              "java.lang.Byte",
+              "java.lang.Character",
+              "java.lang.Short",
+              "java.lang.Integer",
+              "java.lang.Long",
+              "java.lang.Double",
+              "java.lang.Float"));
+
+  /**
+   * Check if the {@code type} represents a boxed primitive type.
+   *
+   * @param type the type to check
+   * @return true iff type represents a boxed primitive type
+   */
   public static boolean isBoxedPrimitive(TypeMirror type) {
-    if (type.getKind() != TypeKind.DECLARED) {
-      return false;
-    }
-
-    String qualifiedName = getQualifiedName((DeclaredType) type).toString();
-
-    return (qualifiedName.equals("java.lang.Boolean")
-        || qualifiedName.equals("java.lang.Byte")
-        || qualifiedName.equals("java.lang.Character")
-        || qualifiedName.equals("java.lang.Short")
-        || qualifiedName.equals("java.lang.Integer")
-        || qualifiedName.equals("java.lang.Long")
-        || qualifiedName.equals("java.lang.Double")
-        || qualifiedName.equals("java.lang.Float"));
+    return isDeclaredOfName(type, fqBoxedTypes);
   }
 
   /**
@@ -402,6 +449,7 @@ public final class TypesUtils {
   /**
    * Returns true iff the argument is a primitive type.
    *
+   * @param type a type
    * @return whether the argument is a primitive type
    */
   public static boolean isPrimitive(TypeMirror type) {
@@ -494,16 +542,7 @@ public final class TypesUtils {
    * @return whether the argument is an integral primitive type
    */
   public static boolean isIntegralPrimitive(TypeMirror type) {
-    switch (type.getKind()) {
-      case BYTE:
-      case CHAR:
-      case INT:
-      case LONG:
-      case SHORT:
-        return true;
-      default:
-        return false;
-    }
+    return TypeKindUtils.isIntegral(type.getKind());
   }
 
   /**
@@ -517,6 +556,28 @@ public final class TypesUtils {
     return kind != null && TypeKindUtils.isIntegral(kind);
   }
 
+  /**
+   * Returns true iff the argument is an integral primitive type.
+   *
+   * @param type a type
+   * @return whether the argument is an integral primitive type
+   */
+  public static boolean isIntegralNumericPrimitive(TypeMirror type) {
+    return TypeKindUtils.isIntegralNumeric(type.getKind());
+  }
+
+  /**
+   * Return true if the argument TypeMirror is a (possibly boxed) integral type, excluding char and
+   * Character which do not print as numbers.
+   *
+   * @param type the type to inspect
+   * @return true if type is an integral numeric type
+   */
+  public static boolean isIntegralNumericOrBoxed(TypeMirror type) {
+    TypeKind kind = TypeKindUtils.primitiveOrBoxedToTypeKind(type);
+    return kind != null && TypeKindUtils.isIntegralNumeric(kind);
+  }
+
   /**
    * Returns true if declaredType is a Class that is used to box primitive type (e.g.
    * declaredType=java.lang.Double and primitiveType=22.5d )
@@ -530,7 +591,7 @@ public final class TypesUtils {
       return false;
     }
 
-    final String qualifiedName = getQualifiedName((DeclaredType) declaredType).toString();
+    String qualifiedName = getQualifiedName((DeclaredType) declaredType).toString();
     switch (primitiveType.getKind()) {
       case BOOLEAN:
         return qualifiedName.equals("java.lang.Boolean");
@@ -606,6 +667,44 @@ public final class TypesUtils {
     return (type instanceof Type.ClassType);
   }
 
+  /**
+   * Returns whether or not {@code type} is a functional interface type (as defined in JLS 9.8).
+   *
+   * @param type possible functional interface type
+   * @param env the processing environment
+   * @return whether or not {@code type} is a functional interface type (as defined in JLS 9.8)
+   */
+  public static boolean isFunctionalInterface(TypeMirror type, ProcessingEnvironment env) {
+    Context ctx = ((JavacProcessingEnvironment) env).getContext();
+    com.sun.tools.javac.code.Types javacTypes = com.sun.tools.javac.code.Types.instance(ctx);
+    return javacTypes.isFunctionalInterface((Type) type);
+  }
+
+  /**
+   * Returns true if the given type is a compound type.
+   *
+   * @param type a type
+   * @return true if the given type is a compound type
+   */
+  public static boolean isCompoundType(TypeMirror type) {
+    switch (type.getKind()) {
+      case ARRAY:
+      case EXECUTABLE:
+      case INTERSECTION:
+      case UNION:
+      case TYPEVAR:
+      case WILDCARD:
+        return true;
+
+      case DECLARED:
+        DeclaredType declaredType = (DeclaredType) type;
+        return !declaredType.getTypeArguments().isEmpty();
+
+      default:
+        return false;
+    }
+  }
+
   /**
    * Returns true if {@code type} has an enclosing type.
    *
@@ -617,20 +716,7 @@ public final class TypesUtils {
     return e.getKind() != TypeKind.NONE;
   }
 
-  /**
-   * Returns whether or not {@code type} is a functional interface type (as defined in JLS 9.8).
-   *
-   * @param type possible functional interface type
-   * @param env ProcessingEnvironment
-   * @return whether or not {@code type} is a functional interface type (as defined in JLS 9.8)
-   */
-  public static boolean isFunctionalInterface(TypeMirror type, ProcessingEnvironment env) {
-    Context ctx = ((JavacProcessingEnvironment) env).getContext();
-    com.sun.tools.javac.code.Types javacTypes = com.sun.tools.javac.code.Types.instance(ctx);
-    return javacTypes.isFunctionalInterface((Type) type);
-  }
-
-  /// Type variables and wildcards
+  // Type variables and wildcards
 
   /**
    * If the argument is a bounded TypeVariable or WildcardType, return its non-variable,
@@ -664,13 +750,24 @@ public final class TypesUtils {
   }
 
   /**
-   * Get the type parameter for this wildcard from the underlying type's bound field This field is
+   * Get the type parameter for this wildcard from the underlying type's bound field. This field is
    * sometimes null, in that case this method will return null.
    *
+   * @param wildcard wildcard type
    * @return the TypeParameterElement the wildcard is an argument to, {@code null} otherwise
    */
-  public static @Nullable TypeParameterElement wildcardToTypeParam(
-      final Type.WildcardType wildcard) {
+  public static @Nullable TypeParameterElement wildcardToTypeParam(WildcardType wildcard) {
+    return wildcardToTypeParam((Type.WildcardType) wildcard);
+  }
+
+  /**
+   * Get the type parameter for this wildcard from the underlying type's bound field. This field is
+   * sometimes null, in that case this method will return null.
+   *
+   * @param wildcard wildcard type
+   * @return the TypeParameterElement the wildcard is an argument to, {@code null} otherwise
+   */
+  public static @Nullable TypeParameterElement wildcardToTypeParam(Type.WildcardType wildcard) {
 
     final Element typeParamElement;
     if (wildcard.bound != null) {
@@ -716,6 +813,27 @@ public final class TypesUtils {
     return (DeclaredType) syms.objectType;
   }
 
+  /**
+   * Returns the lower bound of {@code typeVariable}. If it does not have a lower bound, returns the
+   * null type.
+   *
+   * @param typeVariable a type variable
+   * @param env the proceProcessingEnvironment
+   * @return the lower bound of {@code typeVariable} or the null type
+   */
+  public static TypeMirror getTypeVariableLowerBound(
+      TypeVariable typeVariable, ProcessingEnvironment env) {
+    TypeMirror lb = typeVariable.getLowerBound();
+    if (lb != null) {
+      return lb;
+    }
+
+    // Use bottom type to ensure there is a lower bound.
+    Context context = ((JavacProcessingEnvironment) env).getContext();
+    Symtab syms = Symtab.instance(context);
+    return syms.botType;
+  }
+
   /**
    * Version of com.sun.tools.javac.code.Types.wildLowerBound(Type) that works with both jdk8
    * (called upperBound there) and jdk8u.
@@ -740,7 +858,7 @@ public final class TypesUtils {
    * @return a type that is not a wildcard or typevar, or {@code null} if this type is an unbounded
    *     wildcard
    */
-  public static @Nullable TypeMirror findConcreteUpperBound(final TypeMirror boundedType) {
+  public static @Nullable TypeMirror findConcreteUpperBound(TypeMirror boundedType) {
     TypeMirror effectiveUpper = boundedType;
     outerLoop:
     while (true) {
@@ -763,8 +881,67 @@ public final class TypesUtils {
     return effectiveUpper;
   }
 
+  // For Wildcards, isSuperBound() and isExtendsBound() will return true if isUnbound() does.
+  // But don't use isUnbound(), because as of Java 18, it returns true for "? extends Object".
+
   /**
-   * Returns true if the erased type of subtype is a subtype of the erased type of supertype.
+   * Returns true if {@code type} is an unbounded wildcard.
+   *
+   * @param type the type to check
+   * @return true if the given type is an unbounded wildcard
+   */
+  public static boolean hasNoExplicitBound(TypeMirror type) {
+    return type.getKind() == TypeKind.WILDCARD
+        && ((Type.WildcardType) type).kind == BoundKind.UNBOUND;
+  }
+
+  /**
+   * Returns true if {@code type} is a wildcard with an explicit super bound.
+   *
+   * @param type the {@code type} to test
+   * @return true if {@code type} is explicitly super bounded
+   */
+  public static boolean hasExplicitSuperBound(TypeMirror type) {
+    return type.getKind() == TypeKind.WILDCARD
+        && !hasNoExplicitBound(type)
+        && ((Type.WildcardType) type).isSuperBound();
+  }
+
+  /**
+   * Returns true if {@code type} is a wildcard with an explicit extends bound.
+   *
+   * @param type the type to test
+   * @return true if {@code type} is a wildcard with an explicit extends bound
+   */
+  public static boolean hasExplicitExtendsBound(TypeMirror type) {
+    return type.getKind() == TypeKind.WILDCARD
+        && !hasNoExplicitBound(type)
+        && ((Type.WildcardType) type).isExtendsBound();
+  }
+
+  /**
+   * Returns true if this type is super bounded or unbounded.
+   *
+   * @param wildcardType the wildcard type to test
+   * @return true if this type is super bounded or unbounded
+   */
+  public static boolean isUnboundedOrSuperBounded(WildcardType wildcardType) {
+    return ((Type.WildcardType) wildcardType).isSuperBound();
+  }
+
+  /**
+   * Returns true if this type is extends bounded or unbounded.
+   *
+   * @param wildcardType the wildcard type to test
+   * @return true if this type is extends bounded or unbounded
+   */
+  public static boolean isUnboundedOrExtendsBounded(WildcardType wildcardType) {
+    return ((Type.WildcardType) wildcardType).isExtendsBound();
+  }
+
+  /**
+   * Returns true if the erased type of {@code subtype} is a subtype of the erased type of {@code
+   * supertype}.
    *
    * @param subtype possible subtype
    * @param supertype possible supertype
@@ -775,21 +952,6 @@ public final class TypesUtils {
     return types.isSubtype(types.erasure(subtype), types.erasure(supertype));
   }
 
-  /**
-   * Returns true if {@code type} is a type variable created during capture conversion.
-   *
-   * @param type a type mirror
-   * @return true if {@code type} is a type variable created during capture conversion
-   * @deprecated use {@link #isCapturedTypeVariable(TypeMirror)} instead
-   */
-  @Deprecated // 2021-07-06
-  public static boolean isCaptured(TypeMirror type) {
-    if (type.getKind() != TypeKind.TYPEVAR) {
-      return false;
-    }
-    return ((Type.TypeVar) TypeAnnotationUtils.unannotatedType(type)).isCaptured();
-  }
-
   /**
    * Returns true if {@code type} is a type variable created during capture conversion.
    *
@@ -818,7 +980,7 @@ public final class TypesUtils {
     return null;
   }
 
-  /// Least upper bound and greatest lower bound
+  // Least upper bound and greatest lower bound
 
   /**
    * Returns the least upper bound of two {@link TypeMirror}s, ignoring any annotations on the
@@ -869,8 +1031,8 @@ public final class TypesUtils {
     }
     // Special case for primitives.
     if (isPrimitive(t1) || isPrimitive(t2)) {
-      // NOTE: we need to know which type is primitive because e.g. int and Integer are assignable
-      // to each other.
+      // NOTE: we need to know which type is primitive because e.g. int and Integer
+      // are assignable to each other.
       if (isPrimitive(t1) && types.isAssignable(t1, t2)) {
         return t2;
       } else if (isPrimitive(t2) && types.isAssignable(t2, t1)) {
@@ -880,7 +1042,18 @@ public final class TypesUtils {
         return elements.getTypeElement("java.lang.Object").asType();
       }
     }
-    return types.lub(t1, t2);
+    try {
+      return types.lub(t1, t2);
+    } catch (Exception e) {
+      // typetools issue #3025: In at least Java 8/9, types.lub throws an NPE
+      // on capture/wildcard combinations, see test case
+      // checker/tests/nullness/generics/Issue3025.java.
+      // Using j.l.Object is too coarse in case the type actually matters.
+      // This problem doesn't exist anymore in Java 11+, so let's
+      // see whether this is a problem for anyone in practice.
+      Elements elements = processingEnv.getElementUtils();
+      return elements.getTypeElement("java.lang.Object").asType();
+    }
   }
 
   /**
@@ -971,11 +1144,12 @@ public final class TypesUtils {
    */
   private static com.sun.tools.javac.util.List<Type> typeMirrorListToTypeList(
       List<TypeMirror> typeMirrors) {
+    @SuppressWarnings("nullness:type.arguments.not.inferred") // Poly + inference bug.
     List<Type> typeList = CollectionsPlume.mapList(Type.class::cast, typeMirrors);
     return com.sun.tools.javac.util.List.from(typeList);
   }
 
-  /// Substitutions
+  // Substitutions
 
   /**
    * Returns the return type of a method, given the receiver of the method call.
@@ -1003,7 +1177,7 @@ public final class TypesUtils {
    * @return {@code type} as {@code superType} if {@code superType} is a super type of {@code type};
    *     otherwise, null
    */
-  public static TypeMirror asSuper(
+  public static @Nullable TypeMirror asSuper(
       TypeMirror type, TypeMirror superType, ProcessingEnvironment env) {
     Context ctx = ((JavacProcessingEnvironment) env).getContext();
     com.sun.tools.javac.code.Types javacTypes = com.sun.tools.javac.code.Types.instance(ctx);
@@ -1084,9 +1258,9 @@ public final class TypesUtils {
       List<? extends TypeMirror> typeVariables,
       List<? extends TypeMirror> typeArgs,
       ProcessingEnvironment env) {
-
+    @SuppressWarnings("nullness:type.arguments.not.inferred") // Poly + inference bug.
     List<Type> newP = CollectionsPlume.mapList(Type.class::cast, typeVariables);
-
+    @SuppressWarnings("nullness:type.arguments.not.inferred") // Poly + inference bug.
     List<Type> newT = CollectionsPlume.mapList(Type.class::cast, typeArgs);
 
     JavacProcessingEnvironment javacEnv = (JavacProcessingEnvironment) env;
@@ -1130,6 +1304,45 @@ public final class TypesUtils {
     return types.freshTypeVariables(com.sun.tools.javac.util.List.of((Type) typeMirror)).head;
   }
 
+  /**
+   * Creates a fresh type variable with bounds {@code upper} and {@code lower}.
+   *
+   * @param upper the upper bound to use, or if {@code null}, then {@code Object} is the upper bound
+   * @param lower the lower bound to use, or if {@code null}, then {@code NullType} is the lower
+   *     bound
+   * @param env processing environment
+   * @return a fresh type variable
+   */
+  public static TypeMirror freshTypeVariable(
+      @Nullable TypeMirror upper, @Nullable TypeMirror lower, ProcessingEnvironment env) {
+    JavacProcessingEnvironment javacEnv = (JavacProcessingEnvironment) env;
+    Names names = Names.instance(javacEnv.getContext());
+    Symtab syms = Symtab.instance(javacEnv.getContext());
+    com.sun.tools.javac.util.Name capturedName = names.fromString("<captured wildcard>");
+    WildcardType wildcardType = null;
+    if (lower != null
+        && (lower.getKind() == TypeKind.ARRAY
+            || lower.getKind() == TypeKind.DECLARED
+            || lower.getKind() == TypeKind.TYPEVAR)) {
+      wildcardType = env.getTypeUtils().getWildcardType(null, lower);
+    } else if (upper != null
+        && (upper.getKind() == TypeKind.ARRAY
+            || upper.getKind() == TypeKind.DECLARED
+            || upper.getKind() == TypeKind.TYPEVAR)) {
+      wildcardType = env.getTypeUtils().getWildcardType(upper, null);
+    } else {
+      wildcardType = env.getTypeUtils().getWildcardType(null, null);
+    }
+    if (lower == null) {
+      lower = syms.botType;
+    }
+    if (upper == null) {
+      upper = syms.objectType;
+    }
+    return new CapturedType(
+        capturedName, syms.noSymbol, (Type) upper, (Type) lower, (Type.WildcardType) wildcardType);
+  }
+
   /**
    * Returns the list of type variables such that a type variable in the list only references type
    * variables at a lower index than itself.
@@ -1141,7 +1354,7 @@ public final class TypesUtils {
    */
   public static List<TypeVariable> order(Collection<TypeVariable> collection, Types types) {
     List<TypeVariable> list = new ArrayList<>(collection);
-    List<TypeVariable> ordered = new ArrayList<>();
+    List<TypeVariable> ordered = new ArrayList<>(list.size());
     while (!list.isEmpty()) {
       TypeVariable free = doesNotContainOthers(list, types);
       list.remove(free);
@@ -1176,4 +1389,167 @@ public final class TypesUtils {
     }
     throw new BugInCF("Not found: %s", StringsPlume.join(",", collection));
   }
+
+  /**
+   * This method returns the single abstract method declared by {@code functionalInterfaceType}.
+   * (The type of this method is referred to as the function type.)
+   *
+   * @param functionalInterfaceType a functional interface type
+   * @param env the processing environment
+   * @return the single abstract method declared by the type
+   */
+  public static ExecutableElement findFunction(
+      TypeMirror functionalInterfaceType, ProcessingEnvironment env) {
+    Context ctx = ((JavacProcessingEnvironment) env).getContext();
+    com.sun.tools.javac.code.Types javacTypes = com.sun.tools.javac.code.Types.instance(ctx);
+    return (ExecutableElement)
+        javacTypes.findDescriptorSymbol(((Type) functionalInterfaceType).asElement());
+  }
+
+  /**
+   * This method returns the type of the single abstract method declared by {@code
+   * functionalInterfaceType}.
+   *
+   * @param functionalInterfaceType functional interface
+   * @param env ProcessingEnvironment
+   * @return the single abstract method declared by the type of the tree
+   */
+  public static ExecutableType findFunctionType(
+      TypeMirror functionalInterfaceType, ProcessingEnvironment env) {
+    return (ExecutableType) findFunction(functionalInterfaceType, env).asType();
+  }
+
+  /**
+   * Return whether or not {@code type} is raw.
+   *
+   * @param type the type to check
+   * @return whether or not {@code type} is raw
+   */
+  public static boolean isRaw(TypeMirror type) {
+    if (type.getKind() != TypeKind.DECLARED) {
+      return false;
+    }
+    TypeElement typeelem = (TypeElement) ((DeclaredType) type).asElement();
+    DeclaredType declType = (DeclaredType) typeelem.asType();
+    return !declType.getTypeArguments().isEmpty()
+        && ((DeclaredType) type).getTypeArguments().isEmpty();
+  }
+
+  /**
+   * Returns the most specific supertype of {@code type} that is an array, or null if {@code type}
+   * is not a subtype of an array.
+   *
+   * @param type a type
+   * @param types TypesUtils
+   * @return the most specific supertype of {@code type} that is an array, or null if {@code type}
+   *     is not a subtype of an array
+   */
+  public static @Nullable TypeMirror getMostSpecificArrayType(TypeMirror type, Types types) {
+    if (type.getKind() == TypeKind.ARRAY) {
+      return type;
+    } else {
+      for (TypeMirror superType : types.directSupertypes(type)) {
+        TypeMirror arrayType = getMostSpecificArrayType(superType, types);
+        if (arrayType != null) {
+          // Only one of the types can be an array type, so return the first one found.
+          return arrayType;
+        }
+      }
+      return null;
+    }
+  }
+
+  /**
+   * Returns true if {@code type} is a parameterized type. A declared type is parameterized if it
+   * has parameters. An array type is parameterized if the inner-most component type has parameters.
+   *
+   * @param type type to check
+   * @return true if {@code type} is a parameterized declared type or array type
+   */
+  public static boolean isParameterizedType(TypeMirror type) {
+    return ((Type) type).isParameterized();
+  }
+
+  /**
+   * Return true if {@code typeMirror} is a declared type that has at least one wildcard as a type
+   * argument.
+   *
+   * @param typeMirror type to check
+   * @return true if {@code typeMirror} is a declared type that has at least one wildcard as a type
+   *     argument
+   */
+  public static boolean isWildcardParameterized(TypeMirror typeMirror) {
+    if (isParameterizedType(typeMirror) && typeMirror.getKind() == TypeKind.DECLARED) {
+      for (TypeMirror t : ((DeclaredType) typeMirror).getTypeArguments()) {
+        if (t.getKind() == TypeKind.WILDCARD) {
+          return true;
+        }
+      }
+    }
+    return false;
+  }
+
+  /**
+   * Creates a wildcard with the given bounds. If {@code lowerBound} is non-null, the {@code
+   * upperBound} must be {@code null} or {@code Object}. If {@code upperBound} is non-null and not
+   * {@code Object}, then {@code lowerBound} must be {@code null};
+   *
+   * @param lowerBound the lower bound for the wildcard
+   * @param upperBound the upper bound for the wildcard
+   * @param types TypesUtils
+   * @return a wildcard with the given bounds
+   */
+  public static TypeMirror createWildcard(
+      TypeMirror lowerBound, TypeMirror upperBound, Types types) {
+    TypeMirror nonObjectUpperBound = upperBound;
+    if (isObject(upperBound)) {
+      nonObjectUpperBound = null;
+    }
+
+    assert lowerBound == null || nonObjectUpperBound == null;
+    WildcardType wildcardType = types.getWildcardType(nonObjectUpperBound, lowerBound);
+    return com.sun.tools.javac.util.List.of((Type) wildcardType).head;
+  }
+
+  /**
+   * Returns true if the type is byte, short, char, Byte, Short, or Character. All other narrowings
+   * require a cast. See JLS 5.1.3.
+   *
+   * @param type a type
+   * @param types the type utilities
+   * @return true if assignment to the type may be a narrowing
+   */
+  public static boolean canBeNarrowingPrimitiveConversion(TypeMirror type, Types types) {
+    // See CFGBuilder.CFGTranslationPhaseOne#conversionRequiresNarrowing()
+    TypeMirror unboxedType = isBoxedPrimitive(type) ? types.unboxedType(type) : type;
+    TypeKind unboxedKind = unboxedType.getKind();
+    return unboxedKind == TypeKind.BYTE
+        || unboxedKind == TypeKind.SHORT
+        || unboxedKind == TypeKind.CHAR;
+  }
+
+  /**
+   * Returns true if the two type variables are the same type variable. Meaning they have the same
+   * name and the same enclosing element. Unlike {@link Types#isSameType(TypeMirror, TypeMirror)},
+   * they do not have to be the same object.
+   *
+   * <p>This method is needed when a type has gone through type variable substitution, but only some
+   * of the type variables were substituted. Also, a new {@link TypeVariable} object is created as
+   * the type of a tree created by {@link org.checkerframework.javacutil.trees.TreeBuilder}.
+   *
+   * @param typeVariable1 a type variable
+   * @param typeVariable2 a type variable
+   * @return if the two type variables are the same type variable
+   */
+  @EqualsMethod
+  public static boolean areSame(TypeVariable typeVariable1, TypeVariable typeVariable2) {
+    if (typeVariable1 == typeVariable2) {
+      return true;
+    }
+    Name otherName = typeVariable2.asElement().getSimpleName();
+    Element otherEnclosingElement = typeVariable2.asElement().getEnclosingElement();
+
+    return typeVariable1.asElement().getSimpleName().contentEquals(otherName)
+        && otherEnclosingElement.equals(typeVariable1.asElement().getEnclosingElement());
+  }
 }
