@@ -22,6 +22,7 @@ import com.sun.tools.javac.tree.JCTree.JCExpression;
 import com.sun.tools.javac.tree.TreeInfo;
 import com.sun.tools.javac.tree.TreeMaker;
 import com.sun.tools.javac.util.Context;
+import com.sun.tools.javac.util.Name;
 import com.sun.tools.javac.util.Names;
 import java.util.List;
 import javax.annotation.processing.ProcessingEnvironment;
@@ -36,6 +37,7 @@ import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.ElementFilter;
 import javax.lang.model.util.Elements;
 import javax.lang.model.util.Types;
+import org.checkerframework.javacutil.BugInCF;
 import org.checkerframework.javacutil.TreeUtils;
 import org.checkerframework.javacutil.TypesUtils;
 import org.plumelib.util.CollectionsPlume;
@@ -45,14 +47,40 @@ import org.plumelib.util.CollectionsPlume;
  * TreeMaker.
  */
 public class TreeBuilder {
+
+  /** The javac {@link Elements} object. */
   protected final Elements elements;
+
+  /** The javac {@link javax.lang.model.util.Types} object. */
   protected final Types modelTypes;
+
+  /** The internal javac {@link com.sun.tools.javac.code.Types} object. */
   protected final com.sun.tools.javac.code.Types javacTypes;
+
+  /** For constructing trees */
   protected final TreeMaker maker;
+
+  /** The javac {@link Names} object. */
   protected final Names names;
+
+  /** The javac {@link Symtab} object. */
   protected final Symtab symtab;
+
+  /** The javac {@link ProcessingEnvironment} */
   protected final ProcessingEnvironment env;
 
+  /**
+   * {@link Name} object for "close", used when building a tree for a call to {@code close()}.
+   *
+   * @see #buildCloseMethodAccess(ExpressionTree)
+   */
+  private final Name closeName;
+
+  /**
+   * Creates a new TreeBuilder.
+   *
+   * @param env the javac {@link ProcessingEnvironment}
+   */
   public TreeBuilder(ProcessingEnvironment env) {
     this.env = env;
     Context context = ((JavacProcessingEnvironment) env).getContext();
@@ -62,6 +90,7 @@ public class TreeBuilder {
     maker = TreeMaker.instance(context);
     names = Names.instance(context);
     symtab = Symtab.instance(context);
+    closeName = names.fromString("close");
   }
 
   /**
@@ -102,10 +131,11 @@ public class TreeBuilder {
       // Remove captured type variable from a wildcard.
       if (elementType instanceof Type.CapturedType) {
         elementType = ((Type.CapturedType) elementType).wildcard;
+        TypeElement iteratorElt = (TypeElement) modelTypes.asElement(iteratorType);
+        assert iteratorElt != null
+            : "@AssumeAssertion(nullness): the iterator type always has an element";
 
-        iteratorType =
-            modelTypes.getDeclaredType(
-                (TypeElement) modelTypes.asElement(iteratorType), elementType);
+        iteratorType = modelTypes.getDeclaredType(iteratorElt, elementType);
       }
     }
 
@@ -118,13 +148,56 @@ public class TreeBuilder {
             com.sun.tools.javac.util.List.nil(),
             methodClass);
 
-    JCTree.JCFieldAccess iteratorAccess =
-        (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) iterableExpr, iteratorMethod);
+    JCTree.JCFieldAccess iteratorAccess = TreeUtils.Select(maker, iterableExpr, iteratorMethod);
     iteratorAccess.setType(updatedMethodType);
 
     return iteratorAccess;
   }
 
+  /**
+   * Build a {@link MemberSelectTree} for accessing the {@code close} method of an expression that
+   * implements {@link AutoCloseable}. This method is used when desugaring try-with-resources
+   * statements during CFG construction.
+   *
+   * @param autoCloseableExpr the expression
+   * @return the member select tree
+   */
+  public MemberSelectTree buildCloseMethodAccess(ExpressionTree autoCloseableExpr) {
+    DeclaredType exprType =
+        (DeclaredType) TypesUtils.upperBound(TreeUtils.typeOf(autoCloseableExpr));
+    assert exprType != null
+        : "expression must be of declared type AutoCloseable: " + autoCloseableExpr;
+
+    TypeElement exprElement = (TypeElement) exprType.asElement();
+
+    // Find the close() method
+    Symbol.MethodSymbol closeMethod = null;
+
+    // We could use elements.getAllMembers(exprElement) to find the close method, but in rare
+    // cases calling that method crashes with a Symbol$CompletionFailure exception.  See
+    // https://github.com/typetools/checker-framework/issues/6396.  The code below directly
+    // searches all supertypes for the method and avoids the crash.
+    for (Type s : javacTypes.closure(((Symbol) exprElement).type)) {
+      for (Symbol m : s.tsym.members().getSymbolsByName(closeName)) {
+        if (!(m instanceof Symbol.MethodSymbol)) {
+          continue;
+        }
+        Symbol.MethodSymbol msym = (Symbol.MethodSymbol) m;
+        if (!msym.isStatic() && msym.getParameters().isEmpty()) {
+          closeMethod = msym;
+          break;
+        }
+      }
+    }
+
+    assert closeMethod != null
+        : "@AssumeAssertion(nullness): no close method declared for expression type";
+
+    JCTree.JCFieldAccess closeAccess = TreeUtils.Select(maker, autoCloseableExpr, closeMethod);
+
+    return closeAccess;
+  }
+
   /**
    * Builds an AST Tree to access the hasNext() method of an iterator.
    *
@@ -143,13 +216,15 @@ public class TreeBuilder {
     for (ExecutableElement method : ElementFilter.methodsIn(elements.getAllMembers(exprElement))) {
       if (method.getParameters().isEmpty() && method.getSimpleName().contentEquals("hasNext")) {
         hasNextMethod = (Symbol.MethodSymbol) method;
+        break;
       }
     }
 
-    assert hasNextMethod != null : "no hasNext method declared for expression type";
+    if (hasNextMethod == null) {
+      throw new BugInCF("no hasNext method declared for " + exprElement);
+    }
 
-    JCTree.JCFieldAccess hasNextAccess =
-        (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) iteratorExpr, hasNextMethod);
+    JCTree.JCFieldAccess hasNextAccess = TreeUtils.Select(maker, iteratorExpr, hasNextMethod);
     hasNextAccess.setType(hasNextMethod.asType());
 
     return hasNextAccess;
@@ -198,8 +273,7 @@ public class TreeBuilder {
             com.sun.tools.javac.util.List.nil(),
             methodClass);
 
-    JCTree.JCFieldAccess nextAccess =
-        (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) iteratorExpr, nextMethod);
+    JCTree.JCFieldAccess nextAccess = TreeUtils.Select(maker, iteratorExpr, nextMethod);
     nextAccess.setType(updatedMethodType);
 
     return nextAccess;
@@ -212,8 +286,7 @@ public class TreeBuilder {
    * @return a MemberSelectTree to dereference the length of the array
    */
   public MemberSelectTree buildArrayLengthAccess(ExpressionTree expression) {
-
-    return (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) expression, symtab.lengthVar);
+    return TreeUtils.Select(maker, expression, symtab.lengthVar);
   }
 
   /**
@@ -388,8 +461,7 @@ public class TreeBuilder {
 
     Type.MethodType methodType = (Type.MethodType) valueOfMethod.asType();
 
-    JCTree.JCFieldAccess valueOfAccess =
-        (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) expr, valueOfMethod);
+    JCTree.JCFieldAccess valueOfAccess = TreeUtils.Select(maker, expr, valueOfMethod);
     valueOfAccess.setType(methodType);
 
     return valueOfAccess;
@@ -447,8 +519,7 @@ public class TreeBuilder {
 
     Type.MethodType methodType = (Type.MethodType) primValueMethod.asType();
 
-    JCTree.JCFieldAccess primValueAccess =
-        (JCTree.JCFieldAccess) maker.Select((JCTree.JCExpression) expr, primValueMethod);
+    JCTree.JCFieldAccess primValueAccess = TreeUtils.Select(maker, expr, primValueMethod);
     primValueAccess.setType(methodType);
 
     return primValueAccess;
@@ -620,7 +691,7 @@ public class TreeBuilder {
    * Builds an AST Tree to perform a binary operation.
    *
    * @param type result type of the operation
-   * @param op AST Tree operator
+   * @param op an AST Tree operator
    * @param left the left operand tree
    * @param right the right operand tree
    * @return a Tree representing "left &lt; right"
@@ -642,6 +713,7 @@ public class TreeBuilder {
    * @return a NewArrayTree to create a new array with initializers
    */
   public NewArrayTree buildNewArray(TypeMirror componentType, List<ExpressionTree> elems) {
+    @SuppressWarnings("nullness:type.arguments.not.inferred") // Poly + inference bug.
     List<JCExpression> exprs = CollectionsPlume.mapList(JCExpression.class::cast, elems);
 
     JCTree.JCNewArray newArray =
